@@ -208,6 +208,7 @@ class AuthStore:
         client_icon: Optional[str] = None,
         token_type: str = models.TOKEN_TYPE_NORMAL,
         access_token_expiration: timedelta = ACCESS_TOKEN_EXPIRATION,
+        credential: Optional[models.Credentials] = None,
     ) -> models.RefreshToken:
         """Create a new token for a user."""
         kwargs: Dict[str, Any] = {
@@ -215,6 +216,7 @@ class AuthStore:
             "client_id": client_id,
             "token_type": token_type,
             "access_token_expiration": access_token_expiration,
+            "credential": credential,
         }
         if client_name:
             kwargs["client_name"] = client_name
@@ -309,6 +311,7 @@ class AuthStore:
 
         users: Dict[str, models.User] = OrderedDict()
         groups: Dict[str, models.Group] = OrderedDict()
+        credentials: Dict[str, models.Credentials] = OrderedDict()
 
         # Soft-migrating data as we load. We are going to make sure we have a
         # read only group and an admin group. There are two states that we can
@@ -321,7 +324,7 @@ class AuthStore:
         group_without_policy = None
 
         # When creating objects we mention each attribute explicitly. This
-        # prevents crashing if user rolls back HA version after a new property
+        # prevents crashing if user rolls back OP version after a new property
         # was added.
 
         for group_dict in data.get("groups", []):
@@ -415,15 +418,15 @@ class AuthStore:
             )
 
         for cred_dict in data["credentials"]:
-            users[cred_dict["user_id"]].credentials.append(
-                models.Credentials(
-                    id=cred_dict["id"],
-                    is_new=False,
-                    auth_provider_type=cred_dict["auth_provider_type"],
-                    auth_provider_id=cred_dict["auth_provider_id"],
-                    data=cred_dict["data"],
-                )
+            credential = models.Credentials(
+                id=cred_dict["id"],
+                is_new=False,
+                auth_provider_type=cred_dict["auth_provider_type"],
+                auth_provider_id=cred_dict["auth_provider_id"],
+                data=cred_dict["data"],
             )
+            credentials[cred_dict["id"]] = credential
+            users[cred_dict["user_id"]].credentials.append(credential)
 
         for rt_dict in data["refresh_tokens"]:
             # Filter out the old keys that don't have jwt_key (pre-0.76)
@@ -469,6 +472,8 @@ class AuthStore:
                 jwt_key=rt_dict["jwt_key"],
                 last_used_at=last_used_at,
                 last_used_ip=rt_dict.get("last_used_ip"),
+                credential=credentials.get(rt_dict.get("credential_id")),
+                version=rt_dict.get("version"),
             )
             users[rt_dict["user_id"]].refresh_tokens[token.id] = token
 
@@ -542,6 +547,10 @@ class AuthStore:
                 if refresh_token.last_used_at
                 else None,
                 "last_used_ip": refresh_token.last_used_ip,
+                "credential_id": refresh_token.credential.id
+                if refresh_token.credential
+                else None,
+                "version": refresh_token.version,
             }
             for user in self._users.values()
             for refresh_token in user.refresh_tokens.values()
