@@ -1,11 +1,13 @@
 """Alexa entity adapters."""
-from typing import List
+import logging
+from typing import TYPE_CHECKING, List
 
 from openpeerpower.components import (
     alarm_control_panel,
     alert,
     automation,
     binary_sensor,
+    camera,
     cover,
     fan,
     group,
@@ -31,13 +33,17 @@ from openpeerpower.const import (
     CONF_NAME,
     TEMP_CELSIUS,
     TEMP_FAHRENHEIT,
+    __version__,
 )
-from openpeerpower.core import callback
+from openpeerpower.core import OpenPeerPower, State, callback
+from openpeerpower.helpers import network
 from openpeerpower.util.decorator import Registry
 
 from .capabilities import (
     Alexa,
     AlexaBrightnessController,
+    AlexaCameraStreamController,
+    AlexaCapability,
     AlexaChannelController,
     AlexaColorController,
     AlexaColorTemperatureController,
@@ -68,6 +74,11 @@ from .capabilities import (
 )
 from .const import CONF_DESCRIPTION, CONF_DISPLAY_CATEGORIES
 
+if TYPE_CHECKING:
+    from .config import AbstractConfig
+
+_LOGGER = logging.getLogger(__name__)
+
 ENTITY_ADAPTERS = Registry()
 
 TRANSLATION_TABLE = dict.fromkeys(map(ord, r"}{\/|\"()[]+~!><*%"), None)
@@ -85,9 +96,24 @@ class DisplayCategory:
     # to HDMI1. Applies to Scenes
     ACTIVITY_TRIGGER = "ACTIVITY_TRIGGER"
 
-    # Indicates media devices with video or photo capabilities.
+    # Indicates a device that emits pleasant odors and masks unpleasant odors in interior spaces.
+    AIR_FRESHENER = "AIR_FRESHENER"
+
+    # Indicates a device that improves the quality of air in interior spaces.
+    AIR_PURIFIER = "AIR_PURIFIER"
+
+    # Indicates a smart device in an automobile, such as a dash camera.
+    AUTO_ACCESSORY = "AUTO_ACCESSORY"
+
+    # Indicates a security device with video or photo functionality.
     CAMERA = "CAMERA"
 
+    # Indicates a religious holiday decoration that often contains lights.
+    CHRISTMAS_TREE = "CHRISTMAS_TREE"
+
+    # Indicates a device that makes coffee.
+    COFFEE_MAKER = "COFFEE_MAKER"
+
     # Indicates a non-mobile computer, such as a desktop computer.
     COMPUTER = "COMPUTER"
 
@@ -109,9 +135,16 @@ class DisplayCategory:
     # Indicates a game console, such as Microsoft Xbox or Nintendo Switch
     GAME_CONSOLE = "GAME_CONSOLE"
 
-    # Indicates a garage door. Garage doors must implement the ModeController interface to open and close the door.
+    # Indicates a garage door.
+    # Garage doors must implement the ModeController interface to open and close the door.
     GARAGE_DOOR = "GARAGE_DOOR"
 
+    # Indicates a wearable device that transmits audio directly into the ear.
+    HEADPHONES = "HEADPHONES"
+
+    # Indicates a smart-home hub.
+    HUB = "HUB"
+
     # Indicates a window covering on the inside of a structure.
     INTERIOR_BLIND = "INTERIOR_BLIND"
 
@@ -133,18 +166,24 @@ class DisplayCategory:
     # Indicates a network-connected music system.
     MUSIC_SYSTEM = "MUSIC_SYSTEM"
 
-    # An endpoint that cannot be described in on of the other categories.
-    OTHER = "OTHER"
-
     # Indicates a network router.
     NETWORK_HARDWARE = "NETWORK_HARDWARE"
 
+    # An endpoint that cannot be described in on of the other categories.
+    OTHER = "OTHER"
+
     # Indicates an oven cooking appliance.
     OVEN = "OVEN"
 
     # Indicates a non-mobile phone, such as landline or an IP phone.
     PHONE = "PHONE"
 
+    # Indicates a device that prints.
+    PRINTER = "PRINTER"
+
+    # Indicates a network router.
+    ROUTER = "ROUTER"
+
     # Describes a combination of devices set to a specific state, when the
     # order of the state change is not important. For example a bedtime scene
     # might include turning off lights and lowering the thermostat, but the
@@ -157,6 +196,13 @@ class DisplayCategory:
     # Indicates a security panel.
     SECURITY_PANEL = "SECURITY_PANEL"
 
+    # Indicates a security system.
+    SECURITY_SYSTEM = "SECURITY_SYSTEM"
+
+    # Indicates an electric cooking device that sits on a countertop, cooks at low temperatures,
+    # and is often shaped like a cooking pot.
+    SLOW_COOKER = "SLOW_COOKER"
+
     # Indicates an endpoint that locks.
     SMARTLOCK = "SMARTLOCK"
 
@@ -187,17 +233,25 @@ class DisplayCategory:
     # Indicates the endpoint is a television.
     TV = "TV"
 
+    # Indicates a vacuum cleaner.
+    VACUUM_CLEANER = "VACUUM_CLEANER"
+
     # Indicates a network-connected wearable device, such as an Apple Watch, Fitbit, or Samsung Gear.
     WEARABLE = "WEARABLE"
 
 
+def generate_alexa_id(entity_id: str) -> str:
+    """Return the alexa ID for an entity ID."""
+    return entity_id.replace(".", "#").translate(TRANSLATION_TABLE)
+
+
 class AlexaEntity:
     """An adaptation of an entity, expressed in Alexa's terms.
 
     The API handlers should manipulate entities only through this interface.
     """
 
-    def __init__(self, opp, config, entity):
+    def __init__(self, opp: OpenPeerPower, config: "AbstractConfig", entity: State):
         """Initialize Alexa Entity."""
         self.opp = opp
         self.config = config
@@ -222,7 +276,7 @@ class AlexaEntity:
 
     def alexa_id(self):
         """Return the Alexa API entity id."""
-        return self.entity.entity_id.replace(".", "#").translate(TRANSLATION_TABLE)
+        return generate_alexa_id(self.entity.entity_id)
 
     def display_categories(self):
         """Return a list of display categories."""
@@ -240,14 +294,13 @@ class AlexaEntity:
         """
         raise NotImplementedError
 
-    def get_interface(self, capability):
+    def get_interface(self, capability) -> AlexaCapability:
         """Return the given AlexaInterface.
 
         Raises _UnsupportedInterface.
         """
-        pass
 
-    def interfaces(self):
+    def interfaces(self) -> List[AlexaCapability]:
         """Return a list of supported interfaces.
 
         Used for discovery. The list should contain AlexaInterface instances.
@@ -261,8 +314,7 @@ class AlexaEntity:
             if not interface.properties_proactively_reported():
                 continue
 
-            for prop in interface.serialize_properties():
-                yield prop
+            yield from interface.serialize_properties()
 
     def serialize_discovery(self):
         """Serialize the entity for discovery."""
@@ -273,13 +325,28 @@ class AlexaEntity:
             "friendlyName": self.friendly_name(),
             "description": self.description(),
             "manufacturerName": "Open Peer Power",
+            "additionalAttributes": {
+                "manufacturer": "Open Peer Power",
+                "model": self.entity.domain,
+                "softwareVersion": __version__,
+                "customIdentifier": f"{self.config.user_identifier()}-{self.entity_id}",
+            },
         }
 
         locale = self.config.locale
         capabilities = []
+
         for i in self.interfaces():
-            if locale in i.supported_locales:
+            if locale not in i.supported_locales:
+                continue
+
+            try:
                 capabilities.append(i.serialize_discovery())
+            except Exception:  # pylint: disable=broad-except
+                _LOGGER.exception(
+                    "Error serializing %s discovery for %s", i.name(), self.entity
+                )
+
         result["capabilities"] = capabilities
 
         return result
@@ -318,6 +385,9 @@ class GenericCapabilities(AlexaEntity):
 
     def default_display_categories(self):
         """Return the display categories for this entity."""
+        if self.entity.domain == automation.DOMAIN:
+            return [DisplayCategory.ACTIVITY_TRIGGER]
+
         return [DisplayCategory.OTHER]
 
     def interfaces(self):
@@ -379,7 +449,7 @@ class CoverCapabilities(AlexaEntity):
     def default_display_categories(self):
         """Return the display categories for this entity."""
         device_class = self.entity.attributes.get(ATTR_DEVICE_CLASS)
-        if device_class == cover.DEVICE_CLASS_GARAGE:
+        if device_class in (cover.DEVICE_CLASS_GARAGE, cover.DEVICE_CLASS_GATE):
             return [DisplayCategory.GARAGE_DOOR]
         if device_class == cover.DEVICE_CLASS_DOOR:
             return [DisplayCategory.DOOR]
@@ -400,7 +470,10 @@ class CoverCapabilities(AlexaEntity):
 
     def interfaces(self):
         """Yield the supported interfaces."""
-        yield AlexaPowerController(self.entity)
+        device_class = self.entity.attributes.get(ATTR_DEVICE_CLASS)
+        if device_class not in (cover.DEVICE_CLASS_GARAGE, cover.DEVICE_CLASS_GATE):
+            yield AlexaPowerController(self.entity)
+
         supported = self.entity.attributes.get(ATTR_SUPPORTED_FEATURES, 0)
         if supported & cover.SUPPORT_SET_POSITION:
             yield AlexaRangeController(
@@ -538,7 +611,11 @@ class MediaPlayerCapabilities(AlexaEntity):
             yield AlexaChannelController(self.entity)
 
         if supported & media_player.const.SUPPORT_SELECT_SOUND_MODE:
-            yield AlexaEqualizerController(self.entity)
+            inputs = AlexaInputController.get_valid_inputs(
+                self.entity.attributes.get(media_player.const.ATTR_SOUND_MODE_LIST, [])
+            )
+            if len(inputs) > 0:
+                yield AlexaEqualizerController(self.entity)
 
         yield AlexaEndpointHealth(self.opp, self.entity)
         yield Alexa(self.opp)
@@ -577,9 +654,8 @@ class ScriptCapabilities(AlexaEntity):
 
     def interfaces(self):
         """Yield the supported interfaces."""
-        can_cancel = bool(self.entity.attributes.get("can_cancel"))
         return [
-            AlexaSceneController(self.entity, supports_deactivation=can_cancel),
+            AlexaSceneController(self.entity, supports_deactivation=True),
             Alexa(self.opp),
         ]
 
@@ -724,6 +800,7 @@ class TimerCapabilities(AlexaEntity):
     def interfaces(self):
         """Yield the supported interfaces."""
         yield AlexaTimeHoldController(self.entity, allow_remote_resume=True)
+        yield AlexaPowerController(self.entity)
         yield Alexa(self.entity)
 
 
@@ -733,13 +810,16 @@ class VacuumCapabilities(AlexaEntity):
 
     def default_display_categories(self):
         """Return the display categories for this entity."""
-        return [DisplayCategory.OTHER]
+        return [DisplayCategory.VACUUM_CLEANER]
 
     def interfaces(self):
         """Yield the supported interfaces."""
         supported = self.entity.attributes.get(ATTR_SUPPORTED_FEATURES, 0)
-        if (supported & vacuum.SUPPORT_TURN_ON) and (
-            supported & vacuum.SUPPORT_TURN_OFF
+        if (
+            (supported & vacuum.SUPPORT_TURN_ON) or (supported & vacuum.SUPPORT_START)
+        ) and (
+            (supported & vacuum.SUPPORT_TURN_OFF)
+            or (supported & vacuum.SUPPORT_RETURN_HOME)
         ):
             yield AlexaPowerController(self.entity)
 
@@ -756,3 +836,47 @@ class VacuumCapabilities(AlexaEntity):
 
         yield AlexaEndpointHealth(self.opp, self.entity)
         yield Alexa(self.opp)
+
+
+@ENTITY_ADAPTERS.register(camera.DOMAIN)
+class CameraCapabilities(AlexaEntity):
+    """Class to represent Camera capabilities."""
+
+    def default_display_categories(self):
+        """Return the display categories for this entity."""
+        return [DisplayCategory.CAMERA]
+
+    def interfaces(self):
+        """Yield the supported interfaces."""
+        if self._check_requirements():
+            supported = self.entity.attributes.get(ATTR_SUPPORTED_FEATURES, 0)
+            if supported & camera.SUPPORT_STREAM:
+                yield AlexaCameraStreamController(self.entity)
+
+        yield AlexaEndpointHealth(self.opp, self.entity)
+        yield Alexa(self.opp)
+
+    def _check_requirements(self):
+        """Check the opp URL for HTTPS scheme."""
+        if "stream" not in self.opp.config.components:
+            _LOGGER.debug(
+                "%s requires stream component for AlexaCameraStreamController",
+                self.entity_id,
+            )
+            return False
+
+        try:
+            network.get_url(
+                self.opp,
+                allow_internal=False,
+                allow_ip=False,
+                require_ssl=True,
+                require_standard_port=True,
+            )
+        except network.NoURLAvailableError:
+            _LOGGER.debug(
+                "%s requires HTTPS for AlexaCameraStreamController", self.entity_id
+            )
+            return False
+
+        return True
