@@ -2,8 +2,9 @@
 import logging
 import math
 
-from openpeerpower import core as op
+from openpeerpower import core as ha
 from openpeerpower.components import (
+    camera,
     cover,
     fan,
     group,
@@ -16,6 +17,7 @@ from openpeerpower.components import (
 from openpeerpower.components.climate import const as climate
 from openpeerpower.const import (
     ATTR_ENTITY_ID,
+    ATTR_ENTITY_PICTURE,
     ATTR_SUPPORTED_FEATURES,
     ATTR_TEMPERATURE,
     SERVICE_ALARM_ARM_AWAY,
@@ -41,6 +43,7 @@ from openpeerpower.const import (
     TEMP_CELSIUS,
     TEMP_FAHRENHEIT,
 )
+from openpeerpower.helpers import network
 import openpeerpower.util.color as color_util
 from openpeerpower.util.decorator import Registry
 import openpeerpower.util.dt as dt_util
@@ -51,7 +54,6 @@ from .const import (
     API_THERMOSTAT_MODES,
     API_THERMOSTAT_MODES_CUSTOM,
     API_THERMOSTAT_PRESETS,
-    PERCENTAGE_FAN_MAP,
     Cause,
     Inputs,
 )
@@ -116,11 +118,17 @@ async def async_api_turn_on(opp, config, directive, context):
     entity = directive.entity
     domain = entity.domain
     if domain == group.DOMAIN:
-        domain = op.DOMAIN
+        domain = ha.DOMAIN
 
     service = SERVICE_TURN_ON
     if domain == cover.DOMAIN:
         service = cover.SERVICE_OPEN_COVER
+    elif domain == vacuum.DOMAIN:
+        supported = entity.attributes.get(ATTR_SUPPORTED_FEATURES, 0)
+        if not supported & vacuum.SUPPORT_TURN_ON and supported & vacuum.SUPPORT_START:
+            service = vacuum.SERVICE_START
+    elif domain == timer.DOMAIN:
+        service = timer.SERVICE_START
     elif domain == media_player.DOMAIN:
         supported = entity.attributes.get(ATTR_SUPPORTED_FEATURES, 0)
         power_features = media_player.SUPPORT_TURN_ON | media_player.SUPPORT_TURN_OFF
@@ -144,11 +152,20 @@ async def async_api_turn_off(opp, config, directive, context):
     entity = directive.entity
     domain = entity.domain
     if entity.domain == group.DOMAIN:
-        domain = op.DOMAIN
+        domain = ha.DOMAIN
 
     service = SERVICE_TURN_OFF
     if entity.domain == cover.DOMAIN:
         service = cover.SERVICE_CLOSE_COVER
+    elif domain == vacuum.DOMAIN:
+        supported = entity.attributes.get(ATTR_SUPPORTED_FEATURES, 0)
+        if (
+            not supported & vacuum.SUPPORT_TURN_OFF
+            and supported & vacuum.SUPPORT_RETURN_HOME
+        ):
+            service = vacuum.SERVICE_RETURN_TO_BASE
+    elif domain == timer.DOMAIN:
+        service = timer.SERVICE_CANCEL
     elif domain == media_player.DOMAIN:
         supported = entity.attributes.get(ATTR_SUPPORTED_FEATURES, 0)
         power_features = media_player.SUPPORT_TURN_ON | media_player.SUPPORT_TURN_OFF
@@ -338,21 +355,13 @@ async def async_api_deactivate(opp, config, directive, context):
 async def async_api_set_percentage(opp, config, directive, context):
     """Process a set percentage request."""
     entity = directive.entity
-    percentage = int(directive.payload["percentage"])
     service = None
     data = {ATTR_ENTITY_ID: entity.entity_id}
 
     if entity.domain == fan.DOMAIN:
-        service = fan.SERVICE_SET_SPEED
-        speed = "off"
-
-        if percentage <= 33:
-            speed = "low"
-        elif percentage <= 66:
-            speed = "medium"
-        elif percentage <= 100:
-            speed = "high"
-        data[fan.ATTR_SPEED] = speed
+        service = fan.SERVICE_SET_PERCENTAGE
+        percentage = int(directive.payload["percentage"])
+        data[fan.ATTR_PERCENTAGE] = percentage
 
     await opp.services.async_call(
         entity.domain, service, data, blocking=False, context=context
@@ -370,22 +379,12 @@ async def async_api_adjust_percentage(opp, config, directive, context):
     data = {ATTR_ENTITY_ID: entity.entity_id}
 
     if entity.domain == fan.DOMAIN:
-        service = fan.SERVICE_SET_SPEED
-        speed = entity.attributes.get(fan.ATTR_SPEED)
-        current = PERCENTAGE_FAN_MAP.get(speed, 100)
+        service = fan.SERVICE_SET_PERCENTAGE
+        current = entity.attributes.get(fan.ATTR_PERCENTAGE) or 0
 
         # set percentage
-        percentage = max(0, percentage_delta + current)
-        speed = "off"
-
-        if percentage <= 33:
-            speed = "low"
-        elif percentage <= 66:
-            speed = "medium"
-        elif percentage <= 100:
-            speed = "high"
-
-        data[fan.ATTR_SPEED] = speed
+        percentage = min(100, max(0, percentage_delta + current))
+        data[fan.ATTR_PERCENTAGE] = percentage
 
     await opp.services.async_call(
         entity.domain, service, data, blocking=False, context=context
@@ -470,7 +469,7 @@ async def async_api_select_input(opp, config, directive, context):
         )
         media_input = media_input.lower().replace(" ", "")
         if (
-            formatted_source in Inputs.VALID_SOURCE_NAME_MAP.keys()
+            formatted_source in Inputs.VALID_SOURCE_NAME_MAP
             and formatted_source == media_input
         ) or (
             media_input.endswith("1") and formatted_source == media_input.rstrip("1")
@@ -478,8 +477,8 @@ async def async_api_select_input(opp, config, directive, context):
             media_input = source
             break
     else:
-        msg = "failed to map input {} to a media source on {}".format(
-            media_input, entity.entity_id
+        msg = (
+            f"failed to map input {media_input} to a media source on {entity.entity_id}"
         )
         raise AlexaInvalidValueError(msg)
 
@@ -551,7 +550,7 @@ async def async_api_adjust_volume_step(opp, config, directive, context):
 
     data = {ATTR_ENTITY_ID: entity.entity_id}
 
-    for _ in range(0, abs(volume_int)):
+    for _ in range(abs(volume_int)):
         await opp.services.async_call(
             entity.domain, service_volume, data, blocking=False, context=context
         )
@@ -768,17 +767,17 @@ async def async_api_set_thermostat_mode(opp, config, directive, context):
 
     data = {ATTR_ENTITY_ID: entity.entity_id}
 
-    op_preset = next((k for k, v in API_THERMOSTAT_PRESETS.items() if v == mode), None)
+    ha_preset = next((k for k, v in API_THERMOSTAT_PRESETS.items() if v == mode), None)
 
-    if op_preset:
+    if ha_preset:
         presets = entity.attributes.get(climate.ATTR_PRESET_MODES, [])
 
-        if op_preset not in presets:
-            msg = f"The requested thermostat mode {op_preset} is not supported"
+        if ha_preset not in presets:
+            msg = f"The requested thermostat mode {ha_preset} is not supported"
             raise AlexaUnsupportedThermostatModeError(msg)
 
         service = climate.SERVICE_SET_PRESET_MODE
-        data[climate.ATTR_PRESET_MODE] = op_preset
+        data[climate.ATTR_PRESET_MODE] = ha_preset
 
     elif mode == "CUSTOM":
         operation_list = entity.attributes.get(climate.ATTR_HVAC_MODES)
@@ -798,14 +797,14 @@ async def async_api_set_thermostat_mode(opp, config, directive, context):
 
     else:
         operation_list = entity.attributes.get(climate.ATTR_HVAC_MODES)
-        op_modes = {k: v for k, v in API_THERMOSTAT_MODES.items() if v == mode}
-        op_mode = next(iter(set(op_modes).intersection(operation_list)), None)
-        if op_mode not in operation_list:
+        ha_modes = {k: v for k, v in API_THERMOSTAT_MODES.items() if v == mode}
+        ha_mode = next(iter(set(ha_modes).intersection(operation_list)), None)
+        if ha_mode not in operation_list:
             msg = f"The requested thermostat mode {mode} is not supported"
             raise AlexaUnsupportedThermostatModeError(msg)
 
         service = climate.SERVICE_SET_HVAC_MODE
-        data[climate.ATTR_HVAC_MODE] = op_mode
+        data[climate.ATTR_HVAC_MODE] = ha_mode
 
     response = directive.response()
     await opp.services.async_call(
@@ -832,22 +831,13 @@ async def async_api_reportstate(opp, config, directive, context):
 async def async_api_set_power_level(opp, config, directive, context):
     """Process a SetPowerLevel request."""
     entity = directive.entity
-    percentage = int(directive.payload["powerLevel"])
     service = None
     data = {ATTR_ENTITY_ID: entity.entity_id}
 
     if entity.domain == fan.DOMAIN:
-        service = fan.SERVICE_SET_SPEED
-        speed = "off"
-
-        if percentage <= 33:
-            speed = "low"
-        elif percentage <= 66:
-            speed = "medium"
-        else:
-            speed = "high"
-
-        data[fan.ATTR_SPEED] = speed
+        service = fan.SERVICE_SET_PERCENTAGE
+        percentage = int(directive.payload["powerLevel"])
+        data[fan.ATTR_PERCENTAGE] = percentage
 
     await opp.services.async_call(
         entity.domain, service, data, blocking=False, context=context
@@ -865,22 +855,12 @@ async def async_api_adjust_power_level(opp, config, directive, context):
     data = {ATTR_ENTITY_ID: entity.entity_id}
 
     if entity.domain == fan.DOMAIN:
-        service = fan.SERVICE_SET_SPEED
-        speed = entity.attributes.get(fan.ATTR_SPEED)
-        current = PERCENTAGE_FAN_MAP.get(speed, 100)
+        service = fan.SERVICE_SET_PERCENTAGE
+        current = entity.attributes.get(fan.ATTR_PERCENTAGE) or 0
 
         # set percentage
-        percentage = max(0, percentage_delta + current)
-        speed = "off"
-
-        if percentage <= 33:
-            speed = "low"
-        elif percentage <= 66:
-            speed = "medium"
-        else:
-            speed = "high"
-
-        data[fan.ATTR_SPEED] = speed
+        percentage = min(100, max(0, percentage_delta + current))
+        data[fan.ATTR_PERCENTAGE] = percentage
 
     await opp.services.async_call(
         entity.domain, service, data, blocking=False, context=context
@@ -903,10 +883,10 @@ async def async_api_arm(opp, config, directive, context):
 
     if arm_state == "ARMED_AWAY":
         service = SERVICE_ALARM_ARM_AWAY
-    if arm_state == "ARMED_STAY":
-        service = SERVICE_ALARM_ARM_HOME
-    if arm_state == "ARMED_NIGHT":
+    elif arm_state == "ARMED_NIGHT":
         service = SERVICE_ALARM_ARM_NIGHT
+    elif arm_state == "ARMED_STAY":
+        service = SERVICE_ALARM_ARM_HOME
 
     await opp.services.async_call(
         entity.domain, service, data, blocking=False, context=context
@@ -977,14 +957,14 @@ async def async_api_set_mode(opp, config, directive, context):
 
     # Fan Direction
     if instance == f"{fan.DOMAIN}.{fan.ATTR_DIRECTION}":
-        _, direction = mode.split(".")
+        direction = mode.split(".")[1]
         if direction in (fan.DIRECTION_REVERSE, fan.DIRECTION_FORWARD):
             service = fan.SERVICE_SET_DIRECTION
             data[fan.ATTR_DIRECTION] = direction
 
     # Cover Position
     elif instance == f"{cover.DOMAIN}.{cover.ATTR_POSITION}":
-        _, position = mode.split(".")
+        position = mode.split(".")[1]
 
         if position == cover.STATE_CLOSED:
             service = cover.SERVICE_CLOSE_COVER
@@ -1225,7 +1205,7 @@ async def async_api_adjust_range(opp, config, directive, context):
         service = SERVICE_SET_COVER_POSITION
         current = entity.attributes.get(cover.ATTR_POSITION)
         if not current:
-            msg = "Unable to determine {} current position".format(entity.entity_id)
+            msg = f"Unable to determine {entity.entity_id} current position"
             raise AlexaInvalidValueError(msg)
         position = response_value = min(100, max(0, range_delta + current))
         if position == 100:
@@ -1241,9 +1221,7 @@ async def async_api_adjust_range(opp, config, directive, context):
         service = SERVICE_SET_COVER_TILT_POSITION
         current = entity.attributes.get(cover.ATTR_TILT_POSITION)
         if not current:
-            msg = "Unable to determine {} current tilt position".format(
-                entity.entity_id
-            )
+            msg = f"Unable to determine {entity.entity_id} current tilt position"
             raise AlexaInvalidValueError(msg)
         tilt_position = response_value = min(100, max(0, range_delta + current))
         if tilt_position == 100:
@@ -1368,7 +1346,7 @@ async def async_api_skipchannel(opp, config, directive, context):
     else:
         service_media = SERVICE_MEDIA_NEXT_TRACK
 
-    for _ in range(0, abs(channel)):
+    for _ in range(abs(channel)):
         await opp.services.async_call(
             entity.domain, service_media, data, blocking=False, context=context
         )
@@ -1439,9 +1417,7 @@ async def async_api_set_eq_mode(opp, config, directive, context):
     if sound_mode_list and mode.lower() in sound_mode_list:
         data[media_player.const.ATTR_SOUND_MODE] = mode.lower()
     else:
-        msg = "failed to map sound mode {} to a mode on {}".format(
-            mode, entity.entity_id
-        )
+        msg = f"failed to map sound mode {mode} to a mode on {entity.entity_id}"
         raise AlexaInvalidValueError(msg)
 
     await opp.services.async_call(
@@ -1512,3 +1488,41 @@ async def async_api_resume(opp, config, directive, context):
     )
 
     return directive.response()
+
+
+@HANDLERS.register(("Alexa.CameraStreamController", "InitializeCameraStreams"))
+async def async_api_initialize_camera_stream(opp, config, directive, context):
+    """Process a InitializeCameraStreams request."""
+    entity = directive.entity
+    stream_source = await camera.async_request_stream(opp, entity.entity_id, fmt="hls")
+    camera_image = opp.states.get(entity.entity_id).attributes[ATTR_ENTITY_PICTURE]
+
+    try:
+        external_url = network.get_url(
+            opp,
+            allow_internal=False,
+            allow_ip=False,
+            require_ssl=True,
+            require_standard_port=True,
+        )
+    except network.NoURLAvailableError as err:
+        raise AlexaInvalidValueError(
+            "Failed to find suitable URL to serve to Alexa"
+        ) from err
+
+    payload = {
+        "cameraStreams": [
+            {
+                "uri": f"{external_url}{stream_source}",
+                "protocol": "HLS",
+                "resolution": {"width": 1280, "height": 720},
+                "authorizationType": "NONE",
+                "videoCodec": "H264",
+                "audioCodec": "AAC",
+            }
+        ],
+        "imageUri": f"{external_url}{camera_image}",
+    }
+    return directive.response(
+        name="Response", namespace="Alexa.CameraStreamController", payload=payload
+    )
