@@ -1,116 +1,98 @@
 """Allow to set up simple automation rules via the config file."""
-import importlib
 import logging
-from typing import Any, Awaitable, Callable, List, Optional, Set
+from typing import Any, Awaitable, Callable, Dict, List, Optional, Set, Union, cast
 
 import voluptuous as vol
+from voluptuous.humanize import humanize_error
 
+from openpeerpower.components import blueprint
 from openpeerpower.const import (
     ATTR_ENTITY_ID,
     ATTR_NAME,
+    CONF_ALIAS,
+    CONF_CONDITION,
     CONF_DEVICE_ID,
     CONF_ENTITY_ID,
     CONF_ID,
+    CONF_MODE,
     CONF_PLATFORM,
+    CONF_VARIABLES,
     CONF_ZONE,
-    EVENT_AUTOMATION_TRIGGERED,
-    EVENT_OPENPEERPOWER_START,
+    EVENT_OPENPEERPOWER_STARTED,
     SERVICE_RELOAD,
     SERVICE_TOGGLE,
     SERVICE_TURN_OFF,
     SERVICE_TURN_ON,
     STATE_ON,
 )
-from openpeerpower.core import Context, CoreState, OpenPeerPower, callback
-from openpeerpower.exceptions import OpenPeerPowerError
-from openpeerpower.helpers import condition, extract_domain_configs, script
+from openpeerpower.core import (
+    Context,
+    CoreState,
+    OpenPeerPower,
+    callback,
+    split_entity_id,
+)
+from openpeerpower.exceptions import (
+    ConditionError,
+    ConditionErrorContainer,
+    ConditionErrorIndex,
+    OpenPeerPowerError,
+)
+from openpeerpower.helpers import condition, extract_domain_configs, template
 import openpeerpower.helpers.config_validation as cv
 from openpeerpower.helpers.entity import ToggleEntity
 from openpeerpower.helpers.entity_component import EntityComponent
 from openpeerpower.helpers.restore_state import RestoreEntity
+from openpeerpower.helpers.script import (
+    ATTR_CUR,
+    ATTR_MAX,
+    ATTR_MODE,
+    CONF_MAX,
+    CONF_MAX_EXCEEDED,
+    Script,
+)
+from openpeerpower.helpers.script_variables import ScriptVariables
 from openpeerpower.helpers.service import async_register_admin_service
+from openpeerpower.helpers.trigger import async_initialize_triggers
 from openpeerpower.helpers.typing import TemplateVarsType
 from openpeerpower.loader import bind_opp
-from openpeerpower.util.dt import parse_datetime, utcnow
+from openpeerpower.util.dt import parse_datetime
+
+# Not used except by packages to check config structure
+from .config import PLATFORM_SCHEMA  # noqa: F401
+from .config import async_validate_config_item
+from .const import (
+    CONF_ACTION,
+    CONF_INITIAL_STATE,
+    CONF_TRIGGER,
+    CONF_TRIGGER_VARIABLES,
+    DEFAULT_INITIAL_STATE,
+    DOMAIN,
+    LOGGER,
+)
+from .helpers import async_get_blueprints
 
 # mypy: allow-untyped-calls, allow-untyped-defs
 # mypy: no-check-untyped-defs, no-warn-return-any
 
-DOMAIN = "automation"
 ENTITY_ID_FORMAT = DOMAIN + ".{}"
 
-GROUP_NAME_ALL_AUTOMATIONS = "all automations"
 
-CONF_ALIAS = "alias"
-CONF_DESCRIPTION = "description"
-CONF_HIDE_ENTITY = "hide_entity"
-
-CONF_CONDITION = "condition"
-CONF_ACTION = "action"
-CONF_TRIGGER = "trigger"
-CONF_CONDITION_TYPE = "condition_type"
-CONF_INITIAL_STATE = "initial_state"
 CONF_SKIP_CONDITION = "skip_condition"
+CONF_STOP_ACTIONS = "stop_actions"
+DEFAULT_STOP_ACTIONS = True
 
-CONDITION_USE_TRIGGER_VALUES = "use_trigger_values"
-CONDITION_TYPE_AND = "and"
-CONDITION_TYPE_OR = "or"
-
-DEFAULT_CONDITION_TYPE = CONDITION_TYPE_AND
-DEFAULT_HIDE_ENTITY = False
-DEFAULT_INITIAL_STATE = True
+EVENT_AUTOMATION_RELOADED = "automation_reloaded"
+EVENT_AUTOMATION_TRIGGERED = "automation_triggered"
 
 ATTR_LAST_TRIGGERED = "last_triggered"
+ATTR_SOURCE = "source"
 ATTR_VARIABLES = "variables"
 SERVICE_TRIGGER = "trigger"
 
-_LOGGER = logging.getLogger(__name__)
-
 AutomationActionType = Callable[[OpenPeerPower, TemplateVarsType], Awaitable[None]]
 
 
-def _platform_validator(config):
-    """Validate it is a valid platform."""
-    try:
-        platform = importlib.import_module(
-            ".{}".format(config[CONF_PLATFORM]), __name__
-        )
-    except ImportError:
-        raise vol.Invalid("Invalid platform specified") from None
-
-    return platform.TRIGGER_SCHEMA(config)
-
-
-_TRIGGER_SCHEMA = vol.All(
-    cv.ensure_list,
-    [
-        vol.All(
-            vol.Schema({vol.Required(CONF_PLATFORM): str}, extra=vol.ALLOW_EXTRA),
-            _platform_validator,
-        )
-    ],
-)
-
-_CONDITION_SCHEMA = vol.All(cv.ensure_list, [cv.CONDITION_SCHEMA])
-
-PLATFORM_SCHEMA = vol.All(
-    cv.deprecated(CONF_HIDE_ENTITY, invalidation_version="0.107"),
-    vol.Schema(
-        {
-            # str on purpose
-            CONF_ID: str,
-            CONF_ALIAS: cv.string,
-            vol.Optional(CONF_DESCRIPTION): cv.string,
-            vol.Optional(CONF_INITIAL_STATE): cv.boolean,
-            vol.Optional(CONF_HIDE_ENTITY, default=DEFAULT_HIDE_ENTITY): cv.boolean,
-            vol.Required(CONF_TRIGGER): _TRIGGER_SCHEMA,
-            vol.Optional(CONF_CONDITION): _CONDITION_SCHEMA,
-            vol.Required(CONF_ACTION): cv.SCRIPT_SCHEMA,
-        }
-    ),
-)
-
-
 @bind_opp
 def is_on(opp, entity_id):
     """
@@ -129,13 +111,11 @@ def automations_with_entity(opp: OpenPeerPower, entity_id: str) -> List[str]:
 
     component = opp.data[DOMAIN]
 
-    results = []
-
-    for automation_entity in component.entities:
-        if entity_id in automation_entity.referenced_entities:
-            results.append(automation_entity.entity_id)
-
-    return results
+    return [
+        automation_entity.entity_id
+        for automation_entity in component.entities
+        if entity_id in automation_entity.referenced_entities
+    ]
 
 
 @callback
@@ -162,13 +142,11 @@ def automations_with_device(opp: OpenPeerPower, device_id: str) -> List[str]:
 
     component = opp.data[DOMAIN]
 
-    results = []
-
-    for automation_entity in component.entities:
-        if device_id in automation_entity.referenced_devices:
-            results.append(automation_entity.entity_id)
-
-    return results
+    return [
+        automation_entity.entity_id
+        for automation_entity in component.entities
+        if device_id in automation_entity.referenced_devices
+    ]
 
 
 @callback
@@ -189,9 +167,13 @@ def devices_in_automation(opp: OpenPeerPower, entity_id: str) -> List[str]:
 
 async def async_setup(opp, config):
     """Set up the automation."""
-    opp.data[DOMAIN] = component = EntityComponent(_LOGGER, DOMAIN, opp)
+    opp.data[DOMAIN] = component = EntityComponent(LOGGER, DOMAIN, opp)
 
-    await _async_process_config(opp, config, component)
+    # To register the automation blueprints
+    async_get_blueprints(opp)
+
+    if not await _async_process_config(opp, config, component):
+        await async_get_blueprints(opp).async_populate()
 
     async def trigger_service_handler(entity, service_call):
         """Handle automation triggers."""
@@ -211,17 +193,23 @@ async def async_setup(opp, config):
     )
     component.async_register_entity_service(SERVICE_TOGGLE, {}, "async_toggle")
     component.async_register_entity_service(SERVICE_TURN_ON, {}, "async_turn_on")
-    component.async_register_entity_service(SERVICE_TURN_OFF, {}, "async_turn_off")
+    component.async_register_entity_service(
+        SERVICE_TURN_OFF,
+        {vol.Optional(CONF_STOP_ACTIONS, default=DEFAULT_STOP_ACTIONS): cv.boolean},
+        "async_turn_off",
+    )
 
     async def reload_service_handler(service_call):
         """Remove all automations and load new ones from config."""
         conf = await component.async_prepare_reload()
         if conf is None:
             return
+        async_get_blueprints(opp).async_reset_cache()
         await _async_process_config(opp, conf, component)
+        opp.bus.async_fire(EVENT_AUTOMATION_RELOADED, context=service_call.context)
 
     async_register_admin_service(
-        opp, DOMAIN, SERVICE_RELOAD, reload_service_handler, schema=vol.Schema({}),
+        opp, DOMAIN, SERVICE_RELOAD, reload_service_handler, schema=vol.Schema({})
     )
 
     return True
@@ -237,8 +225,9 @@ class AutomationEntity(ToggleEntity, RestoreEntity):
         trigger_config,
         cond_func,
         action_script,
-        hidden,
         initial_state,
+        variables,
+        trigger_variables,
     ):
         """Initialize an automation entity."""
         self._id = automation_id
@@ -247,12 +236,14 @@ class AutomationEntity(ToggleEntity, RestoreEntity):
         self._async_detach_triggers = None
         self._cond_func = cond_func
         self.action_script = action_script
-        self._last_triggered = None
-        self._hidden = hidden
+        self.action_script.change_listener = self.async_write_op_state
         self._initial_state = initial_state
         self._is_enabled = False
         self._referenced_entities: Optional[Set[str]] = None
         self._referenced_devices: Optional[Set[str]] = None
+        self._logger = LOGGER
+        self._variables: ScriptVariables = variables
+        self._trigger_variables: ScriptVariables = trigger_variables
 
     @property
     def name(self):
@@ -272,12 +263,14 @@ class AutomationEntity(ToggleEntity, RestoreEntity):
     @property
     def state_attributes(self):
         """Return the entity state attributes."""
-        return {ATTR_LAST_TRIGGERED: self._last_triggered}
-
-    @property
-    def hidden(self) -> bool:
-        """Return True if the automation entity should be hidden from UIs."""
-        return self._hidden
+        attrs = {
+            ATTR_LAST_TRIGGERED: self.action_script.last_triggered,
+            ATTR_MODE: self.action_script.script_mode,
+            ATTR_CUR: self.action_script.runs,
+        }
+        if self.action_script.supports_max:
+            attrs[ATTR_MAX] = self.action_script.max_runs
+        return attrs
 
     @property
     def is_on(self) -> bool:
@@ -327,13 +320,18 @@ class AutomationEntity(ToggleEntity, RestoreEntity):
         """Startup with initial state or previous state."""
         await super().async_added_to_opp()
 
+        self._logger = logging.getLogger(
+            f"{__name__}.{split_entity_id(self.entity_id)[1]}"
+        )
+        self.action_script.update_logger(self._logger)
+
         state = await self.async_get_last_state()
         if state:
             enable_automation = state.state == STATE_ON
             last_triggered = state.attributes.get("last_triggered")
             if last_triggered is not None:
-                self._last_triggered = parse_datetime(last_triggered)
-            _LOGGER.debug(
+                self.action_script.last_triggered = parse_datetime(last_triggered)
+            self._logger.debug(
                 "Loaded automation %s with state %s from state "
                 " storage last state %s",
                 self.entity_id,
@@ -342,15 +340,15 @@ class AutomationEntity(ToggleEntity, RestoreEntity):
             )
         else:
             enable_automation = DEFAULT_INITIAL_STATE
-            _LOGGER.debug(
-                "Automation %s not in state storage, state %s from default is used.",
+            self._logger.debug(
+                "Automation %s not in state storage, state %s from default is used",
                 self.entity_id,
                 enable_automation,
             )
 
         if self._initial_state is not None:
             enable_automation = self._initial_state
-            _LOGGER.debug(
+            self._logger.debug(
                 "Automation %s initial state %s overridden from "
                 "config initial_state",
                 self.entity_id,
@@ -366,13 +364,25 @@ class AutomationEntity(ToggleEntity, RestoreEntity):
 
     async def async_turn_off(self, **kwargs: Any) -> None:
         """Turn the entity off."""
-        await self.async_disable()
+        if CONF_STOP_ACTIONS in kwargs:
+            await self.async_disable(kwargs[CONF_STOP_ACTIONS])
+        else:
+            await self.async_disable()
 
-    async def async_trigger(self, variables, skip_condition=False, context=None):
+    async def async_trigger(self, run_variables, context=None, skip_condition=False):
         """Trigger automation.
 
         This method is a coroutine.
         """
+        if self._variables:
+            try:
+                variables = self._variables.async_render(self.opp, run_variables)
+            except template.TemplateError as err:
+                self._logger.error("Error rendering variables: %s", err)
+                return
+        else:
+            variables = run_variables
+
         if (
             not skip_condition
             and self._cond_func is not None
@@ -385,23 +395,31 @@ class AutomationEntity(ToggleEntity, RestoreEntity):
         trigger_context = Context(parent_id=parent_id)
 
         self.async_set_context(trigger_context)
-        self.opp.bus.async_fire(
-            EVENT_AUTOMATION_TRIGGERED,
-            {ATTR_NAME: self._name, ATTR_ENTITY_ID: self.entity_id},
-            context=trigger_context,
-        )
+        event_data = {
+            ATTR_NAME: self._name,
+            ATTR_ENTITY_ID: self.entity_id,
+        }
+        if "trigger" in variables and "description" in variables["trigger"]:
+            event_data[ATTR_SOURCE] = variables["trigger"]["description"]
 
-        _LOGGER.info("Executing %s", self._name)
-
-        try:
-            await self.action_script.async_run(variables, trigger_context)
-        except Exception as err:  # pylint: disable=broad-except
-            self.action_script.async_log_exception(
-                _LOGGER, f"Error while executing automation {self.entity_id}", err
+        @callback
+        def started_action():
+            self.opp.bus.async_fire(
+                EVENT_AUTOMATION_TRIGGERED, event_data, context=trigger_context
             )
 
-        self._last_triggered = utcnow()
-        await self.async_update_op_state()
+        try:
+            await self.action_script.async_run(
+                variables, trigger_context, started_action
+            )
+        except (vol.Invalid, OpenPeerPowerError) as err:
+            self._logger.error(
+                "Error while executing automation %s: %s",
+                self.entity_id,
+                err,
+            )
+        except Exception:  # pylint: disable=broad-except
+            self._logger.exception("While executing automation %s", self.entity_id)
 
     async def async_will_remove_from_opp(self):
         """Remove listeners when removing automation from Open Peer Power."""
@@ -420,7 +438,7 @@ class AutomationEntity(ToggleEntity, RestoreEntity):
 
         # OpenPeerPower is starting up
         if self.opp.state != CoreState.not_running:
-            self._async_detach_triggers = await self._async_attach_triggers()
+            self._async_detach_triggers = await self._async_attach_triggers(False)
             self.async_write_op_state()
             return
 
@@ -430,16 +448,16 @@ class AutomationEntity(ToggleEntity, RestoreEntity):
             if not self._is_enabled or self._async_detach_triggers is not None:
                 return
 
-            self._async_detach_triggers = await self._async_attach_triggers()
+            self._async_detach_triggers = await self._async_attach_triggers(True)
 
         self.opp.bus.async_listen_once(
-            EVENT_OPENPEERPOWER_START, async_enable_automation
+            EVENT_OPENPEERPOWER_STARTED, async_enable_automation
         )
         self.async_write_op_state()
 
-    async def async_disable(self):
+    async def async_disable(self, stop_actions=DEFAULT_STOP_ACTIONS):
         """Disable the automation entity."""
-        if not self._is_enabled:
+        if not self._is_enabled and not self.action_script.runs:
             return
 
         self._is_enabled = False
@@ -448,39 +466,39 @@ class AutomationEntity(ToggleEntity, RestoreEntity):
             self._async_detach_triggers()
             self._async_detach_triggers = None
 
+        if stop_actions:
+            await self.action_script.async_stop()
+
         self.async_write_op_state()
 
-    async def _async_attach_triggers(self):
+    async def _async_attach_triggers(
+        self, open_peer_power_start: bool
+    ) -> Optional[Callable[[], None]]:
         """Set up the triggers."""
-        removes = []
-        info = {"name": self._name}
 
-        for conf in self._trigger_config:
-            platform = importlib.import_module(
-                ".{}".format(conf[CONF_PLATFORM]), __name__
-            )
+        def log_cb(level, msg, **kwargs):
+            self._logger.log(level, "%s %s", msg, self._name, **kwargs)
 
-            remove = await platform.async_attach_trigger(
-                self.opp, conf, self.async_trigger, info
-            )
+        variables = None
+        if self._trigger_variables:
+            try:
+                variables = self._trigger_variables.async_render(
+                    cast(OpenPeerPower, self.opp), None, limited=True
+                )
+            except template.TemplateError as err:
+                self._logger.error("Error rendering trigger variables: %s", err)
+                return None
 
-            if not remove:
-                _LOGGER.error("Error setting up trigger %s", self._name)
-                continue
-
-            _LOGGER.info("Initialized trigger %s", self._name)
-            removes.append(remove)
-
-        if not removes:
-            return None
-
-        @callback
-        def remove_triggers():
-            """Remove attached triggers."""
-            for remove in removes:
-                remove()
-
-        return remove_triggers
+        return await async_initialize_triggers(
+            cast(OpenPeerPower, self.opp),
+            self._trigger_config,
+            self.async_trigger,
+            DOMAIN,
+            self._name,
+            log_cb,
+            open_peer_power_start,
+            variables,
+        )
 
     @property
     def device_state_attributes(self):
@@ -491,41 +509,93 @@ class AutomationEntity(ToggleEntity, RestoreEntity):
         return {CONF_ID: self._id}
 
 
-async def _async_process_config(opp, config, component):
+async def _async_process_config(
+    opp: OpenPeerPower,
+    config: Dict[str, Any],
+    component: EntityComponent,
+) -> bool:
     """Process config and add automations.
 
-    This method is a coroutine.
+    Returns if blueprints were used.
     """
     entities = []
+    blueprints_used = False
 
     for config_key in extract_domain_configs(config, DOMAIN):
-        conf = config[config_key]
+        conf: List[Union[Dict[str, Any], blueprint.BlueprintInputs]] = config[  # type: ignore
+            config_key
+        ]
 
         for list_no, config_block in enumerate(conf):
+            if isinstance(config_block, blueprint.BlueprintInputs):  # type: ignore
+                blueprints_used = True
+                blueprint_inputs = config_block
+
+                try:
+                    config_block = cast(
+                        Dict[str, Any],
+                        await async_validate_config_item(
+                            opp, blueprint_inputs.async_substitute()
+                        ),
+                    )
+                except vol.Invalid as err:
+                    LOGGER.error(
+                        "Blueprint %s generated invalid automation with inputs %s: %s",
+                        blueprint_inputs.blueprint.name,
+                        blueprint_inputs.inputs,
+                        humanize_error(config_block, err),
+                    )
+                    continue
+
             automation_id = config_block.get(CONF_ID)
             name = config_block.get(CONF_ALIAS) or f"{config_key} {list_no}"
 
-            hidden = config_block[CONF_HIDE_ENTITY]
             initial_state = config_block.get(CONF_INITIAL_STATE)
 
-            action_script = script.Script(opp, config_block.get(CONF_ACTION, {}), name)
+            action_script = Script(
+                opp,
+                config_block[CONF_ACTION],
+                name,
+                DOMAIN,
+                running_description="automation actions",
+                script_mode=config_block[CONF_MODE],
+                max_runs=config_block[CONF_MAX],
+                max_exceeded=config_block[CONF_MAX_EXCEEDED],
+                logger=LOGGER,
+                # We don't pass variables here
+                # Automation will already render them to use them in the condition
+                # and so will pass them on to the script.
+            )
 
             if CONF_CONDITION in config_block:
-                cond_func = await _async_process_if(opp, config, config_block)
+                cond_func = await _async_process_if(opp, name, config, config_block)
 
                 if cond_func is None:
                     continue
             else:
                 cond_func = None
 
+            # Add trigger variables to variables
+            variables = None
+            if CONF_TRIGGER_VARIABLES in config_block:
+                variables = ScriptVariables(
+                    dict(config_block[CONF_TRIGGER_VARIABLES].as_dict())
+                )
+            if CONF_VARIABLES in config_block:
+                if variables:
+                    variables.variables.update(config_block[CONF_VARIABLES].as_dict())
+                else:
+                    variables = config_block[CONF_VARIABLES]
+
             entity = AutomationEntity(
                 automation_id,
                 name,
                 config_block[CONF_TRIGGER],
                 cond_func,
                 action_script,
-                hidden,
                 initial_state,
+                variables,
+                config_block.get(CONF_TRIGGER_VARIABLES),
             )
 
             entities.append(entity)
@@ -533,8 +603,10 @@ async def _async_process_config(opp, config, component):
     if entities:
         await component.async_add_entities(entities)
 
+    return blueprints_used
 
-async def _async_process_if(opp, config, p_config):
+
+async def _async_process_if(opp, name, config, p_config):
     """Process if checks."""
     if_configs = p_config[CONF_CONDITION]
 
@@ -543,12 +615,32 @@ async def _async_process_if(opp, config, p_config):
         try:
             checks.append(await condition.async_from_config(opp, if_config, False))
         except OpenPeerPowerError as ex:
-            _LOGGER.warning("Invalid condition: %s", ex)
+            LOGGER.warning("Invalid condition: %s", ex)
             return None
 
     def if_action(variables=None):
         """AND all conditions."""
-        return all(check(opp, variables) for check in checks)
+        errors = []
+        for index, check in enumerate(checks):
+            try:
+                if not check(opp, variables):
+                    return False
+            except ConditionError as ex:
+                errors.append(
+                    ConditionErrorIndex(
+                        "condition", index=index, total=len(checks), error=ex
+                    )
+                )
+
+        if errors:
+            LOGGER.warning(
+                "Error evaluating condition in '%s':\n%s",
+                name,
+                ConditionErrorContainer("condition", errors=errors),
+            )
+            return False
+
+        return True
 
     if_action.config = if_configs
 
