@@ -6,6 +6,7 @@ from contextlib import suppress
 from datetime import timedelta
 import hashlib
 import logging
+import os
 from random import SystemRandom
 
 from aiohttp import web
@@ -18,22 +19,17 @@ from openpeerpower.components.http import KEY_AUTHENTICATED, OpenPeerPowerView
 from openpeerpower.components.media_player.const import (
     ATTR_MEDIA_CONTENT_ID,
     ATTR_MEDIA_CONTENT_TYPE,
+    ATTR_MEDIA_EXTRA,
     DOMAIN as DOMAIN_MP,
     SERVICE_PLAY_MEDIA,
 )
-from openpeerpower.components.stream import request_stream
-from openpeerpower.components.stream.const import (
-    CONF_DURATION,
-    CONF_LOOKBACK,
-    CONF_STREAM_SOURCE,
-    DOMAIN as DOMAIN_STREAM,
-    FORMAT_CONTENT_TYPE,
-    OUTPUT_FORMATS,
-    SERVICE_RECORD,
-)
+from openpeerpower.components.stream import Stream, create_stream
+from openpeerpower.components.stream.const import FORMAT_CONTENT_TYPE, OUTPUT_FORMATS
 from openpeerpower.const import (
     ATTR_ENTITY_ID,
     CONF_FILENAME,
+    CONTENT_TYPE_MULTIPART,
+    EVENT_OPENPEERPOWER_START,
     SERVICE_TURN_OFF,
     SERVICE_TURN_ON,
 )
@@ -44,12 +40,20 @@ from openpeerpower.helpers.config_validation import (  # noqa: F401
     PLATFORM_SCHEMA,
     PLATFORM_SCHEMA_BASE,
 )
-from openpeerpower.helpers.entity import Entity
+from openpeerpower.helpers.entity import Entity, entity_sources
 from openpeerpower.helpers.entity_component import EntityComponent
+from openpeerpower.helpers.network import get_url
 from openpeerpower.loader import bind_opp
-from openpeerpower.setup import async_when_setup
 
-from .const import DATA_CAMERA_PREFS, DOMAIN
+from .const import (
+    CAMERA_IMAGE_TIMEOUT,
+    CAMERA_STREAM_SOURCE_TIMEOUT,
+    CONF_DURATION,
+    CONF_LOOKBACK,
+    DATA_CAMERA_PREFS,
+    DOMAIN,
+    SERVICE_RECORD,
+)
 from .prefs import CameraPreferences
 
 # mypy: allow-untyped-calls, allow-untyped-defs
@@ -118,25 +122,15 @@ SCHEMA_WS_CAMERA_THUMBNAIL = websocket_api.BASE_COMMAND_MESSAGE_SCHEMA.extend(
 class Image:
     """Represent an image."""
 
-    content_type = attr.ib(type=str)
-    content = attr.ib(type=bytes)
+    content_type: str = attr.ib()
+    content: bytes = attr.ib()
 
 
 @bind_opp
 async def async_request_stream(opp, entity_id, fmt):
     """Request a stream for a camera entity."""
     camera = _get_camera_from_entity_id(opp, entity_id)
-    camera_prefs = opp.data[DATA_CAMERA_PREFS].get(entity_id)
-
-    async with async_timeout.timeout(10):
-        source = await camera.stream_source()
-
-    if not source:
-        raise OpenPeerPowerError(
-            f"{camera.entity_id} does not support play stream service"
-        )
-
-    return request_stream(opp, source, fmt=fmt, keepalive=camera_prefs.preload_stream)
+    return await _async_stream_endpoint_url(opp, camera, fmt)
 
 
 @bind_opp
@@ -154,6 +148,14 @@ async def async_get_image(opp, entity_id, timeout=10):
     raise OpenPeerPowerError("Unable to get image")
 
 
+@bind_opp
+async def async_get_stream_source(opp, entity_id):
+    """Fetch the stream source for a camera entity."""
+    camera = _get_camera_from_entity_id(opp, entity_id)
+
+    return await camera.stream_source()
+
+
 @bind_opp
 async def async_get_mjpeg_stream(opp, request, entity_id):
     """Fetch an mjpeg stream from a camera entity."""
@@ -168,7 +170,7 @@ async def async_get_still_stream(request, image_cb, content_type, interval):
     This method must be run in the event loop.
     """
     response = web.StreamResponse()
-    response.content_type = "multipart/x-mixed-replace; boundary=--frameboundary"
+    response.content_type = CONTENT_TYPE_MULTIPART.format("--frameboundary")
     await response.prepare(request)
 
     async def write_to_mjpeg_stream(img_bytes):
@@ -225,9 +227,7 @@ def _get_camera_from_entity_id(opp, entity_id):
 
 async def async_setup(opp, config):
     """Set up the camera component."""
-    component = opp.data[DOMAIN] = EntityComponent(
-        _LOGGER, DOMAIN, opp, SCAN_INTERVAL
-    )
+    component = opp.data[DOMAIN] = EntityComponent(_LOGGER, DOMAIN, opp, SCAN_INTERVAL)
 
     prefs = CameraPreferences(opp)
     await prefs.async_initialize()
@@ -244,28 +244,26 @@ async def async_setup(opp, config):
 
     await component.async_setup(config)
 
-    async def preload_stream(opp, _):
+    async def preload_stream(_):
         for camera in component.entities:
             camera_prefs = prefs.get(camera.entity_id)
             if not camera_prefs.preload_stream:
                 continue
-
-            async with async_timeout.timeout(10):
-                source = await camera.stream_source()
-
-            if not source:
+            stream = await camera.create_stream()
+            if not stream:
                 continue
+            stream.keepalive = True
+            stream.add_provider("hls")
+            stream.start()
 
-            request_stream(opp, source, keepalive=True)
-
-    async_when_setup(opp, DOMAIN_STREAM, preload_stream)
+    opp.bus.async_listen_once(EVENT_OPENPEERPOWER_START, preload_stream)
 
     @callback
     def update_tokens(time):
         """Update tokens of the entities."""
         for entity in component.entities:
             entity.async_update_token()
-            opp.async_create_task(entity.async_update_op_state())
+            entity.async_write_op_state()
 
     opp.helpers.event.async_track_time_interval(update_tokens, TOKEN_CHANGE_INTERVAL)
 
@@ -312,6 +310,8 @@ class Camera(Entity):
     def __init__(self):
         """Initialize a camera."""
         self.is_streaming = False
+        self.stream = None
+        self.stream_options = {}
         self.content_type = DEFAULT_CONTENT_TYPE
         self.access_tokens: collections.deque = collections.deque([], 2)
         self.async_update_token()
@@ -356,6 +356,17 @@ class Camera(Entity):
         """Return the interval between frames of the mjpeg stream."""
         return 0.5
 
+    async def create_stream(self) -> Stream:
+        """Create a Stream for stream_source."""
+        # There is at most one stream (a decode worker) per camera
+        if not self.stream:
+            async with async_timeout.timeout(CAMERA_STREAM_SOURCE_TIMEOUT):
+                source = await self.stream_source()
+            if not source:
+                return None
+            self.stream = create_stream(self.opp, source, options=self.stream_options)
+        return self.stream
+
     async def stream_source(self):
         """Return the source of the stream."""
         return None
@@ -366,7 +377,7 @@ class Camera(Entity):
 
     async def async_camera_image(self):
         """Return bytes of camera image."""
-        return await self.opp.async_add_job(self.camera_image)
+        return await self.opp.async_add_executor_job(self.camera_image)
 
     async def handle_async_still_stream(self, request, interval):
         """Generate an HTTP MJPEG stream from camera images."""
@@ -402,7 +413,7 @@ class Camera(Entity):
 
     async def async_turn_off(self):
         """Turn off camera."""
-        await self.opp.async_add_job(self.turn_off)
+        await self.opp.async_add_executor_job(self.turn_off)
 
     def turn_on(self):
         """Turn off camera."""
@@ -410,25 +421,23 @@ class Camera(Entity):
 
     async def async_turn_on(self):
         """Turn off camera."""
-        await self.opp.async_add_job(self.turn_on)
+        await self.opp.async_add_executor_job(self.turn_on)
 
     def enable_motion_detection(self):
         """Enable motion detection in the camera."""
         raise NotImplementedError()
 
-    @callback
-    def async_enable_motion_detection(self):
+    async def async_enable_motion_detection(self):
         """Call the job and enable motion detection."""
-        return self.opp.async_add_job(self.enable_motion_detection)
+        await self.opp.async_add_executor_job(self.enable_motion_detection)
 
     def disable_motion_detection(self):
         """Disable motion detection in camera."""
         raise NotImplementedError()
 
-    @callback
-    def async_disable_motion_detection(self):
+    async def async_disable_motion_detection(self):
         """Call the job and disable motion detection."""
-        return self.opp.async_add_job(self.disable_motion_detection)
+        await self.opp.async_add_executor_job(self.disable_motion_detection)
 
     @property
     def state_attributes(self):
@@ -459,11 +468,11 @@ class CameraView(OpenPeerPowerView):
 
     requires_auth = False
 
-    def __init__(self, component):
+    def __init__(self, component: EntityComponent) -> None:
         """Initialize a basic camera view."""
         self.component = component
 
-    async def get(self, request, entity_id):
+    async def get(self, request: web.Request, entity_id: str) -> web.Response:
         """Start a GET request."""
         camera = self.component.get_entity(entity_id)
 
@@ -479,7 +488,7 @@ class CameraView(OpenPeerPowerView):
             raise web.HTTPUnauthorized()
 
         if not camera.is_on:
-            _LOGGER.debug("Camera is off.")
+            _LOGGER.debug("Camera is off")
             raise web.HTTPServiceUnavailable()
 
         return await self.handle(request, camera)
@@ -495,10 +504,10 @@ class CameraImageView(CameraView):
     url = "/api/camera_proxy/{entity_id}"
     name = "api:camera:image"
 
-    async def handle(self, request, camera):
+    async def handle(self, request: web.Request, camera: Camera) -> web.Response:
         """Serve camera image."""
         with suppress(asyncio.CancelledError, asyncio.TimeoutError):
-            async with async_timeout.timeout(10):
+            async with async_timeout.timeout(CAMERA_IMAGE_TIMEOUT):
                 image = await camera.async_camera_image()
 
             if image:
@@ -513,7 +522,7 @@ class CameraMjpegStream(CameraView):
     url = "/api/camera_proxy_stream/{entity_id}"
     name = "api:camera:stream"
 
-    async def handle(self, request, camera):
+    async def handle(self, request: web.Request, camera: Camera) -> web.Response:
         """Serve camera stream, possibly with interval."""
         interval = request.query.get("interval")
         if interval is None:
@@ -525,8 +534,8 @@ class CameraMjpegStream(CameraView):
             if interval < MIN_STREAM_INTERVAL:
                 raise ValueError(f"Stream interval must be be > {MIN_STREAM_INTERVAL}")
             return await camera.handle_async_still_stream(request, interval)
-        except ValueError:
-            raise web.HTTPBadRequest()
+        except ValueError as err:
+            raise web.HTTPBadRequest() from err
 
 
 @websocket_api.async_response
@@ -535,6 +544,7 @@ async def websocket_camera_thumbnail(opp, connection, msg):
 
     Async friendly.
     """
+    _LOGGER.warning("The websocket command 'camera_thumbnail' has been deprecated")
     try:
         image = await async_get_image(opp, msg["entity_id"])
         await connection.send_big_result(
@@ -568,20 +578,7 @@ async def ws_camera_stream(opp, connection, msg):
     try:
         entity_id = msg["entity_id"]
         camera = _get_camera_from_entity_id(opp, entity_id)
-        camera_prefs = opp.data[DATA_CAMERA_PREFS].get(entity_id)
-
-        async with async_timeout.timeout(10):
-            source = await camera.stream_source()
-
-        if not source:
-            raise OpenPeerPowerError(
-                f"{camera.entity_id} does not support play stream service"
-            )
-
-        fmt = msg["format"]
-        url = request_stream(
-            opp, source, fmt=fmt, keepalive=camera_prefs.preload_stream
-        )
+        url = await _async_stream_endpoint_url(opp, camera, fmt=msg["format"])
         connection.send_result(msg["id"], {"url": url})
     except OpenPeerPowerError as ex:
         _LOGGER.error("Error requesting stream: %s", ex)
@@ -641,6 +638,8 @@ async def async_handle_snapshot_service(camera, service):
 
     def _write_image(to_file, image_data):
         """Executor helper to write image."""
+        if not os.path.exists(os.path.dirname(to_file)):
+            os.makedirs(os.path.dirname(to_file), exist_ok=True)
         with open(to_file, "wb") as img_file:
             img_file.write(image_data)
 
@@ -652,37 +651,76 @@ async def async_handle_snapshot_service(camera, service):
 
 async def async_handle_play_stream_service(camera, service_call):
     """Handle play stream services calls."""
-    async with async_timeout.timeout(10):
-        source = await camera.stream_source()
+    fmt = service_call.data[ATTR_FORMAT]
+    url = await _async_stream_endpoint_url(camera.opp, camera, fmt)
 
-    if not source:
+    opp = camera.opp
+    data = {
+        ATTR_MEDIA_CONTENT_ID: f"{get_url(opp)}{url}",
+        ATTR_MEDIA_CONTENT_TYPE: FORMAT_CONTENT_TYPE[fmt],
+    }
+
+    # It is required to send a different payload for cast media players
+    entity_ids = service_call.data[ATTR_MEDIA_PLAYER]
+    cast_entity_ids = [
+        entity
+        for entity, source in entity_sources(opp).items()
+        if entity in entity_ids and source["domain"] == "cast"
+    ]
+    other_entity_ids = list(set(entity_ids) - set(cast_entity_ids))
+
+    if cast_entity_ids:
+        await opp.services.async_call(
+            DOMAIN_MP,
+            SERVICE_PLAY_MEDIA,
+            {
+                ATTR_ENTITY_ID: cast_entity_ids,
+                **data,
+                ATTR_MEDIA_EXTRA: {
+                    "stream_type": "LIVE",
+                    "media_info": {
+                        "hlsVideoSegmentFormat": "fmp4",
+                    },
+                },
+            },
+            blocking=True,
+            context=service_call.context,
+        )
+
+    if other_entity_ids:
+        await opp.services.async_call(
+            DOMAIN_MP,
+            SERVICE_PLAY_MEDIA,
+            {
+                ATTR_ENTITY_ID: other_entity_ids,
+                **data,
+            },
+            blocking=True,
+            context=service_call.context,
+        )
+
+
+async def _async_stream_endpoint_url(opp, camera, fmt):
+    stream = await camera.create_stream()
+    if not stream:
         raise OpenPeerPowerError(
             f"{camera.entity_id} does not support play stream service"
         )
 
-    opp = camera.opp
+    # Update keepalive setting which manages idle shutdown
     camera_prefs = opp.data[DATA_CAMERA_PREFS].get(camera.entity_id)
-    fmt = service_call.data[ATTR_FORMAT]
-    entity_ids = service_call.data[ATTR_MEDIA_PLAYER]
+    stream.keepalive = camera_prefs.preload_stream
 
-    url = request_stream(opp, source, fmt=fmt, keepalive=camera_prefs.preload_stream)
-    data = {
-        ATTR_ENTITY_ID: entity_ids,
-        ATTR_MEDIA_CONTENT_ID: f"{opp.config.api.base_url}{url}",
-        ATTR_MEDIA_CONTENT_TYPE: FORMAT_CONTENT_TYPE[fmt],
-    }
-
-    await opp.services.async_call(
-        DOMAIN_MP, SERVICE_PLAY_MEDIA, data, blocking=True, context=service_call.context
-    )
+    stream.add_provider(fmt)
+    stream.start()
+    return stream.endpoint_url(fmt)
 
 
 async def async_handle_record_service(camera, call):
     """Handle stream recording service calls."""
-    async with async_timeout.timeout(10):
-        source = await camera.stream_source()
+    stream = await camera.create_stream()
 
-    if not source:
+    if not stream:
         raise OpenPeerPowerError(f"{camera.entity_id} does not support record service")
 
     opp = camera.opp
@@ -690,13 +728,6 @@ async def async_handle_record_service(camera, call):
     filename.opp = opp
     video_path = filename.async_render(variables={ATTR_ENTITY_ID: camera})
 
-    data = {
-        CONF_STREAM_SOURCE: source,
-        CONF_FILENAME: video_path,
-        CONF_DURATION: call.data[CONF_DURATION],
-        CONF_LOOKBACK: call.data[CONF_LOOKBACK],
-    }
-
-    await opp.services.async_call(
-        DOMAIN_STREAM, SERVICE_RECORD, data, blocking=True, context=call.context
+    await stream.async_record(
+        video_path, duration=call.data[CONF_DURATION], lookback=call.data[CONF_LOOKBACK]
     )
