@@ -6,8 +6,9 @@ import logging
 import aiohttp
 import async_timeout
 import attr
-from opp_nabucasa import Cloud, auth, thingtalk
-from opp_nabucasa.const import STATE_DISCONNECTED
+from opp_net import Cloud, auth, thingtalk
+from opp_net.const import STATE_DISCONNECTED
+from opp_net.voice import MAP_VOICE
 import voluptuous as vol
 
 from openpeerpower.components import websocket_api
@@ -19,15 +20,25 @@ from openpeerpower.components.google_assistant import helpers as google_helpers
 from openpeerpower.components.http import OpenPeerPowerView
 from openpeerpower.components.http.data_validator import RequestDataValidator
 from openpeerpower.components.websocket_api import const as ws_const
+from openpeerpower.const import (
+    HTTP_BAD_GATEWAY,
+    HTTP_BAD_REQUEST,
+    HTTP_INTERNAL_SERVER_ERROR,
+    HTTP_OK,
+    HTTP_UNAUTHORIZED,
+)
 from openpeerpower.core import callback
 
 from .const import (
     DOMAIN,
+    PREF_ALEXA_DEFAULT_EXPOSE,
     PREF_ALEXA_REPORT_STATE,
     PREF_ENABLE_ALEXA,
     PREF_ENABLE_GOOGLE,
+    PREF_GOOGLE_DEFAULT_EXPOSE,
     PREF_GOOGLE_REPORT_STATE,
     PREF_GOOGLE_SECURE_DEVICES_PIN,
+    PREF_TTS_DEFAULT_VOICE,
     REQUEST_TIMEOUT,
     InvalidTrustedNetworks,
     InvalidTrustedProxies,
@@ -63,13 +74,21 @@ SCHEMA_WS_HOOK_DELETE = websocket_api.BASE_COMMAND_MESSAGE_SCHEMA.extend(
 
 _CLOUD_ERRORS = {
     InvalidTrustedNetworks: (
-        500,
+        HTTP_INTERNAL_SERVER_ERROR,
         "Remote UI not compatible with 127.0.0.1/::1 as a trusted network.",
     ),
     InvalidTrustedProxies: (
-        500,
+        HTTP_INTERNAL_SERVER_ERROR,
         "Remote UI not compatible with 127.0.0.1/::1 as trusted proxies.",
     ),
+    asyncio.TimeoutError: (
+        HTTP_BAD_GATEWAY,
+        "Unable to reach the Open Peer Power cloud.",
+    ),
+    aiohttp.ClientError: (
+        HTTP_INTERNAL_SERVER_ERROR,
+        "Error making internal request",
+    ),
 }
 
 
@@ -98,6 +117,7 @@ async def async_setup(opp):
     async_register_command(alexa_sync)
 
     async_register_command(thingtalk_convert)
+    async_register_command(tts_info)
 
     opp.http.register_view(GoogleActionsSyncView)
     opp.http.register_view(CloudLoginView)
@@ -108,13 +128,17 @@ async def async_setup(opp):
 
     _CLOUD_ERRORS.update(
         {
-            auth.UserNotFound: (400, "User does not exist."),
-            auth.UserNotConfirmed: (400, "Email not confirmed."),
-            auth.UserExists: (400, "An account with the given email already exists."),
-            auth.Unauthenticated: (401, "Authentication failed."),
-            auth.PasswordChangeRequired: (400, "Password change required."),
-            asyncio.TimeoutError: (502, "Unable to reach the Open Peer Power cloud."),
-            aiohttp.ClientError: (500, "Error making internal request"),
+            auth.UserNotFound: (HTTP_BAD_REQUEST, "User does not exist."),
+            auth.UserNotConfirmed: (HTTP_BAD_REQUEST, "Email not confirmed."),
+            auth.UserExists: (
+                HTTP_BAD_REQUEST,
+                "An account with the given email already exists.",
+            ),
+            auth.Unauthenticated: (HTTP_UNAUTHORIZED, "Authentication failed."),
+            auth.PasswordChangeRequired: (
+                HTTP_BAD_REQUEST,
+                "Password change required.",
+            ),
         }
     )
 
@@ -156,10 +180,17 @@ def _ws_handle_cloud_errors(handler):
 
 def _process_cloud_exception(exc, where):
     """Process a cloud exception."""
-    err_info = _CLOUD_ERRORS.get(exc.__class__)
+    err_info = None
+
+    for err, value_info in _CLOUD_ERRORS.items():
+        if isinstance(exc, err):
+            err_info = value_info
+            break
+
     if err_info is None:
         _LOGGER.exception("Unexpected error processing request for %s", where)
-        err_info = (502, f"Unexpected error: {exc}")
+        err_info = (HTTP_BAD_GATEWAY, f"Unexpected error: {exc}")
+
     return err_info
 
 
@@ -236,9 +267,7 @@ class CloudRegisterView(OpenPeerPowerView):
         cloud = opp.data[DOMAIN]
 
         with async_timeout.timeout(REQUEST_TIMEOUT):
-            await opp.async_add_job(
-                cloud.auth.register, data["email"], data["password"]
-            )
+            await cloud.auth.async_register(data["email"], data["password"])
 
         return self.json_message("ok")
 
@@ -257,7 +286,7 @@ class CloudResendConfirmView(OpenPeerPowerView):
         cloud = opp.data[DOMAIN]
 
         with async_timeout.timeout(REQUEST_TIMEOUT):
-            await opp.async_add_job(cloud.auth.resend_email_confirm, data["email"])
+            await cloud.auth.async_resend_email_confirm(data["email"])
 
         return self.json_message("ok")
 
@@ -276,7 +305,7 @@ class CloudForgotPasswordView(OpenPeerPowerView):
         cloud = opp.data[DOMAIN]
 
         with async_timeout.timeout(REQUEST_TIMEOUT):
-            await opp.async_add_job(cloud.auth.forgot_password, data["email"])
+            await cloud.auth.async_forgot_password(data["email"])
 
         return self.json_message("ok")
 
@@ -323,7 +352,7 @@ async def websocket_subscription(opp, connection, msg):
     with async_timeout.timeout(REQUEST_TIMEOUT):
         response = await cloud.fetch_subscription_info()
 
-    if response.status != 200:
+    if response.status != HTTP_OK:
         connection.send_message(
             websocket_api.error_message(
                 msg["id"], "request_failed", "Failed to request subscription"
@@ -336,7 +365,7 @@ async def websocket_subscription(opp, connection, msg):
     # In that case, let's refresh and reconnect
     if data.get("provider") and not cloud.is_connected:
         _LOGGER.debug("Found disconnected account with valid subscriotion, connecting")
-        await opp.async_add_executor_job(cloud.auth.renew_access_token)
+        await cloud.auth.async_renew_access_token()
 
         # Cancel reconnect in progress
         if cloud.iot.state != STATE_DISCONNECTED:
@@ -356,7 +385,12 @@ async def websocket_subscription(opp, connection, msg):
         vol.Optional(PREF_ENABLE_ALEXA): bool,
         vol.Optional(PREF_ALEXA_REPORT_STATE): bool,
         vol.Optional(PREF_GOOGLE_REPORT_STATE): bool,
+        vol.Optional(PREF_ALEXA_DEFAULT_EXPOSE): [str],
+        vol.Optional(PREF_GOOGLE_DEFAULT_EXPOSE): [str],
         vol.Optional(PREF_GOOGLE_SECURE_DEVICES_PIN): vol.Any(None, str),
+        vol.Optional(PREF_TTS_DEFAULT_VOICE): vol.All(
+            vol.Coerce(tuple), vol.In(MAP_VOICE)
+        ),
     }
 )
 async def websocket_update_prefs(opp, connection, msg):
@@ -369,9 +403,10 @@ async def websocket_update_prefs(opp, connection, msg):
 
     # If we turn alexa linking on, validate that we can fetch access token
     if changes.get(PREF_ALEXA_REPORT_STATE):
+        alexa_config = await cloud.client.get_alexa_config()
         try:
             with async_timeout.timeout(10):
-                await cloud.client.alexa_config.async_get_access_token()
+                await alexa_config.async_get_access_token()
         except asyncio.TimeoutError:
             connection.send_error(
                 msg["id"], "alexa_timeout", "Timeout validating Alexa access token."
@@ -484,7 +519,7 @@ async def google_assistant_list(opp, connection, msg):
             {
                 "entity_id": entity.entity_id,
                 "traits": [trait.name for trait in entity.traits()],
-                "might_2fa": entity.might_2fa(),
+                "might_2fa": entity.might_2fa_traits(),
             }
         )
 
@@ -499,7 +534,7 @@ async def google_assistant_list(opp, connection, msg):
     {
         "type": "cloud/google_assistant/entities/update",
         "entity_id": str,
-        vol.Optional("should_expose"): bool,
+        vol.Optional("should_expose"): vol.Any(None, bool),
         vol.Optional("override_name"): str,
         vol.Optional("aliases"): [str],
         vol.Optional("disable_2fa"): bool,
@@ -527,7 +562,8 @@ async def google_assistant_update(opp, connection, msg):
 async def alexa_list(opp, connection, msg):
     """List all alexa entities."""
     cloud = opp.data[DOMAIN]
-    entities = alexa_entities.async_get_entities(opp, cloud.client.alexa_config)
+    alexa_config = await cloud.client.get_alexa_config()
+    entities = alexa_entities.async_get_entities(opp, alexa_config)
 
     result = []
 
@@ -551,7 +587,7 @@ async def alexa_list(opp, connection, msg):
     {
         "type": "cloud/alexa/entities/update",
         "entity_id": str,
-        vol.Optional("should_expose"): bool,
+        vol.Optional("should_expose"): vol.Any(None, bool),
     }
 )
 async def alexa_update(opp, connection, msg):
@@ -575,10 +611,11 @@ async def alexa_update(opp, connection, msg):
 async def alexa_sync(opp, connection, msg):
     """Sync with Alexa."""
     cloud = opp.data[DOMAIN]
+    alexa_config = await cloud.client.get_alexa_config()
 
     with async_timeout.timeout(10):
         try:
-            success = await cloud.client.alexa_config.async_sync_entities()
+            success = await alexa_config.async_sync_entities()
         except alexa_errors.NoTokenAvailable:
             connection.send_error(
                 msg["id"],
@@ -606,3 +643,11 @@ async def thingtalk_convert(opp, connection, msg):
             )
         except thingtalk.ThingTalkConversionError as err:
             connection.send_error(msg["id"], ws_const.ERR_UNKNOWN_ERROR, str(err))
+
+
+@websocket_api.websocket_command({"type": "cloud/tts/info"})
+def tts_info(opp, connection, msg):
+    """Fetch available tts info."""
+    connection.send_result(
+        msg["id"], {"languages": [(lang, gender.value) for lang, gender in MAP_VOICE]}
+    )
