@@ -1,8 +1,14 @@
 """Provide configuration end points for Groups."""
-from openpeerpower.components.group import DOMAIN, GROUP_SCHEMA
+from openpeerpower.components.group import (
+    DOMAIN,
+    GROUP_SCHEMA,
+    GroupIntegrationRegistry,
+)
 from openpeerpower.config import GROUP_CONFIG_PATH
 from openpeerpower.const import SERVICE_RELOAD
+from openpeerpower.core import callback
 import openpeerpower.helpers.config_validation as cv
+from openpeerpower.helpers.typing import OpenPeerPowerType
 
 from . import EditKeyBasedConfigView
 
@@ -25,3 +31,11 @@ async def async_setup(opp):
         )
     )
     return True
+
+
+@callback
+def async_describe_on_off_states(
+    opp: OpenPeerPowerType, registry: GroupIntegrationRegistry
+) -> None:
+    """Describe group on off states."""
+    return
