@@ -1,5 +1,6 @@
 """Provide configuration end points for scripts."""
 from openpeerpower.components.script import DOMAIN, SCRIPT_ENTRY_SCHEMA
+from openpeerpower.components.script.config import async_validate_config_item
 from openpeerpower.config import SCRIPT_CONFIG_PATH
 from openpeerpower.const import SERVICE_RELOAD
 import openpeerpower.helpers.config_validation as cv
@@ -16,12 +17,13 @@ async def async_setup(opp):
 
     opp.http.register_view(
         EditKeyBasedConfigView(
-            "script",
+            DOMAIN,
             "config",
             SCRIPT_CONFIG_PATH,
             cv.slug,
             SCRIPT_ENTRY_SCHEMA,
             post_write_hook=hook,
+            data_validator=async_validate_config_item,
         )
     )
     return True
