@@ -14,19 +14,19 @@ from yarl import URL
 from openpeerpower.components import websocket_api
 from openpeerpower.components.http.view import OpenPeerPowerView
 from openpeerpower.config import async_opp_config_yaml
-from openpeerpower.const import CONF_NAME, EVENT_THEMES_UPDATED
+from openpeerpower.const import CONF_MODE, CONF_NAME, EVENT_THEMES_UPDATED
 from openpeerpower.core import callback
 from openpeerpower.helpers import service
 import openpeerpower.helpers.config_validation as cv
 from openpeerpower.helpers.translation import async_get_translations
-from openpeerpower.loader import bind_opp
+from openpeerpower.loader import async_get_integration, bind_opp
 
 from .storage import async_setup_frontend_storage
 
 # mypy: allow-untyped-defs, no-check-untyped-defs
 
 # Fix mimetypes for borked Windows machines
-# https://github.com/open-peer-power/open-peer-power-polymer/issues/3336
+# https://github.com/openpeerpower/frontend/issues/3336
 mimetypes.add_type("text/css", ".css")
 mimetypes.add_type("application/javascript", ".js")
 
@@ -50,8 +50,8 @@ MANIFEST_JSON = {
     "display": "standalone",
     "icons": [
         {
-            "src": "/static/icons/favicon-{size}x{size}.png".format(size=size),
-            "sizes": "{size}x{size}".format(size=size),
+            "src": f"/static/icons/favicon-{size}x{size}.png",
+            "sizes": f"{size}x{size}",
             "type": "image/png",
             "purpose": "maskable any",
         }
@@ -59,7 +59,7 @@ MANIFEST_JSON = {
     ],
     "lang": "en-US",
     "name": "Open Peer Power",
-    "short_name": "Assistant",
+    "short_name": "OPP",
     "start_url": "/?homescreen=1",
     "theme_color": DEFAULT_THEME_COLOR,
     "prefer_related_applications": True,
@@ -70,13 +70,18 @@ MANIFEST_JSON = {
 
 DATA_PANELS = "frontend_panels"
 DATA_JS_VERSION = "frontend_js_version"
-DATA_EXTRA_HTML_URL = "frontend_extra_html_url"
-DATA_EXTRA_HTML_URL_ES5 = "frontend_extra_html_url_es5"
 DATA_EXTRA_MODULE_URL = "frontend_extra_module_url"
 DATA_EXTRA_JS_URL_ES5 = "frontend_extra_js_url_es5"
+
+THEMES_STORAGE_KEY = f"{DOMAIN}_theme"
+THEMES_STORAGE_VERSION = 1
+THEMES_SAVE_DELAY = 60
+DATA_THEMES_STORE = "frontend_themes_store"
 DATA_THEMES = "frontend_themes"
 DATA_DEFAULT_THEME = "frontend_default_theme"
+DATA_DEFAULT_DARK_THEME = "frontend_default_dark_theme"
 DEFAULT_THEME = "default"
+VALUE_NO_THEME = "none"
 
 PRIMARY_COLOR = "primary-color"
 
@@ -90,7 +95,6 @@ CONFIG_SCHEMA = vol.Schema(
                 vol.Optional(CONF_THEMES): vol.Schema(
                     {cv.string: {cv.string: cv.string}}
                 ),
-                vol.Optional(CONF_EXTRA_HTML_URL): vol.All(cv.ensure_list, [cv.string]),
                 vol.Optional(CONF_EXTRA_MODULE_URL): vol.All(
                     cv.ensure_list, [cv.string]
                 ),
@@ -98,9 +102,10 @@ CONFIG_SCHEMA = vol.Schema(
                     cv.ensure_list, [cv.string]
                 ),
                 # We no longer use these options.
+                vol.Optional(CONF_EXTRA_HTML_URL): cv.match_all,
                 vol.Optional(CONF_EXTRA_HTML_URL_ES5): cv.match_all,
                 vol.Optional(CONF_JS_VERSION): cv.match_all,
-            }
+            },
         )
     },
     extra=vol.ALLOW_EXTRA,
@@ -171,6 +176,8 @@ def async_register_built_in_panel(
     frontend_url_path=None,
     config=None,
     require_admin=False,
+    *,
+    update=False,
 ):
     """Register a built-in panel."""
     panel = Panel(
@@ -184,8 +191,8 @@ def async_register_built_in_panel(
 
     panels = opp.data.setdefault(DATA_PANELS, {})
 
-    if panel.frontend_url_path in panels:
-        _LOGGER.warning("Overwriting integration %s", panel.frontend_url_path)
+    if not update and panel.frontend_url_path in panels:
+        raise ValueError(f"Overwriting panel {panel.frontend_url_path}")
 
     panels[panel.frontend_url_path] = panel
 
@@ -204,17 +211,6 @@ def async_remove_panel(opp, frontend_url_path):
     opp.bus.async_fire(EVENT_PANELS_UPDATED)
 
 
-@bind_opp
-@callback
-def add_extra_html_url(opp, url, es5=False):
-    """Register extra html url to load."""
-    key = DATA_EXTRA_HTML_URL_ES5 if es5 else DATA_EXTRA_HTML_URL
-    url_set = opp.data.get(key)
-    if url_set is None:
-        url_set = opp.data[key] = set()
-    url_set.add(url)
-
-
 def add_extra_js_url(opp, url, es5=False):
     """Register extra js or module url to load."""
     key = DATA_EXTRA_JS_URL_ES5 if es5 else DATA_EXTRA_MODULE_URL
@@ -246,10 +242,18 @@ async def async_setup(opp, config):
     opp.components.websocket_api.async_register_command(websocket_get_panels)
     opp.components.websocket_api.async_register_command(websocket_get_themes)
     opp.components.websocket_api.async_register_command(websocket_get_translations)
+    opp.components.websocket_api.async_register_command(websocket_get_version)
     opp.http.register_view(ManifestJSONView)
 
     conf = config.get(DOMAIN, {})
 
+    for key in (CONF_EXTRA_HTML_URL, CONF_EXTRA_HTML_URL_ES5, CONF_JS_VERSION):
+        if key in conf:
+            _LOGGER.error(
+                "Please remove %s from your frontend config. It is no longer supported",
+                key,
+            )
+
     repo_path = conf.get(CONF_FRONTEND_REPO)
     is_dev = repo_path is not None
     root_path = _frontend_root(repo_path)
@@ -257,16 +261,20 @@ async def async_setup(opp, config):
     for path, should_cache in (
         ("service_worker.js", False),
         ("robots.txt", False),
-        ("onboarding.html", True),
-        ("static", True),
-        ("frontend_latest", True),
-        ("frontend_es5", True),
+        ("onboarding.html", not is_dev),
+        ("static", not is_dev),
+        ("frontend_latest", not is_dev),
+        ("frontend_es5", not is_dev),
     ):
         opp.http.register_static_path(f"/{path}", str(root_path / path), should_cache)
 
     opp.http.register_static_path(
         "/auth/authorize", str(root_path / "authorize.html"), False
     )
+    # https://wicg.github.io/change-password-url/
+    opp.http.register_redirect(
+        "/.well-known/change-password", "/profile", redirect_exc=web.HTTPFound
+    )
 
     local = opp.config.path("www")
     if os.path.isdir(local):
@@ -274,13 +282,17 @@ async def async_setup(opp, config):
 
     opp.http.app.router.register_resource(IndexView(repo_path, opp))
 
-    for panel in ("kiosk", "states", "profile"):
-        async_register_built_in_panel(opp, panel)
+    async_register_built_in_panel(opp, "profile")
 
     # To smooth transition to new urls, add redirects to new urls of dev tools
     # Added June 27, 2019. Can be removed in 2021.
-    for panel in ("event", "info", "service", "state", "template", "mqtt"):
+    for panel in ("event", "service", "state", "template"):
         opp.http.register_redirect(f"/dev-{panel}", f"/developer-tools/{panel}")
+    for panel in ("logs", "info", "mqtt"):
+        # Can be removed in 2021.
+        opp.http.register_redirect(f"/dev-{panel}", f"/config/{panel}")
+        # Added June 20 2020. Can be removed in 2022.
+        opp.http.register_redirect(f"/developer-tools/{panel}", f"/config/{panel}")
 
     async_register_built_in_panel(
         opp,
@@ -290,12 +302,6 @@ async def async_setup(opp, config):
         sidebar_icon="opp:hammer",
     )
 
-    if DATA_EXTRA_HTML_URL not in opp.data:
-        opp.data[DATA_EXTRA_HTML_URL] = set()
-
-    for url in conf.get(CONF_EXTRA_HTML_URL, []):
-        add_extra_html_url(opp, url, False)
-
     if DATA_EXTRA_MODULE_URL not in opp.data:
         opp.data[DATA_EXTRA_MODULE_URL] = set()
 
@@ -308,17 +314,31 @@ async def async_setup(opp, config):
     for url in conf.get(CONF_EXTRA_JS_URL_ES5, []):
         add_extra_js_url(opp, url, True)
 
-    _async_setup_themes(opp, conf.get(CONF_THEMES))
+    await _async_setup_themes(opp, conf.get(CONF_THEMES))
 
     return True
 
 
-@callback
-def _async_setup_themes(opp, themes):
+async def _async_setup_themes(opp, themes):
     """Set up themes data and services."""
-    opp.data[DATA_DEFAULT_THEME] = DEFAULT_THEME
     opp.data[DATA_THEMES] = themes or {}
 
+    store = opp.data[DATA_THEMES_STORE] = opp.helpers.storage.Store(
+        THEMES_STORAGE_VERSION, THEMES_STORAGE_KEY
+    )
+
+    theme_data = await store.async_load() or {}
+    theme_name = theme_data.get(DATA_DEFAULT_THEME, DEFAULT_THEME)
+    dark_theme_name = theme_data.get(DATA_DEFAULT_DARK_THEME)
+
+    if theme_name == DEFAULT_THEME or theme_name in opp.data[DATA_THEMES]:
+        opp.data[DATA_DEFAULT_THEME] = theme_name
+    else:
+        opp.data[DATA_DEFAULT_THEME] = DEFAULT_THEME
+
+    if dark_theme_name == DEFAULT_THEME or dark_theme_name in opp.data[DATA_THEMES]:
+        opp.data[DATA_DEFAULT_DARK_THEME] = dark_theme_name
+
     @callback
     def update_theme_and_fire_event():
         """Update theme_color in manifest."""
@@ -335,14 +355,35 @@ def _async_setup_themes(opp, themes):
     @callback
     def set_theme(call):
         """Set backend-preferred theme."""
-        data = call.data
-        name = data[CONF_NAME]
-        if name == DEFAULT_THEME or name in opp.data[DATA_THEMES]:
-            _LOGGER.info("Theme %s set as default", name)
-            opp.data[DATA_DEFAULT_THEME] = name
-            update_theme_and_fire_event()
+        name = call.data[CONF_NAME]
+        mode = call.data.get("mode", "light")
+
+        if (
+            name not in (DEFAULT_THEME, VALUE_NO_THEME)
+            and name not in opp.data[DATA_THEMES]
+        ):
+            _LOGGER.warning("Theme %s not found", name)
+            return
+
+        light_mode = mode == "light"
+
+        theme_key = DATA_DEFAULT_THEME if light_mode else DATA_DEFAULT_DARK_THEME
+
+        if name == VALUE_NO_THEME:
+            to_set = DEFAULT_THEME if light_mode else None
         else:
-            _LOGGER.warning("Theme %s is not defined.", name)
+            _LOGGER.info("Theme %s set as default %s theme", name, mode)
+            to_set = name
+
+        opp.data[theme_key] = to_set
+        store.async_delay_save(
+            lambda: {
+                DATA_DEFAULT_THEME: opp.data[DATA_DEFAULT_THEME],
+                DATA_DEFAULT_DARK_THEME: opp.data.get(DATA_DEFAULT_DARK_THEME),
+            },
+            THEMES_SAVE_DELAY,
+        )
+        update_theme_and_fire_event()
 
     async def reload_themes(_):
         """Reload themes."""
@@ -351,6 +392,11 @@ def _async_setup_themes(opp, themes):
         opp.data[DATA_THEMES] = new_themes
         if opp.data[DATA_DEFAULT_THEME] not in new_themes:
             opp.data[DATA_DEFAULT_THEME] = DEFAULT_THEME
+        if (
+            opp.data.get(DATA_DEFAULT_DARK_THEME)
+            and opp.data.get(DATA_DEFAULT_DARK_THEME) not in new_themes
+        ):
+            opp.data[DATA_DEFAULT_DARK_THEME] = None
         update_theme_and_fire_event()
 
     service.async_register_admin_service(
@@ -358,7 +404,12 @@ def _async_setup_themes(opp, themes):
         DOMAIN,
         SERVICE_SET_THEME,
         set_theme,
-        vol.Schema({vol.Required(CONF_NAME): cv.string}),
+        vol.Schema(
+            {
+                vol.Required(CONF_NAME): cv.string,
+                vol.Optional(CONF_MODE): vol.Any("dark", "light"),
+            }
+        ),
     )
 
     service.async_register_admin_service(
@@ -420,11 +471,9 @@ class IndexView(web_urldispatcher.AbstractResource):
 
     def freeze(self) -> None:
         """Freeze the resource."""
-        pass
 
     def raw_match(self, path: str) -> bool:
         """Perform a raw match against path."""
-        pass
 
     def get_template(self):
         """Get template."""
@@ -454,7 +503,6 @@ class IndexView(web_urldispatcher.AbstractResource):
         return web.Response(
             text=template.render(
                 theme_color=MANIFEST_JSON["theme_color"],
-                extra_urls=opp.data[DATA_EXTRA_HTML_URL],
                 extra_modules=opp.data[DATA_EXTRA_MODULE_URL],
                 extra_js_es5=opp.data[DATA_EXTRA_JS_URL_ES5],
             ),
@@ -487,10 +535,7 @@ class ManifestJSONView(OpenPeerPowerView):
 @callback
 @websocket_api.websocket_command({"type": "get_panels"})
 def websocket_get_panels(opp, connection, msg):
-    """Handle get panels command.
-
-    Async friendly.
-    """
+    """Handle get panels command."""
     user_is_admin = connection.user.is_admin
     panels = {
         panel_key: panel.to_response()
@@ -504,10 +549,7 @@ def websocket_get_panels(opp, connection, msg):
 @callback
 @websocket_api.websocket_command({"type": "frontend/get_themes"})
 def websocket_get_themes(opp, connection, msg):
-    """Handle get themes command.
-
-    Async friendly.
-    """
+    """Handle get themes command."""
     if opp.config.safe_mode:
         connection.send_message(
             websocket_api.result_message(
@@ -516,7 +558,7 @@ def websocket_get_themes(opp, connection, msg):
                     "themes": {
                         "safe_mode": {
                             "primary-color": "#db4437",
-                            "accent-color": "#eeee02",
+                            "accent-color": "#ffca28",
                         }
                     },
                     "default_theme": "safe_mode",
@@ -531,21 +573,49 @@ def websocket_get_themes(opp, connection, msg):
             {
                 "themes": opp.data[DATA_THEMES],
                 "default_theme": opp.data[DATA_DEFAULT_THEME],
+                "default_dark_theme": opp.data.get(DATA_DEFAULT_DARK_THEME),
             },
         )
     )
 
 
 @websocket_api.websocket_command(
-    {"type": "frontend/get_translations", vol.Required("language"): str}
+    {
+        "type": "frontend/get_translations",
+        vol.Required("language"): str,
+        vol.Required("category"): str,
+        vol.Optional("integration"): str,
+        vol.Optional("config_flow"): bool,
+    }
 )
 @websocket_api.async_response
 async def websocket_get_translations(opp, connection, msg):
-    """Handle get translations command.
-
-    Async friendly.
-    """
-    resources = await async_get_translations(opp, msg["language"])
+    """Handle get translations command."""
+    resources = await async_get_translations(
+        opp,
+        msg["language"],
+        msg["category"],
+        msg.get("integration"),
+        msg.get("config_flow"),
+    )
     connection.send_message(
         websocket_api.result_message(msg["id"], {"resources": resources})
     )
+
+
+@websocket_api.websocket_command({"type": "frontend/get_version"})
+@websocket_api.async_response
+async def websocket_get_version(opp, connection, msg):
+    """Handle get version command."""
+    integration = await async_get_integration(opp, "frontend")
+
+    frontend = None
+
+    for req in integration.requirements:
+        if req.startswith("open-peer-power-frontend=="):
+            frontend = req.split("==", 1)[1]
+
+    if frontend is None:
+        connection.send_error(msg["id"], "unknown_version", "Version not found")
+    else:
+        connection.send_result(msg["id"], {"version": frontend})
