@@ -1,71 +1,67 @@
 """Provide the functionality to group entities."""
+from abc import abstractmethod
 import asyncio
+from contextvars import ContextVar
 import logging
-from typing import Any, Iterable, List, Optional, cast
+from typing import Any, Dict, Iterable, List, Optional, Set, cast
 
 import voluptuous as vol
 
-from openpeerpower import core as op
+from openpeerpower import core as ha
 from openpeerpower.const import (
     ATTR_ASSUMED_STATE,
     ATTR_ENTITY_ID,
     ATTR_ICON,
     ATTR_NAME,
+    CONF_ENTITIES,
     CONF_ICON,
     CONF_NAME,
     ENTITY_MATCH_ALL,
     ENTITY_MATCH_NONE,
+    EVENT_OPENPEERPOWER_START,
     SERVICE_RELOAD,
-    STATE_CLOSED,
-    STATE_HOME,
-    STATE_LOCKED,
-    STATE_NOT_HOME,
     STATE_OFF,
-    STATE_OK,
     STATE_ON,
-    STATE_OPEN,
-    STATE_PROBLEM,
-    STATE_UNKNOWN,
-    STATE_UNLOCKED,
 )
-from openpeerpower.core import callback
+from openpeerpower.core import CoreState, callback, split_entity_id
 import openpeerpower.helpers.config_validation as cv
-from openpeerpower.helpers.config_validation import make_entity_service_schema
 from openpeerpower.helpers.entity import Entity, async_generate_entity_id
 from openpeerpower.helpers.entity_component import EntityComponent
-from openpeerpower.helpers.event import async_track_state_change
+from openpeerpower.helpers.event import async_track_state_change_event
+from openpeerpower.helpers.integration_platform import (
+    async_process_integration_platforms,
+)
+from openpeerpower.helpers.reload import async_reload_integration_platforms
 from openpeerpower.helpers.typing import OpenPeerPowerType
 from openpeerpower.loader import bind_opp
 
 # mypy: allow-untyped-calls, allow-untyped-defs, no-check-untyped-defs
 
 DOMAIN = "group"
+GROUP_ORDER = "group_order"
 
 ENTITY_ID_FORMAT = DOMAIN + ".{}"
 
-CONF_ENTITIES = "entities"
-CONF_VIEW = "view"
-CONF_CONTROL = "control"
 CONF_ALL = "all"
 
 ATTR_ADD_ENTITIES = "add_entities"
 ATTR_AUTO = "auto"
-ATTR_CONTROL = "control"
 ATTR_ENTITIES = "entities"
 ATTR_OBJECT_ID = "object_id"
 ATTR_ORDER = "order"
-ATTR_VIEW = "view"
-ATTR_VISIBLE = "visible"
 ATTR_ALL = "all"
 
-SERVICE_SET_VISIBILITY = "set_visibility"
 SERVICE_SET = "set"
 SERVICE_REMOVE = "remove"
 
-CONTROL_TYPES = vol.In(["hidden", None])
+PLATFORMS = ["light", "cover", "notify"]
+
+REG_KEY = f"{DOMAIN}_registry"
 
 _LOGGER = logging.getLogger(__name__)
 
+current_domain: ContextVar[str] = ContextVar("current_domain")
+
 
 def _conf_preprocess(value):
     """Preprocess alternative configuration formats."""
@@ -76,18 +72,14 @@ def _conf_preprocess(value):
 
 
 GROUP_SCHEMA = vol.All(
-    cv.deprecated(CONF_CONTROL, invalidation_version="0.107.0"),
-    cv.deprecated(CONF_VIEW, invalidation_version="0.107.0"),
     vol.Schema(
         {
             vol.Optional(CONF_ENTITIES): vol.Any(cv.entity_ids, None),
-            CONF_VIEW: cv.boolean,
             CONF_NAME: cv.string,
             CONF_ICON: cv.icon,
-            CONF_CONTROL: CONTROL_TYPES,
             CONF_ALL: cv.boolean,
         }
-    ),
+    )
 )
 
 CONFIG_SCHEMA = vol.Schema(
@@ -95,35 +87,42 @@ CONFIG_SCHEMA = vol.Schema(
     extra=vol.ALLOW_EXTRA,
 )
 
-# List of ON/OFF state tuples for groupable states
-_GROUP_TYPES = [
-    (STATE_ON, STATE_OFF),
-    (STATE_HOME, STATE_NOT_HOME),
-    (STATE_OPEN, STATE_CLOSED),
-    (STATE_LOCKED, STATE_UNLOCKED),
-    (STATE_PROBLEM, STATE_OK),
-]
 
+class GroupIntegrationRegistry:
+    """Class to hold a registry of integrations."""
 
-def _get_group_on_off(state):
-    """Determine the group on/off states based on a state."""
-    for states in _GROUP_TYPES:
-        if state in states:
-            return states
+    on_off_mapping: Dict[str, str] = {STATE_ON: STATE_OFF}
+    off_on_mapping: Dict[str, str] = {STATE_OFF: STATE_ON}
+    on_states_by_domain: Dict[str, Set] = {}
+    exclude_domains: Set = set()
 
-    return None, None
+    def exclude_domain(self) -> None:
+        """Exclude the current domain."""
+        self.exclude_domains.add(current_domain.get())
+
+    def on_off_states(self, on_states: Set, off_state: str) -> None:
+        """Register on and off states for the current domain."""
+        for on_state in on_states:
+            if on_state not in self.on_off_mapping:
+                self.on_off_mapping[on_state] = off_state
+
+        if len(on_states) == 1 and off_state not in self.off_on_mapping:
+            self.off_on_mapping[off_state] = list(on_states)[0]
+
+        self.on_states_by_domain[current_domain.get()] = set(on_states)
 
 
 @bind_opp
 def is_on(opp, entity_id):
     """Test if the group state is in its ON-state."""
+    if REG_KEY not in opp.data:
+        # Integration not setup yet, it cannot be on
+        return False
+
     state = opp.states.get(entity_id)
 
-    if state:
-        group_on, _ = _get_group_on_off(state.state)
-
-        # If we found a group_type, compare to ON-state
-        return group_on is not None and state.state == group_on
+    if state is not None:
+        return state.state in opp.data[REG_KEY].on_off_mapping
 
     return False
 
@@ -146,7 +145,7 @@ def expand_entity_ids(opp: OpenPeerPowerType, entity_ids: Iterable[Any]) -> List
 
         try:
             # If entity_id points at a group, expand it
-            domain, _ = op.split_entity_id(entity_id)
+            domain, _ = ha.split_entity_id(entity_id)
 
             if domain == DOMAIN:
                 child_entities = get_entity_ids(opp, entity_id)
@@ -187,7 +186,7 @@ def get_entity_ids(
     if not domain_filter:
         return cast(List[str], entity_ids)
 
-    domain_filter = domain_filter.lower() + "."
+    domain_filter = f"{domain_filter.lower()}."
 
     return [ent_id for ent_id in entity_ids if ent_id.startswith(domain_filter)]
 
@@ -217,6 +216,10 @@ async def async_setup(opp, config):
     if component is None:
         component = opp.data[DOMAIN] = EntityComponent(_LOGGER, DOMAIN, opp)
 
+    opp.data[REG_KEY] = GroupIntegrationRegistry()
+
+    await async_process_integration_platforms(opp, DOMAIN, _process_group_platform)
+
     await _async_process_config(opp, config, component)
 
     async def reload_service_handler(service):
@@ -230,6 +233,8 @@ async def async_setup(opp, config):
 
         await component.async_add_entities(auto)
 
+        await async_reload_integration_platforms(opp, DOMAIN, PLATFORMS)
+
     opp.services.async_register(
         DOMAIN, SERVICE_RELOAD, reload_service_handler, schema=vol.Schema({})
     )
@@ -244,7 +249,7 @@ async def async_setup(opp, config):
     async def groups_service_handler(service):
         """Handle dynamic group service functions."""
         object_id = service.data[ATTR_OBJECT_ID]
-        entity_id = ENTITY_ID_FORMAT.format(object_id)
+        entity_id = f"{DOMAIN}.{object_id}"
         group = component.get_entity(entity_id)
 
         # new group
@@ -257,7 +262,7 @@ async def async_setup(opp, config):
 
             extra_arg = {
                 attr: service.data[attr]
-                for attr in (ATTR_VISIBLE, ATTR_ICON, ATTR_VIEW, ATTR_CONTROL)
+                for attr in (ATTR_ICON,)
                 if service.data.get(attr) is not None
             }
 
@@ -293,28 +298,16 @@ async def async_setup(opp, config):
                 group.name = service.data[ATTR_NAME]
                 need_update = True
 
-            if ATTR_VISIBLE in service.data:
-                group.visible = service.data[ATTR_VISIBLE]
-                need_update = True
-
             if ATTR_ICON in service.data:
                 group.icon = service.data[ATTR_ICON]
                 need_update = True
 
-            if ATTR_CONTROL in service.data:
-                group.control = service.data[ATTR_CONTROL]
-                need_update = True
-
-            if ATTR_VIEW in service.data:
-                group.view = service.data[ATTR_VIEW]
-                need_update = True
-
             if ATTR_ALL in service.data:
                 group.mode = all if service.data[ATTR_ALL] else any
                 need_update = True
 
             if need_update:
-                await group.async_update_op_state()
+                group.async_write_op_state()
 
             return
 
@@ -327,22 +320,16 @@ async def async_setup(opp, config):
         SERVICE_SET,
         locked_service_handler,
         schema=vol.All(
-            cv.deprecated(ATTR_CONTROL, invalidation_version="0.107.0"),
-            cv.deprecated(ATTR_VIEW, invalidation_version="0.107.0"),
-            cv.deprecated(ATTR_VISIBLE, invalidation_version="0.107.0"),
             vol.Schema(
                 {
                     vol.Required(ATTR_OBJECT_ID): cv.slug,
                     vol.Optional(ATTR_NAME): cv.string,
-                    vol.Optional(ATTR_VIEW): cv.boolean,
                     vol.Optional(ATTR_ICON): cv.string,
-                    vol.Optional(ATTR_CONTROL): CONTROL_TYPES,
-                    vol.Optional(ATTR_VISIBLE): cv.boolean,
                     vol.Optional(ATTR_ALL): cv.boolean,
                     vol.Exclusive(ATTR_ENTITIES, "entities"): cv.entity_ids,
                     vol.Exclusive(ATTR_ADD_ENTITIES, "entities"): cv.entity_ids,
                 }
-            ),
+            )
         ),
     )
 
@@ -353,58 +340,83 @@ async def async_setup(opp, config):
         schema=vol.Schema({vol.Required(ATTR_OBJECT_ID): cv.slug}),
     )
 
-    async def visibility_service_handler(service):
-        """Change visibility of a group."""
-        visible = service.data.get(ATTR_VISIBLE)
-
-        _LOGGER.warning(
-            "The group.set_visibility service has been deprecated and will"
-            "be removed in Open Peer Power 0.107.0."
-        )
-
-        tasks = []
-        for group in await component.async_extract_from_service(
-            service, expand_group=False
-        ):
-            group.visible = visible
-            tasks.append(group.async_update_op_state())
-
-        if tasks:
-            await asyncio.wait(tasks)
-
-    opp.services.async_register(
-        DOMAIN,
-        SERVICE_SET_VISIBILITY,
-        visibility_service_handler,
-        schema=make_entity_service_schema({vol.Required(ATTR_VISIBLE): cv.boolean}),
-    )
-
     return True
 
 
+async def _process_group_platform(opp, domain, platform):
+    """Process a group platform."""
+    current_domain.set(domain)
+    platform.async_describe_on_off_states(opp, opp.data[REG_KEY])
+
+
 async def _async_process_config(opp, config, component):
     """Process group configuration."""
+    opp.data.setdefault(GROUP_ORDER, 0)
+
+    tasks = []
+
     for object_id, conf in config.get(DOMAIN, {}).items():
         name = conf.get(CONF_NAME, object_id)
         entity_ids = conf.get(CONF_ENTITIES) or []
         icon = conf.get(CONF_ICON)
-        view = conf.get(CONF_VIEW)
-        control = conf.get(CONF_CONTROL)
         mode = conf.get(CONF_ALL)
 
-        # Don't create tasks and await them all. The order is important as
-        # groups get a number based on creation order.
-        await Group.async_create_group(
-            opp,
-            name,
-            entity_ids,
-            icon=icon,
-            view=view,
-            control=control,
-            object_id=object_id,
-            mode=mode,
+        # We keep track of the order when we are creating the tasks
+        # in the same way that async_create_group does to make
+        # sure we use the same ordering system.  This overcomes
+        # the problem with concurrently creating the groups
+        tasks.append(
+            Group.async_create_group(
+                opp,
+                name,
+                entity_ids,
+                icon=icon,
+                object_id=object_id,
+                mode=mode,
+                order=opp.data[GROUP_ORDER],
+            )
         )
 
+        # Keep track of the group order without iterating
+        # every state in the state machine every time
+        # we setup a new group
+        opp.data[GROUP_ORDER] += 1
+
+    await asyncio.gather(*tasks)
+
+
+class GroupEntity(Entity):
+    """Representation of a Group of entities."""
+
+    @property
+    def should_poll(self) -> bool:
+        """Disable polling for group."""
+        return False
+
+    async def async_added_to_opp(self) -> None:
+        """Register listeners."""
+        assert self.opp is not None
+
+        async def _update_at_start(_):
+            await self.async_update()
+            self.async_write_op_state()
+
+        self.opp.bus.async_listen_once(EVENT_OPENPEERPOWER_START, _update_at_start)
+
+    async def async_defer_or_update_op_state(self) -> None:
+        """Only update once at start."""
+        assert self.opp is not None
+
+        if self.opp.state != CoreState.running:
+            return
+
+        await self.async_update()
+        self.async_write_op_state()
+
+    @abstractmethod
+    async def async_update(self) -> None:
+        """Abstract method to update the entity."""
+
 
 class Group(Entity):
     """Track a group of entity ids."""
@@ -414,10 +426,7 @@ class Group(Entity):
         opp,
         name,
         order=None,
-        visible=True,
         icon=None,
-        view=False,
-        control=None,
         user_defined=True,
         entity_ids=None,
         mode=None,
@@ -428,17 +437,12 @@ class Group(Entity):
         """
         self.opp = opp
         self._name = name
-        self._state = STATE_UNKNOWN
+        self._state = None
         self._icon = icon
-        self.view = view
-        if entity_ids:
-            self.tracking = tuple(ent_id.lower() for ent_id in entity_ids)
-        else:
-            self.tracking = tuple()
-        self.group_on = None
-        self.group_off = None
-        self.visible = visible
-        self.control = control
+        self._set_tracked(entity_ids)
+        self._on_off = None
+        self._assumed = None
+        self._on_states = None
         self.user_defined = user_defined
         self.mode = any
         if mode:
@@ -453,26 +457,15 @@ class Group(Entity):
         name,
         entity_ids=None,
         user_defined=True,
-        visible=True,
         icon=None,
-        view=False,
-        control=None,
         object_id=None,
         mode=None,
+        order=None,
     ):
         """Initialize a group."""
         return asyncio.run_coroutine_threadsafe(
             Group.async_create_group(
-                opp,
-                name,
-                entity_ids,
-                user_defined,
-                visible,
-                icon,
-                view,
-                control,
-                object_id,
-                mode,
+                opp, name, entity_ids, user_defined, icon, object_id, mode, order
             ),
             opp.loop,
         ).result()
@@ -483,25 +476,28 @@ class Group(Entity):
         name,
         entity_ids=None,
         user_defined=True,
-        visible=True,
         icon=None,
-        view=False,
-        control=None,
         object_id=None,
         mode=None,
+        order=None,
     ):
         """Initialize a group.
 
         This method must be run in the event loop.
         """
+        if order is None:
+            opp.data.setdefault(GROUP_ORDER, 0)
+            order = opp.data[GROUP_ORDER]
+            # Keep track of the group order without iterating
+            # every state in the state machine every time
+            # we setup a new group
+            opp.data[GROUP_ORDER] += 1
+
         group = Group(
             opp,
             name,
-            order=len(opp.states.async_entity_ids(DOMAIN)),
-            visible=visible,
+            order=order,
             icon=icon,
-            view=view,
-            control=control,
             user_defined=user_defined,
             entity_ids=entity_ids,
             mode=mode,
@@ -517,7 +513,7 @@ class Group(Entity):
         if component is None:
             component = opp.data[DOMAIN] = EntityComponent(_LOGGER, DOMAIN, opp)
 
-        await component.async_add_entities([group], True)
+        await component.async_add_entities([group])
 
         return group
 
@@ -551,23 +547,13 @@ class Group(Entity):
         """Set Icon for group."""
         self._icon = value
 
-    @property
-    def hidden(self):
-        """If group should be hidden or not."""
-        if self.visible and not self.view:
-            return False
-        return True
-
     @property
     def state_attributes(self):
         """Return the state attributes for the group."""
         data = {ATTR_ENTITY_ID: self.tracking, ATTR_ORDER: self._order}
         if not self.user_defined:
             data[ATTR_AUTO] = True
-        if self.view:
-            data[ATTR_VIEW] = True
-        if self.control:
-            data[ATTR_CONTROL] = self.control
+
         return data
 
     @property
@@ -586,25 +572,57 @@ class Group(Entity):
 
         This method must be run in the event loop.
         """
-        await self.async_stop()
-        self.tracking = tuple(ent_id.lower() for ent_id in entity_ids)
-        self.group_on, self.group_off = None, None
+        self._async_stop()
+        self._set_tracked(entity_ids)
+        self._reset_tracked_state()
+        self._async_start()
 
-        await self.async_update_op_state(True)
-        self.async_start()
+    def _set_tracked(self, entity_ids):
+        """Tuple of entities to be tracked."""
+        # tracking are the entities we want to track
+        # trackable are the entities we actually watch
+
+        if not entity_ids:
+            self.tracking = ()
+            self.trackable = ()
+            return
+
+        excluded_domains = self.opp.data[REG_KEY].exclude_domains
+
+        tracking = []
+        trackable = []
+        for ent_id in entity_ids:
+            ent_id_lower = ent_id.lower()
+            domain = split_entity_id(ent_id_lower)[0]
+            tracking.append(ent_id_lower)
+            if domain not in excluded_domains:
+                trackable.append(ent_id_lower)
+
+        self.trackable = tuple(trackable)
+        self.tracking = tuple(tracking)
 
     @callback
-    def async_start(self):
+    def _async_start(self, *_):
+        """Start tracking members and write state."""
+        self._reset_tracked_state()
+        self._async_start_tracking()
+        self.async_write_op_state()
+
+    @callback
+    def _async_start_tracking(self):
         """Start tracking members.
 
         This method must be run in the event loop.
         """
-        if self._async_unsub_state_changed is None:
-            self._async_unsub_state_changed = async_track_state_change(
-                self.opp, self.tracking, self._async_state_changed_listener
+        if self.trackable and self._async_unsub_state_changed is None:
+            self._async_unsub_state_changed = async_track_state_change_event(
+                self.opp, self.trackable, self._async_state_changed_listener
             )
 
-    async def async_stop(self):
+        self._async_update_group_state()
+
+    @callback
+    def _async_stop(self):
         """Unregister the group from Open Peer Power.
 
         This method must be run in the event loop.
@@ -615,21 +633,24 @@ class Group(Entity):
 
     async def async_update(self):
         """Query all members and determine current group state."""
-        self._state = STATE_UNKNOWN
+        self._state = None
         self._async_update_group_state()
 
     async def async_added_to_opp(self):
         """Handle addition to Open Peer Power."""
+        if self.opp.state != CoreState.running:
+            self.opp.bus.async_listen_once(EVENT_OPENPEERPOWER_START, self._async_start)
+            return
+
         if self.tracking:
-            self.async_start()
+            self._reset_tracked_state()
+        self._async_start_tracking()
 
     async def async_will_remove_from_opp(self):
         """Handle removal from Open Peer Power."""
-        if self._async_unsub_state_changed:
-            self._async_unsub_state_changed()
-            self._async_unsub_state_changed = None
+        self._async_stop()
 
-    async def _async_state_changed_listener(self, entity_id, old_state, new_state):
+    async def _async_state_changed_listener(self, event):
         """Respond to a member state changing.
 
         This method must be run in the event loop.
@@ -638,21 +659,48 @@ class Group(Entity):
         if self._async_unsub_state_changed is None:
             return
 
+        self.async_set_context(event.context)
+        new_state = event.data.get("new_state")
+
+        if new_state is None:
+            # The state was removed from the state machine
+            self._reset_tracked_state()
+
         self._async_update_group_state(new_state)
-        await self.async_update_op_state()
+        self.async_write_op_state()
 
-    @property
-    def _tracking_states(self):
-        """Return the states that the group is tracking."""
-        states = []
+    def _reset_tracked_state(self):
+        """Reset tracked state."""
+        self._on_off = {}
+        self._assumed = {}
+        self._on_states = set()
 
-        for entity_id in self.tracking:
+        for entity_id in self.trackable:
             state = self.opp.states.get(entity_id)
 
             if state is not None:
-                states.append(state)
+                self._see_state(state)
 
-        return states
+    def _see_state(self, new_state):
+        """Keep track of the the state."""
+        entity_id = new_state.entity_id
+        domain = new_state.domain
+        state = new_state.state
+        registry = self.opp.data[REG_KEY]
+        self._assumed[entity_id] = new_state.attributes.get(ATTR_ASSUMED_STATE)
+
+        if domain not in registry.on_states_by_domain:
+            # Handle the group of a group case
+            if state in registry.on_off_mapping:
+                self._on_states.add(state)
+            elif state in registry.off_on_mapping:
+                self._on_states.add(registry.off_on_mapping[state])
+            self._on_off[entity_id] = state in registry.on_off_mapping
+        else:
+            entity_on_state = registry.on_states_by_domain[domain]
+            if domain in self.opp.data[REG_KEY].on_states_by_domain:
+                self._on_states.update(entity_on_state)
+            self._on_off[entity_id] = state in entity_on_state
 
     @callback
     def _async_update_group_state(self, tr_state=None):
@@ -664,57 +712,39 @@ class Group(Entity):
         This method must be run in the event loop.
         """
         # To store current states of group entities. Might not be needed.
-        states = None
-        gr_state = self._state
-        gr_on = self.group_on
-        gr_off = self.group_off
+        if tr_state:
+            self._see_state(tr_state)
 
-        # We have not determined type of group yet
-        if gr_on is None:
-            if tr_state is None:
-                states = self._tracking_states
-
-                for state in states:
-                    gr_on, gr_off = _get_group_on_off(state.state)
-                    if gr_on is not None:
-                        break
-            else:
-                gr_on, gr_off = _get_group_on_off(tr_state.state)
-
-            if gr_on is not None:
-                self.group_on, self.group_off = gr_on, gr_off
-
-        # We cannot determine state of the group
-        if gr_on is None:
+        if not self._on_off:
             return
 
-        if tr_state is None or (
-            (gr_state == gr_on and tr_state.state == gr_off)
-            or (gr_state == gr_off and tr_state.state == gr_on)
-            or tr_state.state not in (gr_on, gr_off)
-        ):
-            if states is None:
-                states = self._tracking_states
-
-            if self.mode(state.state == gr_on for state in states):
-                self._state = gr_on
-            else:
-                self._state = gr_off
-
-        elif tr_state.state in (gr_on, gr_off):
-            self._state = tr_state.state
-
         if (
             tr_state is None
             or self._assumed_state
             and not tr_state.attributes.get(ATTR_ASSUMED_STATE)
         ):
-            if states is None:
-                states = self._tracking_states
-
-            self._assumed_state = self.mode(
-                state.attributes.get(ATTR_ASSUMED_STATE) for state in states
-            )
+            self._assumed_state = self.mode(self._assumed.values())
 
         elif tr_state.attributes.get(ATTR_ASSUMED_STATE):
             self._assumed_state = True
+
+        num_on_states = len(self._on_states)
+        # If all the entity domains we are tracking
+        # have the same on state we use this state
+        # and its opp.data[REG_KEY].on_off_mapping to off
+        if num_on_states == 1:
+            on_state = list(self._on_states)[0]
+        # If we do not have an on state for any domains
+        # we use None (which will be STATE_UNKNOWN)
+        elif num_on_states == 0:
+            self._state = None
+            return
+        # If the entity domains have more than one
+        # on state, we use STATE_ON/STATE_OFF
+        else:
+            on_state = STATE_ON
+        group_is_on = self.mode(self._on_off.values())
+        if group_is_on:
+            self._state = on_state
+        else:
+            self._state = self.opp.data[REG_KEY].on_off_mapping[on_state]
