@@ -1,5 +1,5 @@
 """Module that groups code required to handle state restore for component."""
-from typing import Iterable, Optional
+from typing import Any, Dict, Iterable, Optional
 
 from openpeerpower.core import Context, State
 from openpeerpower.helpers.state import async_reproduce_state
@@ -9,10 +9,13 @@ from . import get_entity_ids
 
 
 async def async_reproduce_states(
-    opp: OpenPeerPowerType, states: Iterable[State], context: Optional[Context] = None
+    opp: OpenPeerPowerType,
+    states: Iterable[State],
+    *,
+    context: Optional[Context] = None,
+    reproduce_options: Optional[Dict[str, Any]] = None,
 ) -> None:
     """Reproduce component states."""
-
     states_copy = []
     for state in states:
         members = get_entity_ids(opp, state.entity_id)
@@ -27,4 +30,6 @@ async def async_reproduce_states(
                     context=state.context,
                 )
             )
-    await async_reproduce_state(opp, states_copy, blocking=True, context=context)
+    await async_reproduce_state(
+        opp, states_copy, context=context, reproduce_options=reproduce_options
+    )
