@@ -1,7 +1,7 @@
 """Reproduce an Input text state."""
 import asyncio
 import logging
-from typing import Iterable, Optional
+from typing import Any, Dict, Iterable, Optional
 
 from openpeerpower.const import ATTR_ENTITY_ID
 from openpeerpower.core import Context, State
@@ -13,7 +13,11 @@ _LOGGER = logging.getLogger(__name__)
 
 
 async def _async_reproduce_state(
-    opp: OpenPeerPowerType, state: State, context: Optional[Context] = None
+    opp: OpenPeerPowerType,
+    state: State,
+    *,
+    context: Optional[Context] = None,
+    reproduce_options: Optional[Dict[str, Any]] = None,
 ) -> None:
     """Reproduce a single state."""
     cur_state = opp.states.get(state.entity_id)
@@ -37,10 +41,19 @@ async def _async_reproduce_state(
 
 
 async def async_reproduce_states(
-    opp: OpenPeerPowerType, states: Iterable[State], context: Optional[Context] = None
+    opp: OpenPeerPowerType,
+    states: Iterable[State],
+    *,
+    context: Optional[Context] = None,
+    reproduce_options: Optional[Dict[str, Any]] = None,
 ) -> None:
     """Reproduce Input text states."""
     # Reproduce states in parallel.
     await asyncio.gather(
-        *(_async_reproduce_state(opp, state, context) for state in states)
+        *(
+            _async_reproduce_state(
+                opp, state, context=context, reproduce_options=reproduce_options
+            )
+            for state in states
+        )
     )
