@@ -1,4 +1,6 @@
 """Component to interface with various media players."""
+from __future__ import annotations
+
 import asyncio
 import base64
 import collections
@@ -6,18 +8,29 @@ from datetime import timedelta
 import functools as ft
 import hashlib
 import logging
-from random import SystemRandom
-from typing import Optional
+import secrets
+from typing import List, Optional, Tuple
 from urllib.parse import urlparse
 
 from aiohttp import web
 from aiohttp.hdrs import CACHE_CONTROL, CONTENT_TYPE
+from aiohttp.typedefs import LooseHeaders
 import async_timeout
 import voluptuous as vol
+from yarl import URL
 
 from openpeerpower.components import websocket_api
 from openpeerpower.components.http import KEY_AUTHENTICATED, OpenPeerPowerView
+from openpeerpower.components.websocket_api.const import (
+    ERR_NOT_FOUND,
+    ERR_NOT_SUPPORTED,
+    ERR_UNKNOWN_ERROR,
+)
 from openpeerpower.const import (
+    HTTP_INTERNAL_SERVER_ERROR,
+    HTTP_NOT_FOUND,
+    HTTP_OK,
+    HTTP_UNAUTHORIZED,
     SERVICE_MEDIA_NEXT_TRACK,
     SERVICE_MEDIA_PAUSE,
     SERVICE_MEDIA_PLAY,
@@ -25,6 +38,7 @@ from openpeerpower.const import (
     SERVICE_MEDIA_PREVIOUS_TRACK,
     SERVICE_MEDIA_SEEK,
     SERVICE_MEDIA_STOP,
+    SERVICE_REPEAT_SET,
     SERVICE_SHUFFLE_SET,
     SERVICE_TOGGLE,
     SERVICE_TURN_OFF,
@@ -45,6 +59,7 @@ from openpeerpower.helpers.config_validation import (  # noqa: F401
 )
 from openpeerpower.helpers.entity import Entity
 from openpeerpower.helpers.entity_component import EntityComponent
+from openpeerpower.helpers.network import get_url
 from openpeerpower.loader import bind_opp
 
 from .const import (
@@ -61,9 +76,11 @@ from .const import (
     ATTR_MEDIA_DURATION,
     ATTR_MEDIA_ENQUEUE,
     ATTR_MEDIA_EPISODE,
+    ATTR_MEDIA_EXTRA,
     ATTR_MEDIA_PLAYLIST,
     ATTR_MEDIA_POSITION,
     ATTR_MEDIA_POSITION_UPDATED_AT,
+    ATTR_MEDIA_REPEAT,
     ATTR_MEDIA_SEASON,
     ATTR_MEDIA_SEEK_POSITION,
     ATTR_MEDIA_SERIES_TITLE,
@@ -75,16 +92,20 @@ from .const import (
     ATTR_SOUND_MODE,
     ATTR_SOUND_MODE_LIST,
     DOMAIN,
+    MEDIA_CLASS_DIRECTORY,
+    REPEAT_MODES,
     SERVICE_CLEAR_PLAYLIST,
     SERVICE_PLAY_MEDIA,
     SERVICE_SELECT_SOUND_MODE,
     SERVICE_SELECT_SOURCE,
+    SUPPORT_BROWSE_MEDIA,
     SUPPORT_CLEAR_PLAYLIST,
     SUPPORT_NEXT_TRACK,
     SUPPORT_PAUSE,
     SUPPORT_PLAY,
     SUPPORT_PLAY_MEDIA,
     SUPPORT_PREVIOUS_TRACK,
+    SUPPORT_REPEAT_SET,
     SUPPORT_SEEK,
     SUPPORT_SELECT_SOUND_MODE,
     SUPPORT_SELECT_SOURCE,
@@ -96,15 +117,14 @@ from .const import (
     SUPPORT_VOLUME_SET,
     SUPPORT_VOLUME_STEP,
 )
+from .errors import BrowseError
 
 # mypy: allow-untyped-defs, no-check-untyped-defs
 
 _LOGGER = logging.getLogger(__name__)
-_RND = SystemRandom()
 
 ENTITY_ID_FORMAT = DOMAIN + ".{}"
 
-ENTITY_IMAGE_URL = "/api/media_player_proxy/{0}?token={1}&cache={2}"
 CACHE_IMAGES = "images"
 CACHE_MAXSIZE = "maxsize"
 CACHE_LOCK = "lock"
@@ -116,8 +136,9 @@ SCAN_INTERVAL = timedelta(seconds=10)
 
 DEVICE_CLASS_TV = "tv"
 DEVICE_CLASS_SPEAKER = "speaker"
+DEVICE_CLASS_RECEIVER = "receiver"
 
-DEVICE_CLASSES = [DEVICE_CLASS_TV, DEVICE_CLASS_SPEAKER]
+DEVICE_CLASSES = [DEVICE_CLASS_TV, DEVICE_CLASS_SPEAKER, DEVICE_CLASS_RECEIVER]
 
 DEVICE_CLASSES_SCHEMA = vol.All(vol.Lower, vol.In(DEVICE_CLASSES))
 
@@ -126,6 +147,7 @@ MEDIA_PLAYER_PLAY_MEDIA_SCHEMA = {
     vol.Required(ATTR_MEDIA_CONTENT_TYPE): cv.string,
     vol.Required(ATTR_MEDIA_CONTENT_ID): cv.string,
     vol.Optional(ATTR_MEDIA_ENQUEUE): cv.boolean,
+    vol.Optional(ATTR_MEDIA_EXTRA, default={}): dict,
 }
 
 ATTR_TO_PROPERTY = [
@@ -151,6 +173,7 @@ ATTR_TO_PROPERTY = [
     ATTR_INPUT_SOURCE,
     ATTR_SOUND_MODE,
     ATTR_MEDIA_SHUFFLE,
+    ATTR_MEDIA_REPEAT,
 ]
 
 
@@ -167,12 +190,6 @@ def is_on(opp, entity_id=None):
     )
 
 
-WS_TYPE_MEDIA_PLAYER_THUMBNAIL = "media_player_thumbnail"
-SCHEMA_WEBSOCKET_GET_THUMBNAIL = websocket_api.BASE_COMMAND_MESSAGE_SCHEMA.extend(
-    {"type": WS_TYPE_MEDIA_PLAYER_THUMBNAIL, "entity_id": cv.entity_id}
-)
-
-
 def _rename_keys(**keys):
     """Create validator that renames keys.
 
@@ -196,11 +213,8 @@ async def async_setup(opp, config):
         logging.getLogger(__name__), DOMAIN, opp, SCAN_INTERVAL
     )
 
-    opp.components.websocket_api.async_register_command(
-        WS_TYPE_MEDIA_PLAYER_THUMBNAIL,
-        websocket_handle_thumbnail,
-        SCHEMA_WEBSOCKET_GET_THUMBNAIL,
-    )
+    opp.components.websocket_api.async_register_command(websocket_handle_thumbnail)
+    opp.components.websocket_api.async_register_command(websocket_browse_media)
     opp.http.register_view(MediaPlayerImageView(component))
 
     await component.async_setup(config)
@@ -212,7 +226,7 @@ async def async_setup(opp, config):
         SERVICE_TURN_OFF, {}, "async_turn_off", [SUPPORT_TURN_OFF]
     )
     component.async_register_entity_service(
-        SERVICE_TOGGLE, {}, "async_toggle", [SUPPORT_TURN_OFF | SUPPORT_TURN_ON],
+        SERVICE_TOGGLE, {}, "async_toggle", [SUPPORT_TURN_OFF | SUPPORT_TURN_ON]
     )
     component.async_register_entity_service(
         SERVICE_VOLUME_UP,
@@ -242,7 +256,7 @@ async def async_setup(opp, config):
         SERVICE_MEDIA_STOP, {}, "async_media_stop", [SUPPORT_STOP]
     )
     component.async_register_entity_service(
-        SERVICE_MEDIA_NEXT_TRACK, {}, "async_media_next_track", [SUPPORT_NEXT_TRACK],
+        SERVICE_MEDIA_NEXT_TRACK, {}, "async_media_next_track", [SUPPORT_NEXT_TRACK]
     )
     component.async_register_entity_service(
         SERVICE_MEDIA_PREVIOUS_TRACK,
@@ -251,7 +265,7 @@ async def async_setup(opp, config):
         [SUPPORT_PREVIOUS_TRACK],
     )
     component.async_register_entity_service(
-        SERVICE_CLEAR_PLAYLIST, {}, "async_clear_playlist", [SUPPORT_CLEAR_PLAYLIST],
+        SERVICE_CLEAR_PLAYLIST, {}, "async_clear_playlist", [SUPPORT_CLEAR_PLAYLIST]
     )
     component.async_register_entity_service(
         SERVICE_VOLUME_SET,
@@ -279,11 +293,7 @@ async def async_setup(opp, config):
         SERVICE_MEDIA_SEEK,
         vol.All(
             cv.make_entity_service_schema(
-                {
-                    vol.Required(ATTR_MEDIA_SEEK_POSITION): vol.All(
-                        vol.Coerce(float), vol.Range(min=0)
-                    )
-                }
+                {vol.Required(ATTR_MEDIA_SEEK_POSITION): cv.positive_float}
             ),
             _rename_keys(position=ATTR_MEDIA_SEEK_POSITION),
         ),
@@ -322,6 +332,13 @@ async def async_setup(opp, config):
         [SUPPORT_SHUFFLE_SET],
     )
 
+    component.async_register_entity_service(
+        SERVICE_REPEAT_SET,
+        {vol.Required(ATTR_MEDIA_REPEAT): vol.In(REPEAT_MODES)},
+        "async_set_repeat",
+        [SUPPORT_REPEAT_SET],
+    )
+
     return True
 
 
@@ -335,8 +352,8 @@ async def async_unload_entry(opp, entry):
     return await opp.data[DOMAIN].async_unload_entry(entry)
 
 
-class MediaPlayerDevice(Entity):
-    """ABC for media player devices."""
+class MediaPlayerEntity(Entity):
+    """ABC for media player entities."""
 
     _access_token: Optional[str] = None
 
@@ -350,9 +367,7 @@ class MediaPlayerDevice(Entity):
     def access_token(self) -> str:
         """Access token for this media player."""
         if self._access_token is None:
-            self._access_token = hashlib.sha256(
-                _RND.getrandbits(256).to_bytes(32, "little")
-            ).hexdigest()
+            self._access_token = secrets.token_hex(32)
         return self._access_token
 
     @property
@@ -418,7 +433,20 @@ class MediaPlayerDevice(Entity):
         if url is None:
             return None, None
 
-        return await _async_fetch_image(self.opp, url)
+        return await self._async_fetch_image_from_cache(url)
+
+    async def async_get_browse_image(
+        self,
+        media_content_type: str,
+        media_content_id: str,
+        media_image_id: Optional[str] = None,
+    ) -> Tuple[Optional[str], Optional[str]]:
+        """
+        Optionally fetch internally accessible image for media browser.
+
+        Must be implemented by integration.
+        """
+        return None, None
 
     @property
     def media_title(self):
@@ -505,6 +533,11 @@ class MediaPlayerDevice(Entity):
         """Boolean if shuffle is enabled."""
         return None
 
+    @property
+    def repeat(self):
+        """Return current repeat mode."""
+        return None
+
     @property
     def supported_features(self):
         """Flag media player features that are supported."""
@@ -516,7 +549,7 @@ class MediaPlayerDevice(Entity):
 
     async def async_turn_on(self):
         """Turn the media player on."""
-        await self.opp.async_add_job(self.turn_on)
+        await self.opp.async_add_executor_job(self.turn_on)
 
     def turn_off(self):
         """Turn the media player off."""
@@ -524,7 +557,7 @@ class MediaPlayerDevice(Entity):
 
     async def async_turn_off(self):
         """Turn the media player off."""
-        await self.opp.async_add_job(self.turn_off)
+        await self.opp.async_add_executor_job(self.turn_off)
 
     def mute_volume(self, mute):
         """Mute the volume."""
@@ -532,7 +565,7 @@ class MediaPlayerDevice(Entity):
 
     async def async_mute_volume(self, mute):
         """Mute the volume."""
-        await self.opp.async_add_job(self.mute_volume, mute)
+        await self.opp.async_add_executor_job(self.mute_volume, mute)
 
     def set_volume_level(self, volume):
         """Set volume level, range 0..1."""
@@ -540,7 +573,7 @@ class MediaPlayerDevice(Entity):
 
     async def async_set_volume_level(self, volume):
         """Set volume level, range 0..1."""
-        await self.opp.async_add_job(self.set_volume_level, volume)
+        await self.opp.async_add_executor_job(self.set_volume_level, volume)
 
     def media_play(self):
         """Send play command."""
@@ -548,7 +581,7 @@ class MediaPlayerDevice(Entity):
 
     async def async_media_play(self):
         """Send play command."""
-        await self.opp.async_add_job(self.media_play)
+        await self.opp.async_add_executor_job(self.media_play)
 
     def media_pause(self):
         """Send pause command."""
@@ -556,7 +589,7 @@ class MediaPlayerDevice(Entity):
 
     async def async_media_pause(self):
         """Send pause command."""
-        await self.opp.async_add_job(self.media_pause)
+        await self.opp.async_add_executor_job(self.media_pause)
 
     def media_stop(self):
         """Send stop command."""
@@ -564,7 +597,7 @@ class MediaPlayerDevice(Entity):
 
     async def async_media_stop(self):
         """Send stop command."""
-        await self.opp.async_add_job(self.media_stop)
+        await self.opp.async_add_executor_job(self.media_stop)
 
     def media_previous_track(self):
         """Send previous track command."""
@@ -572,7 +605,7 @@ class MediaPlayerDevice(Entity):
 
     async def async_media_previous_track(self):
         """Send previous track command."""
-        await self.opp.async_add_job(self.media_previous_track)
+        await self.opp.async_add_executor_job(self.media_previous_track)
 
     def media_next_track(self):
         """Send next track command."""
@@ -580,7 +613,7 @@ class MediaPlayerDevice(Entity):
 
     async def async_media_next_track(self):
         """Send next track command."""
-        await self.opp.async_add_job(self.media_next_track)
+        await self.opp.async_add_executor_job(self.media_next_track)
 
     def media_seek(self, position):
         """Send seek command."""
@@ -588,7 +621,7 @@ class MediaPlayerDevice(Entity):
 
     async def async_media_seek(self, position):
         """Send seek command."""
-        await self.opp.async_add_job(self.media_seek, position)
+        await self.opp.async_add_executor_job(self.media_seek, position)
 
     def play_media(self, media_type, media_id, **kwargs):
         """Play a piece of media."""
@@ -596,7 +629,7 @@ class MediaPlayerDevice(Entity):
 
     async def async_play_media(self, media_type, media_id, **kwargs):
         """Play a piece of media."""
-        await self.opp.async_add_job(
+        await self.opp.async_add_executor_job(
             ft.partial(self.play_media, media_type, media_id, **kwargs)
         )
 
@@ -606,7 +639,7 @@ class MediaPlayerDevice(Entity):
 
     async def async_select_source(self, source):
         """Select input source."""
-        await self.opp.async_add_job(self.select_source, source)
+        await self.opp.async_add_executor_job(self.select_source, source)
 
     def select_sound_mode(self, sound_mode):
         """Select sound mode."""
@@ -614,7 +647,7 @@ class MediaPlayerDevice(Entity):
 
     async def async_select_sound_mode(self, sound_mode):
         """Select sound mode."""
-        await self.opp.async_add_job(self.select_sound_mode, sound_mode)
+        await self.opp.async_add_executor_job(self.select_sound_mode, sound_mode)
 
     def clear_playlist(self):
         """Clear players playlist."""
@@ -622,7 +655,7 @@ class MediaPlayerDevice(Entity):
 
     async def async_clear_playlist(self):
         """Clear players playlist."""
-        await self.opp.async_add_job(self.clear_playlist)
+        await self.opp.async_add_executor_job(self.clear_playlist)
 
     def set_shuffle(self, shuffle):
         """Enable/disable shuffle mode."""
@@ -630,7 +663,15 @@ class MediaPlayerDevice(Entity):
 
     async def async_set_shuffle(self, shuffle):
         """Enable/disable shuffle mode."""
-        await self.opp.async_add_job(self.set_shuffle, shuffle)
+        await self.opp.async_add_executor_job(self.set_shuffle, shuffle)
+
+    def set_repeat(self, repeat):
+        """Set repeat mode."""
+        raise NotImplementedError()
+
+    async def async_set_repeat(self, repeat):
+        """Set repeat mode."""
+        await self.opp.async_add_executor_job(self.set_repeat, repeat)
 
     # No need to overwrite these.
     @property
@@ -702,7 +743,7 @@ class MediaPlayerDevice(Entity):
         """Toggle the power on the media player."""
         if hasattr(self, "toggle"):
             # pylint: disable=no-member
-            await self.opp.async_add_job(self.toggle)
+            await self.opp.async_add_executor_job(self.toggle)
             return
 
         if self.state in [STATE_OFF, STATE_IDLE]:
@@ -717,7 +758,7 @@ class MediaPlayerDevice(Entity):
         """
         if hasattr(self, "volume_up"):
             # pylint: disable=no-member
-            await self.opp.async_add_job(self.volume_up)
+            await self.opp.async_add_executor_job(self.volume_up)
             return
 
         if self.volume_level < 1 and self.supported_features & SUPPORT_VOLUME_SET:
@@ -730,7 +771,7 @@ class MediaPlayerDevice(Entity):
         """
         if hasattr(self, "volume_down"):
             # pylint: disable=no-member
-            await self.opp.async_add_job(self.volume_down)
+            await self.opp.async_add_executor_job(self.volume_down)
             return
 
         if self.volume_level > 0 and self.supported_features & SUPPORT_VOLUME_SET:
@@ -740,7 +781,7 @@ class MediaPlayerDevice(Entity):
         """Play or pause the media player."""
         if hasattr(self, "media_play_pause"):
             # pylint: disable=no-member
-            await self.opp.async_add_job(self.media_play_pause)
+            await self.opp.async_add_executor_job(self.media_play_pause)
             return
 
         if self.state == STATE_PLAYING:
@@ -757,12 +798,20 @@ class MediaPlayerDevice(Entity):
         if self.media_image_remotely_accessible:
             return self.media_image_url
 
+        return self.media_image_local
+
+    @property
+    def media_image_local(self):
+        """Return local url to media image."""
         image_hash = self.media_image_hash
 
         if image_hash is None:
             return None
 
-        return ENTITY_IMAGE_URL.format(self.entity_id, self.access_token, image_hash)
+        return (
+            f"/api/media_player_proxy/{self.entity_id}?"
+            f"token={self.access_token}&cache={image_hash}"
+        )
 
     @property
     def capability_attributes(self):
@@ -788,54 +837,97 @@ class MediaPlayerDevice(Entity):
         if self.state == STATE_OFF:
             return None
 
-        state_attr = {
-            attr: getattr(self, attr)
-            for attr in ATTR_TO_PROPERTY
-            if getattr(self, attr) is not None
-        }
+        state_attr = {}
+
+        for attr in ATTR_TO_PROPERTY:
+            value = getattr(self, attr)
+            if value is not None:
+                state_attr[attr] = value
+
+        if self.media_image_remotely_accessible:
+            state_attr["entity_picture_local"] = self.media_image_local
 
         return state_attr
 
+    async def async_browse_media(
+        self,
+        media_content_type: Optional[str] = None,
+        media_content_id: Optional[str] = None,
+    ) -> BrowseMedia:
+        """Return a BrowseMedia instance.
 
-async def _async_fetch_image(opp, url):
-    """Fetch image.
+        The BrowseMedia instance will be used by the
+        "media_player/browse_media" websocket command.
+        """
+        raise NotImplementedError()
 
-    Images are cached in memory (the images are typically 10-100kB in size).
-    """
-    cache_images = ENTITY_IMAGE_CACHE[CACHE_IMAGES]
-    cache_maxsize = ENTITY_IMAGE_CACHE[CACHE_MAXSIZE]
+    async def _async_fetch_image_from_cache(self, url):
+        """Fetch image.
 
-    if urlparse(url).hostname is None:
-        url = opp.config.api.base_url + url
+        Images are cached in memory (the images are typically 10-100kB in size).
+        """
+        cache_images = ENTITY_IMAGE_CACHE[CACHE_IMAGES]
+        cache_maxsize = ENTITY_IMAGE_CACHE[CACHE_MAXSIZE]
 
-    if url not in cache_images:
-        cache_images[url] = {CACHE_LOCK: asyncio.Lock()}
+        if urlparse(url).hostname is None:
+            url = f"{get_url(self.opp)}{url}"
 
-    async with cache_images[url][CACHE_LOCK]:
-        if CACHE_CONTENT in cache_images[url]:
-            return cache_images[url][CACHE_CONTENT]
+        if url not in cache_images:
+            cache_images[url] = {CACHE_LOCK: asyncio.Lock()}
 
+        async with cache_images[url][CACHE_LOCK]:
+            if CACHE_CONTENT in cache_images[url]:
+                return cache_images[url][CACHE_CONTENT]
+
+        (content, content_type) = await self._async_fetch_image(url)
+
+        async with cache_images[url][CACHE_LOCK]:
+            cache_images[url][CACHE_CONTENT] = content, content_type
+            while len(cache_images) > cache_maxsize:
+                cache_images.popitem(last=False)
+
+        return content, content_type
+
+    async def _async_fetch_image(self, url):
+        """Retrieve an image."""
         content, content_type = (None, None)
-        websession = async_get_clientsession(opp)
+        websession = async_get_clientsession(self.opp)
         try:
             with async_timeout.timeout(10):
                 response = await websession.get(url)
 
-                if response.status == 200:
+                if response.status == HTTP_OK:
                     content = await response.read()
                     content_type = response.headers.get(CONTENT_TYPE)
                     if content_type:
                         content_type = content_type.split(";")[0]
-                    cache_images[url][CACHE_CONTENT] = content, content_type
 
         except asyncio.TimeoutError:
             pass
 
-        while len(cache_images) > cache_maxsize:
-            cache_images.popitem(last=False)
+        if content is None:
+            _LOGGER.warning("Error retrieving proxied image from %s", url)
 
         return content, content_type
 
+    def get_browse_image_url(
+        self,
+        media_content_type: str,
+        media_content_id: str,
+        media_image_id: Optional[str] = None,
+    ) -> str:
+        """Generate an url for a media browser image."""
+        url_path = (
+            f"/api/media_player_proxy/{self.entity_id}/browse_media"
+            f"/{media_content_type}/{media_content_id}"
+        )
+
+        url_query = {"token": self.access_token}
+        if media_image_id:
+            url_query["media_image_id"] = media_image_id
+
+        return str(URL(url_path).with_query(url_query))
+
 
 class MediaPlayerImageView(OpenPeerPowerView):
     """Media player view to serve an image."""
@@ -843,16 +935,25 @@ class MediaPlayerImageView(OpenPeerPowerView):
     requires_auth = False
     url = "/api/media_player_proxy/{entity_id}"
     name = "api:media_player:image"
+    extra_urls = [
+        url + "/browse_media/{media_content_type}/{media_content_id}",
+    ]
 
     def __init__(self, component):
         """Initialize a media player view."""
         self.component = component
 
-    async def get(self, request, entity_id):
+    async def get(
+        self,
+        request: web.Request,
+        entity_id: str,
+        media_content_type: Optional[str] = None,
+        media_content_id: Optional[str] = None,
+    ) -> web.Response:
         """Start a get request."""
         player = self.component.get_entity(entity_id)
         if player is None:
-            status = 404 if request[KEY_AUTHENTICATED] else 401
+            status = HTTP_NOT_FOUND if request[KEY_AUTHENTICATED] else HTTP_UNAUTHORIZED
             return web.Response(status=status)
 
         authenticated = (
@@ -861,23 +962,29 @@ class MediaPlayerImageView(OpenPeerPowerView):
         )
 
         if not authenticated:
-            return web.Response(status=401)
+            return web.Response(status=HTTP_UNAUTHORIZED)
 
-        if player.media_image_remotely_accessible:
-            url = player.media_image_url
-            if url is not None:
-                return web.Response(status=302, headers={"location": url})
-            return web.Response(status=500)
-
-        data, content_type = await player.async_get_media_image()
+        if media_content_type and media_content_id:
+            media_image_id = request.query.get("media_image_id")
+            data, content_type = await player.async_get_browse_image(
+                media_content_type, media_content_id, media_image_id
+            )
+        else:
+            data, content_type = await player.async_get_media_image()
 
         if data is None:
-            return web.Response(status=500)
+            return web.Response(status=HTTP_INTERNAL_SERVER_ERROR)
 
-        headers = {CACHE_CONTROL: "max-age=3600"}
+        headers: LooseHeaders = {CACHE_CONTROL: "max-age=3600"}
         return web.Response(body=data, content_type=content_type, headers=headers)
 
 
+@websocket_api.websocket_command(
+    {
+        vol.Required("type"): "media_player_thumbnail",
+        vol.Required("entity_id"): cv.entity_id,
+    }
+)
 @websocket_api.async_response
 async def websocket_handle_thumbnail(opp, connection, msg):
     """Handle get media player cover command.
@@ -889,12 +996,14 @@ async def websocket_handle_thumbnail(opp, connection, msg):
 
     if player is None:
         connection.send_message(
-            websocket_api.error_message(
-                msg["id"], "entity_not_found", "Entity not found"
-            )
+            websocket_api.error_message(msg["id"], ERR_NOT_FOUND, "Entity not found")
         )
         return
 
+    _LOGGER.warning(
+        "The websocket command media_player_thumbnail is deprecated. Use /api/media_player_proxy instead"
+    )
+
     data, content_type = await player.async_get_media_image()
 
     if data is None:
@@ -912,3 +1021,154 @@ async def websocket_handle_thumbnail(opp, connection, msg):
             "content": base64.b64encode(data).decode("utf-8"),
         },
     )
+
+
+@websocket_api.websocket_command(
+    {
+        vol.Required("type"): "media_player/browse_media",
+        vol.Required("entity_id"): cv.entity_id,
+        vol.Inclusive(
+            ATTR_MEDIA_CONTENT_TYPE,
+            "media_ids",
+            "media_content_type and media_content_id must be provided together",
+        ): str,
+        vol.Inclusive(
+            ATTR_MEDIA_CONTENT_ID,
+            "media_ids",
+            "media_content_type and media_content_id must be provided together",
+        ): str,
+    }
+)
+@websocket_api.async_response
+async def websocket_browse_media(opp, connection, msg):
+    """
+    Browse media available to the media_player entity.
+
+    To use, media_player integrations can implement MediaPlayerEntity.async_browse_media()
+    """
+    component = opp.data[DOMAIN]
+    player: Optional[MediaPlayerDevice] = component.get_entity(msg["entity_id"])
+
+    if player is None:
+        connection.send_error(msg["id"], "entity_not_found", "Entity not found")
+        return
+
+    if not player.supported_features & SUPPORT_BROWSE_MEDIA:
+        connection.send_message(
+            websocket_api.error_message(
+                msg["id"], ERR_NOT_SUPPORTED, "Player does not support browsing media"
+            )
+        )
+        return
+
+    media_content_type = msg.get(ATTR_MEDIA_CONTENT_TYPE)
+    media_content_id = msg.get(ATTR_MEDIA_CONTENT_ID)
+
+    try:
+        payload = await player.async_browse_media(media_content_type, media_content_id)
+    except NotImplementedError:
+        _LOGGER.error(
+            "%s allows media browsing but its integration (%s) does not",
+            player.entity_id,
+            player.platform.platform_name,
+        )
+        connection.send_message(
+            websocket_api.error_message(
+                msg["id"],
+                ERR_NOT_SUPPORTED,
+                "Integration does not support browsing media",
+            )
+        )
+        return
+    except BrowseError as err:
+        connection.send_message(
+            websocket_api.error_message(msg["id"], ERR_UNKNOWN_ERROR, str(err))
+        )
+        return
+
+    # For backwards compat
+    if isinstance(payload, BrowseMedia):
+        payload = payload.as_dict()
+    else:
+        _LOGGER.warning("Browse Media should use new BrowseMedia class")
+
+    connection.send_result(msg["id"], payload)
+
+
+class MediaPlayerDevice(MediaPlayerEntity):
+    """ABC for media player devices (for backwards compatibility)."""
+
+    def __init_subclass__(cls, **kwargs):
+        """Print deprecation warning."""
+        super().__init_subclass__(**kwargs)
+        _LOGGER.warning(
+            "MediaPlayerDevice is deprecated, modify %s to extend MediaPlayerEntity",
+            cls.__name__,
+        )
+
+
+class BrowseMedia:
+    """Represent a browsable media file."""
+
+    def __init__(
+        self,
+        *,
+        media_class: str,
+        media_content_id: str,
+        media_content_type: str,
+        title: str,
+        can_play: bool,
+        can_expand: bool,
+        children: Optional[List["BrowseMedia"]] = None,
+        children_media_class: Optional[str] = None,
+        thumbnail: Optional[str] = None,
+    ):
+        """Initialize browse media item."""
+        self.media_class = media_class
+        self.media_content_id = media_content_id
+        self.media_content_type = media_content_type
+        self.title = title
+        self.can_play = can_play
+        self.can_expand = can_expand
+        self.children = children
+        self.children_media_class = children_media_class
+        self.thumbnail = thumbnail
+
+    def as_dict(self, *, parent: bool = True) -> dict:
+        """Convert Media class to browse media dictionary."""
+        if self.children_media_class is None:
+            self.calculate_children_class()
+
+        response = {
+            "title": self.title,
+            "media_class": self.media_class,
+            "media_content_type": self.media_content_type,
+            "media_content_id": self.media_content_id,
+            "can_play": self.can_play,
+            "can_expand": self.can_expand,
+            "children_media_class": self.children_media_class,
+            "thumbnail": self.thumbnail,
+        }
+
+        if not parent:
+            return response
+
+        if self.children:
+            response["children"] = [
+                child.as_dict(parent=False) for child in self.children
+            ]
+        else:
+            response["children"] = []
+
+        return response
+
+    def calculate_children_class(self) -> None:
+        """Count the children media classes and calculate the correct class."""
+        if self.children is None or len(self.children) == 0:
+            return
+
+        self.children_media_class = MEDIA_CLASS_DIRECTORY
+
+        proposed_class = self.children[0].media_class
+        if all(child.media_class == proposed_class for child in self.children):
+            self.children_media_class = proposed_class
