@@ -5,7 +5,6 @@ import logging
 
 import async_timeout
 from awesomeversion import AwesomeVersion
-from distro import linux_distribution
 import voluptuous as vol
 
 from openpeerpower.const import __version__ as current_version
@@ -23,20 +22,22 @@ CONF_COMPONENT_REPORTING = "include_used_components"
 
 DOMAIN = "updater"
 
-UPDATER_URL = "https://updater.openpeerpower.io/"
+UPDATER_URL = "https://openpeerpower.io/version.json"
+
 
 CONFIG_SCHEMA = vol.Schema(
     {
         DOMAIN: {
-            vol.Optional(CONF_REPORTING, default=True): cv.boolean,
-            vol.Optional(CONF_COMPONENT_REPORTING, default=False): cv.boolean,
+            vol.Optional(CONF_REPORTING): cv.boolean,
+            vol.Optional(CONF_COMPONENT_REPORTING): cv.boolean,
         }
     },
     extra=vol.ALLOW_EXTRA,
 )
 
 RESPONSE_SCHEMA = vol.Schema(
-    {vol.Required("version"): cv.string, vol.Required("release-notes"): cv.url}
+    {vol.Required("current_version"): cv.string, vol.Required("release_notes"): cv.url},
+    extra=vol.REMOVE_EXTRA,
 )
 
 
@@ -52,28 +53,27 @@ class Updater:
 
 async def async_setup(opp, config):
     """Set up the updater component."""
-    if "dev" in current_version:
-        # This component only makes sense in release versions
-        _LOGGER.info("Running on 'dev', only analytics will be submitted")
-
     conf = config.get(DOMAIN, {})
-    if conf.get(CONF_REPORTING):
-        huuid = await opp.helpers.instance_id.async_get()
-    else:
-        huuid = None
 
-    include_components = conf.get(CONF_COMPONENT_REPORTING)
+    for option in (CONF_COMPONENT_REPORTING, CONF_REPORTING):
+        if option in conf:
+            _LOGGER.warning(
+                "Analytics reporting with the option '%s' "
+                "is deprecated and you should remove that from your configuration. "
+                "The analytics part of this integration has moved to the new 'analytics' integration",
+                option,
+            )
 
     async def check_new_version() -> Updater:
         """Check if a new version is available and report if one is."""
-        newest, release_notes = await get_newest_version(opp, huuid, include_components)
-
-        _LOGGER.debug("Fetched version %s: %s", newest, release_notes)
-
         # Skip on dev
         if "dev" in current_version:
             return Updater(False, "", "")
 
+        newest, release_notes = await get_newest_version(opp)
+
+        _LOGGER.debug("Fetched version %s: %s", newest, release_notes)
+
         # Load data from Supervisor
         if opp.components.oppio.is_oppio():
             core_info = opp.components.oppio.get_core_info()
@@ -119,34 +119,12 @@ async def async_setup(opp, config):
     return True
 
 
-async def get_newest_version(opp, huuid, include_components):
+async def get_newest_version(opp):
     """Get the newest Open Peer Power version."""
-    if huuid:
-        info_object = await opp.helpers.system_info.async_get_system_info()
-
-        if include_components:
-            info_object["components"] = list(opp.config.components)
-
-        linux_dist = await opp.async_add_executor_job(linux_distribution, False)
-        info_object["distribution"] = linux_dist[0]
-        info_object["os_version"] = linux_dist[1]
-
-        info_object["huuid"] = huuid
-    else:
-        info_object = {}
-
     session = async_get_clientsession(opp)
 
     with async_timeout.timeout(30):
-        req = await session.post(UPDATER_URL, json=info_object)
-
-    _LOGGER.info(
-        (
-            "Submitted analytics to Open Peer Power servers. "
-            "Information submitted includes %s"
-        ),
-        info_object,
-    )
+        req = await session.get(UPDATER_URL)
 
     try:
         res = await req.json()
@@ -157,7 +135,7 @@ async def get_newest_version(opp, huuid, include_components):
 
     try:
         res = RESPONSE_SCHEMA(res)
-        return res["version"], res["release-notes"]
+        return res["current_version"], res["release_notes"]
     except vol.Invalid as err:
         raise update_coordinator.UpdateFailed(
             f"Got unexpected response: {err}"
