@@ -3,7 +3,7 @@
 import logging
 from typing import Optional, Tuple
 
-from openpeerpower.components.climate import ClimateDevice
+from openpeerpower.components.climate import ClimateEntity
 from openpeerpower.components.climate.const import (
     ATTR_TARGET_TEMP_HIGH,
     ATTR_TARGET_TEMP_LOW,
@@ -149,7 +149,7 @@ def get_device(opp, values, **kwargs):
     return None
 
 
-class ZWaveClimateBase(ZWaveDeviceEntity, ClimateDevice):
+class ZWaveClimateBase(ZWaveDeviceEntity, ClimateEntity):
     """Representation of a Z-Wave Climate device."""
 
     def __init__(self, values, temp_unit):
@@ -160,13 +160,13 @@ class ZWaveClimateBase(ZWaveDeviceEntity, ClimateDevice):
         self._current_temperature = None
         self._hvac_action = None
         self._hvac_list = None  # [zwave_mode]
-        self._hvac_mapping = None  # {op_mode:zwave_mode}
-        self._hvac_mode = None  # op_mode
+        self._hvac_mapping = None  # {ha_mode:zwave_mode}
+        self._hvac_mode = None  # ha_mode
         self._aux_heat = None
-        self._default_hvac_mode = None  # op_mode
-        self._preset_mapping = None  # {op_mode:zwave_mode}
+        self._default_hvac_mode = None  # ha_mode
+        self._preset_mapping = None  # {ha_mode:zwave_mode}
         self._preset_list = None  # [zwave_mode]
-        self._preset_mode = None  # op_mode if exists, else zwave_mode
+        self._preset_mode = None  # ha_mode if exists, else zwave_mode
         self._current_fan_mode = None
         self._fan_modes = None
         self._fan_action = None
@@ -248,17 +248,17 @@ class ZWaveClimateBase(ZWaveDeviceEntity, ClimateDevice):
             mode_list = self._mode().data_items
             if mode_list:
                 for mode in mode_list:
-                    op_mode = HVAC_STATE_MAPPINGS.get(str(mode).lower())
-                    op_preset = PRESET_MAPPINGS.get(str(mode).lower())
+                    ha_mode = HVAC_STATE_MAPPINGS.get(str(mode).lower())
+                    ha_preset = PRESET_MAPPINGS.get(str(mode).lower())
                     if mode == AUX_HEAT_ZWAVE_MODE:
                         # Aux Heat should not be included in any mapping
                         self._aux_heat = True
-                    elif op_mode and op_mode not in self._hvac_mapping:
-                        self._hvac_mapping[op_mode] = mode
-                        self._hvac_list.append(op_mode)
-                    elif op_preset and op_preset not in self._preset_mapping:
-                        self._preset_mapping[op_preset] = mode
-                        self._preset_list.append(op_preset)
+                    elif ha_mode and ha_mode not in self._hvac_mapping:
+                        self._hvac_mapping[ha_mode] = mode
+                        self._hvac_list.append(ha_mode)
+                    elif ha_preset and ha_preset not in self._preset_mapping:
+                        self._preset_mapping[ha_preset] = mode
+                        self._preset_list.append(ha_preset)
                     else:
                         # If nothing matches
                         self._preset_list.append(mode)
