@@ -1,29 +1,37 @@
 """Manage config entries in Open Peer Power."""
+from __future__ import annotations
+
 import asyncio
 import functools
 import logging
+from types import MappingProxyType, MethodType
 from typing import Any, Callable, Dict, List, Optional, Set, Union, cast
-import uuid
 import weakref
 
 import attr
 
 from openpeerpower import data_entry_flow, loader
-from openpeerpower.core import OpenPeerPower, callback
+from openpeerpower.core import CALLBACK_TYPE, OpenPeerPower, callback
 from openpeerpower.exceptions import ConfigEntryNotReady, OpenPeerPowerError
-from openpeerpower.helpers import entity_registry
+from openpeerpower.helpers import device_registry, entity_registry
 from openpeerpower.helpers.event import Event
+from openpeerpower.helpers.typing import UNDEFINED, UndefinedType
 from openpeerpower.setup import async_process_deps_reqs, async_setup_component
 from openpeerpower.util.decorator import Registry
+import openpeerpower.util.uuid as uuid_util
 
 _LOGGER = logging.getLogger(__name__)
-_UNDEF: dict = {}
 
 SOURCE_DISCOVERY = "discovery"
+SOURCE_OPPIO = "oppio"
+SOURCE_HOMEKIT = "homekit"
 SOURCE_IMPORT = "import"
+SOURCE_INTEGRATION_DISCOVERY = "integration_discovery"
+SOURCE_MQTT = "mqtt"
 SOURCE_SSDP = "ssdp"
 SOURCE_USER = "user"
 SOURCE_ZEROCONF = "zeroconf"
+SOURCE_DHCP = "dhcp"
 
 # If a user wants to hide a discovery from the UI they can "Ignore" it. The config_entries/ignore_flow
 # websocket command creates a config entry with this source and while it exists normal discoveries
@@ -35,6 +43,9 @@ SOURCE_IGNORE = "ignore"
 # been removed and unloaded.
 SOURCE_UNIGNORE = "unignore"
 
+# This is used to signal that re-authentication is required by the user.
+SOURCE_REAUTH = "reauth"
+
 HANDLERS = Registry()
 
 STORAGE_KEY = "core.config_entries"
@@ -60,6 +71,7 @@ ENTRY_STATE_FAILED_UNLOAD = "failed_unload"
 
 UNRECOVERABLE_STATES = (ENTRY_STATE_MIGRATION_ERROR, ENTRY_STATE_FAILED_UNLOAD)
 
+DEFAULT_DISCOVERY_UNIQUE_ID = "default_discovery_unique_id"
 DISCOVERY_NOTIFICATION_ID = "config_entry_discovery"
 DISCOVERY_SOURCES = (
     SOURCE_SSDP,
@@ -69,6 +81,8 @@ DISCOVERY_SOURCES = (
     SOURCE_UNIGNORE,
 )
 
+RECONFIGURE_NOTIFICATION_ID = "config_entry_reconfigure"
+
 EVENT_FLOW_DISCOVERED = "config_entry_discovered"
 
 CONN_CLASS_CLOUD_PUSH = "cloud_push"
@@ -78,6 +92,10 @@ CONN_CLASS_LOCAL_POLL = "local_poll"
 CONN_CLASS_ASSUMED = "assumed"
 CONN_CLASS_UNKNOWN = "unknown"
 
+DISABLED_USER = "user"
+
+RELOAD_AFTER_UPDATE_DELAY = 30
+
 
 class ConfigError(OpenPeerPowerError):
     """Error while configuring an account."""
@@ -91,6 +109,9 @@ class OperationNotAllowed(ConfigError):
     """Raised when a config entry operation is not allowed."""
 
 
+UpdateListenerType = Callable[[OpenPeerPower, "ConfigEntry"], Any]
+
+
 class ConfigEntry:
     """Hold a configuration entry."""
 
@@ -102,10 +123,12 @@ class ConfigEntry:
         "data",
         "options",
         "unique_id",
+        "supports_unload",
         "system_options",
         "source",
         "connection_class",
         "state",
+        "disabled_by",
         "_setup_lock",
         "update_listeners",
         "_async_cancel_retry_setup",
@@ -124,10 +147,11 @@ class ConfigEntry:
         unique_id: Optional[str] = None,
         entry_id: Optional[str] = None,
         state: str = ENTRY_STATE_NOT_LOADED,
+        disabled_by: Optional[str] = None,
     ) -> None:
         """Initialize a config entry."""
         # Unique id of the config entry
-        self.entry_id = entry_id or uuid.uuid4().hex
+        self.entry_id = entry_id or uuid_util.random_uuid_hex()
 
         # Version of the configuration.
         self.version = version
@@ -139,10 +163,10 @@ class ConfigEntry:
         self.title = title
 
         # Config data
-        self.data = data
+        self.data = MappingProxyType(data)
 
         # Entry options
-        self.options = options or {}
+        self.options = MappingProxyType(options or {})
 
         # Entry system options
         self.system_options = SystemOptions(**system_options)
@@ -159,8 +183,16 @@ class ConfigEntry:
         # Unique ID of this entry.
         self.unique_id = unique_id
 
+        # Config entry is disabled
+        self.disabled_by = disabled_by
+
+        # Supports unload
+        self.supports_unload = False
+
         # Listeners to call on update
-        self.update_listeners: List = []
+        self.update_listeners: List[
+            Union[weakref.ReferenceType[UpdateListenerType], weakref.WeakMethod]
+        ] = []
 
         # Function to cancel a scheduled retry
         self._async_cancel_retry_setup: Optional[Callable[[], Any]] = None
@@ -173,12 +205,14 @@ class ConfigEntry:
         tries: int = 0,
     ) -> None:
         """Set up an entry."""
-        if self.source == SOURCE_IGNORE:
+        if self.source == SOURCE_IGNORE or self.disabled_by:
             return
 
         if integration is None:
             integration = await loader.async_get_integration(opp, self.domain)
 
+        self.supports_unload = await support_entry_unload(opp, self.domain)
+
         try:
             component = integration.get_component()
         except ImportError as err:
@@ -211,9 +245,7 @@ class ConfigEntry:
                 return
 
         try:
-            result = await component.async_setup_entry(  # type: ignore
-                opp, self
-            )
+            result = await component.async_setup_entry(opp, self)  # type: ignore
 
             if not isinstance(result, bool):
                 _LOGGER.error(
@@ -224,11 +256,19 @@ class ConfigEntry:
             self.state = ENTRY_STATE_SETUP_RETRY
             wait_time = 2 ** min(tries, 4) * 5
             tries += 1
-            _LOGGER.warning(
-                "Config entry for %s not ready yet. Retrying in %d seconds.",
-                self.domain,
-                wait_time,
-            )
+            if tries == 1:
+                _LOGGER.warning(
+                    "Config entry '%s' for %s integration not ready yet. Retrying in background",
+                    self.title,
+                    self.domain,
+                )
+            else:
+                _LOGGER.debug(
+                    "Config entry '%s' for %s integration not ready yet. Retrying in %d seconds",
+                    self.title,
+                    self.domain,
+                    wait_time,
+                )
 
             async def setup_again(now: Any) -> None:
                 """Run setup again."""
@@ -266,7 +306,15 @@ class ConfigEntry:
             return True
 
         if integration is None:
-            integration = await loader.async_get_integration(opp, self.domain)
+            try:
+                integration = await loader.async_get_integration(opp, self.domain)
+            except loader.IntegrationNotFound:
+                # The integration was likely a custom_component
+                # that was uninstalled, or an integration
+                # that has been renamed without removing the config
+                # entry.
+                self.state = ENTRY_STATE_NOT_LOADED
+                return True
 
         component = integration.get_component()
 
@@ -290,9 +338,7 @@ class ConfigEntry:
             return False
 
         try:
-            result = await component.async_unload_entry(  # type: ignore
-                opp, self
-            )
+            result = await component.async_unload_entry(opp, self)  # type: ignore
 
             assert isinstance(result, bool)
 
@@ -314,14 +360,20 @@ class ConfigEntry:
         if self.source == SOURCE_IGNORE:
             return
 
-        integration = await loader.async_get_integration(opp, self.domain)
+        try:
+            integration = await loader.async_get_integration(opp, self.domain)
+        except loader.IntegrationNotFound:
+            # The integration was likely a custom_component
+            # that was uninstalled, or an integration
+            # that has been renamed without removing the config
+            # entry.
+            return
+
         component = integration.get_component()
         if not hasattr(component, "async_remove_entry"):
             return
         try:
-            await component.async_remove_entry(  # type: ignore
-                opp, self
-            )
+            await component.async_remove_entry(opp, self)  # type: ignore
         except Exception:  # pylint: disable=broad-except
             _LOGGER.exception(
                 "Error calling entry remove callback %s for %s",
@@ -359,9 +411,7 @@ class ConfigEntry:
             return False
 
         try:
-            result = await component.async_migrate_entry(  # type: ignore
-                opp, self
-            )
+            result = await component.async_migrate_entry(opp, self)  # type: ignore
             if not isinstance(result, bool):
                 _LOGGER.error(
                     "%s.async_migrate_entry did not return boolean", self.domain
@@ -377,14 +427,17 @@ class ConfigEntry:
             )
             return False
 
-    def add_update_listener(self, listener: Callable) -> Callable:
+    def add_update_listener(self, listener: UpdateListenerType) -> CALLBACK_TYPE:
         """Listen for when entry is updated.
 
-        Listener: Callback function(opp, entry)
-
         Returns function to unlisten.
         """
-        weak_listener = weakref.ref(listener)
+        weak_listener: Any
+        # weakref.ref is not applicable to a bound method, e.g. method of a class instance, as reference will die immediately
+        if hasattr(listener, "__self__"):
+            weak_listener = weakref.WeakMethod(cast(MethodType, listener))
+        else:
+            weak_listener = weakref.ref(listener)
         self.update_listeners.append(weak_listener)
 
         return lambda: self.update_listeners.remove(weak_listener)
@@ -396,12 +449,13 @@ class ConfigEntry:
             "version": self.version,
             "domain": self.domain,
             "title": self.title,
-            "data": self.data,
-            "options": self.options,
+            "data": dict(self.data),
+            "options": dict(self.options),
             "system_options": self.system_options.as_dict(),
             "source": self.source,
             "connection_class": self.connection_class,
             "unique_id": self.unique_id,
+            "disabled_by": self.disabled_by,
         }
 
 
@@ -448,6 +502,10 @@ class ConfigEntriesFlowManager(data_entry_flow.FlowManager):
                 ):
                     self.async_abort(progress_flow["flow_id"])
 
+            # Reset unique ID when the default discovery ID has been used
+            if flow.unique_id == DEFAULT_DISCOVERY_UNIQUE_ID:
+                await flow.async_set_unique_id(None)
+
             # Find existing entry.
             for check_entry in self.config_entries.async_entries(result["handler"]):
                 if check_entry.unique_id == flow.unique_id:
@@ -485,16 +543,16 @@ class ConfigEntriesFlowManager(data_entry_flow.FlowManager):
 
     async def async_create_flow(
         self, handler_key: Any, *, context: Optional[Dict] = None, data: Any = None
-    ) -> "ConfigFlow":
+    ) -> ConfigFlow:
         """Create a flow for specified handler.
 
         Handler key is the domain of the component that we want to set up.
         """
         try:
             integration = await loader.async_get_integration(self.opp, handler_key)
-        except loader.IntegrationNotFound:
+        except loader.IntegrationNotFound as err:
             _LOGGER.error("Cannot find integration %s", handler_key)
-            raise data_entry_flow.UnknownHandler
+            raise data_entry_flow.UnknownHandler from err
 
         # Make sure requirements and dependencies of component are resolved
         await async_process_deps_reqs(self.opp, self._opp_config, integration)
@@ -534,10 +592,19 @@ class ConfigEntriesFlowManager(data_entry_flow.FlowManager):
                 title="New devices discovered",
                 message=(
                     "We have discovered new devices on your network. "
-                    "[Check it out](/config/integrations)"
+                    "[Check it out](/config/integrations)."
                 ),
                 notification_id=DISCOVERY_NOTIFICATION_ID,
             )
+        elif source == SOURCE_REAUTH:
+            self.opp.components.persistent_notification.async_create(
+                title="Integration requires reconfiguration",
+                message=(
+                    "At least one of your integrations requires reconfiguration to "
+                    "continue functioning. [Check it out](/config/integrations)."
+                ),
+                notification_id=RECONFIGURE_NOTIFICATION_ID,
+            )
 
 
 class ConfigEntries:
@@ -659,6 +726,8 @@ class ConfigEntries:
                 system_options=entry.get("system_options", {}),
                 # New in 0.104
                 unique_id=entry.get("unique_id"),
+                # New in 2021.3
+                disabled_by=entry.get("disabled_by"),
             )
             for entry in config["entries"]
         ]
@@ -707,42 +776,110 @@ class ConfigEntries:
 
         If an entry was not loaded, will just load.
         """
+        entry = self.async_get_entry(entry_id)
+
+        if entry is None:
+            raise UnknownEntry
+
         unload_result = await self.async_unload(entry_id)
 
-        if not unload_result:
+        if not unload_result or entry.disabled_by:
             return unload_result
 
         return await self.async_setup(entry_id)
 
+    async def async_set_disabled_by(
+        self, entry_id: str, disabled_by: Optional[str]
+    ) -> bool:
+        """Disable an entry.
+
+        If disabled_by is changed, the config entry will be reloaded.
+        """
+        entry = self.async_get_entry(entry_id)
+
+        if entry is None:
+            raise UnknownEntry
+
+        if entry.disabled_by == disabled_by:
+            return True
+
+        entry.disabled_by = disabled_by
+        self._async_schedule_save()
+
+        dev_reg = device_registry.async_get(self.opp)
+        ent_reg = entity_registry.async_get(self.opp)
+
+        if not entry.disabled_by:
+            # The config entry will no longer be disabled, enable devices and entities
+            device_registry.async_config_entry_disabled_by_changed(dev_reg, entry)
+            entity_registry.async_config_entry_disabled_by_changed(ent_reg, entry)
+
+        # Load or unload the config entry
+        reload_result = await self.async_reload(entry_id)
+
+        if entry.disabled_by:
+            # The config entry has been disabled, disable devices and entities
+            device_registry.async_config_entry_disabled_by_changed(dev_reg, entry)
+            entity_registry.async_config_entry_disabled_by_changed(ent_reg, entry)
+
+        return reload_result
+
     @callback
     def async_update_entry(
         self,
         entry: ConfigEntry,
         *,
-        unique_id: Union[str, dict, None] = _UNDEF,
-        data: dict = _UNDEF,
-        options: dict = _UNDEF,
-        system_options: dict = _UNDEF,
-    ) -> None:
-        """Update a config entry."""
-        if unique_id is not _UNDEF:
+        unique_id: Union[str, dict, None, UndefinedType] = UNDEFINED,
+        title: Union[str, dict, UndefinedType] = UNDEFINED,
+        data: Union[dict, UndefinedType] = UNDEFINED,
+        options: Union[dict, UndefinedType] = UNDEFINED,
+        system_options: Union[dict, UndefinedType] = UNDEFINED,
+    ) -> bool:
+        """Update a config entry.
+
+        If the entry was changed, the update_listeners are
+        fired and this function returns True
+
+        If the entry was not changed, the update_listeners are
+        not fired and this function returns False
+        """
+        changed = False
+
+        if unique_id is not UNDEFINED and entry.unique_id != unique_id:
+            changed = True
             entry.unique_id = cast(Optional[str], unique_id)
 
-        if data is not _UNDEF:
-            entry.data = data
+        if title is not UNDEFINED and entry.title != title:
+            changed = True
+            entry.title = cast(str, title)
 
-        if options is not _UNDEF:
-            entry.options = options
+        if data is not UNDEFINED and entry.data != data:  # type: ignore
+            changed = True
+            entry.data = MappingProxyType(data)
 
-        if system_options is not _UNDEF:
+        if options is not UNDEFINED and entry.options != options:  # type: ignore
+            changed = True
+            entry.options = MappingProxyType(options)
+
+        if (
+            system_options is not UNDEFINED
+            and entry.system_options.as_dict() != system_options
+        ):
+            changed = True
             entry.system_options.update(**system_options)
 
+        if not changed:
+            return False
+
         for listener_ref in entry.update_listeners:
             listener = listener_ref()
-            self.opp.async_create_task(listener(self.opp, entry))
+            if listener is not None:
+                self.opp.async_create_task(listener(self.opp, entry))
 
         self._async_schedule_save()
 
+        return True
+
     async def async_forward_entry_setup(self, entry: ConfigEntry, domain: str) -> bool:
         """Forward the setup of an entry to a different component.
 
@@ -805,7 +942,6 @@ class ConfigFlow(data_entry_flow.FlowHandler):
     @property
     def unique_id(self) -> Optional[str]:
         """Return unique ID if available."""
-        # pylint: disable=no-member
         if not self.context:
             return None
 
@@ -813,66 +949,94 @@ class ConfigFlow(data_entry_flow.FlowHandler):
 
     @staticmethod
     @callback
-    def async_get_options_flow(config_entry: ConfigEntry) -> "OptionsFlow":
+    def async_get_options_flow(config_entry: ConfigEntry) -> OptionsFlow:
         """Get the options flow for this handler."""
         raise data_entry_flow.UnknownHandler
 
     @callback
-    def _abort_if_unique_id_configured(self, updates: Dict[Any, Any] = None) -> None:
+    def _abort_if_unique_id_configured(
+        self,
+        updates: Optional[Dict[Any, Any]] = None,
+        reload_on_update: bool = True,
+    ) -> None:
         """Abort if the unique ID is already configured."""
-        assert self.opp
         if self.unique_id is None:
             return
 
-        for entry in self._async_current_entries():
+        for entry in self._async_current_entries(include_ignore=True):
             if entry.unique_id == self.unique_id:
-                if updates is not None and not updates.items() <= entry.data.items():
-                    self.opp.config_entries.async_update_entry(
+                if updates is not None:
+                    changed = self.opp.config_entries.async_update_entry(
                         entry, data={**entry.data, **updates}
                     )
+                    if (
+                        changed
+                        and reload_on_update
+                        and entry.state in (ENTRY_STATE_LOADED, ENTRY_STATE_SETUP_RETRY)
+                    ):
+                        self.opp.async_create_task(
+                            self.opp.config_entries.async_reload(entry.entry_id)
+                        )
+                # Allow ignored entries to be configured on manual user step
+                if entry.source == SOURCE_IGNORE and self.source == SOURCE_USER:
+                    continue
                 raise data_entry_flow.AbortFlow("already_configured")
 
     async def async_set_unique_id(
-        self, unique_id: str, *, raise_on_progress: bool = True
+        self, unique_id: Optional[str] = None, *, raise_on_progress: bool = True
     ) -> Optional[ConfigEntry]:
         """Set a unique ID for the config flow.
 
         Returns optionally existing config entry with same ID.
         """
+        if unique_id is None:
+            self.context["unique_id"] = None
+            return None
+
         if raise_on_progress:
             for progress in self._async_in_progress():
                 if progress["context"].get("unique_id") == unique_id:
                     raise data_entry_flow.AbortFlow("already_in_progress")
 
-        # pylint: disable=no-member
         self.context["unique_id"] = unique_id
 
-        for entry in self._async_current_entries():
+        # Abort discoveries done using the default discovery unique id
+        if unique_id != DEFAULT_DISCOVERY_UNIQUE_ID:
+            for progress in self._async_in_progress():
+                if progress["context"].get("unique_id") == DEFAULT_DISCOVERY_UNIQUE_ID:
+                    self.opp.config_entries.flow.async_abort(progress["flow_id"])
+
+        for entry in self._async_current_entries(include_ignore=True):
             if entry.unique_id == unique_id:
                 return entry
 
         return None
 
     @callback
-    def _async_current_entries(self) -> List[ConfigEntry]:
-        """Return current entries."""
-        assert self.opp is not None
-        return self.opp.config_entries.async_entries(self.handler)
+    def _async_current_entries(self, include_ignore: bool = False) -> List[ConfigEntry]:
+        """Return current entries.
+
+        If the flow is user initiated, filter out ignored entries unless include_ignore is True.
+        """
+        config_entries = self.opp.config_entries.async_entries(self.handler)
+
+        if include_ignore or self.source != SOURCE_USER:
+            return config_entries
+
+        return [entry for entry in config_entries if entry.source != SOURCE_IGNORE]
 
     @callback
     def _async_current_ids(self, include_ignore: bool = True) -> Set[Optional[str]]:
         """Return current unique IDs."""
-        assert self.opp is not None
-        return set(
+        return {
             entry.unique_id
             for entry in self.opp.config_entries.async_entries(self.handler)
             if include_ignore or entry.source != SOURCE_IGNORE
-        )
+        }
 
     @callback
     def _async_in_progress(self) -> List[Dict]:
         """Return other in progress flows for current domain."""
-        assert self.opp is not None
         return [
             flw
             for flw in self.opp.config_entries.flow.async_progress()
@@ -882,12 +1046,75 @@ class ConfigFlow(data_entry_flow.FlowHandler):
     async def async_step_ignore(self, user_input: Dict[str, Any]) -> Dict[str, Any]:
         """Ignore this config flow."""
         await self.async_set_unique_id(user_input["unique_id"], raise_on_progress=False)
-        return self.async_create_entry(title="Ignored", data={})
+        return self.async_create_entry(title=user_input["title"], data={})
 
     async def async_step_unignore(self, user_input: Dict[str, Any]) -> Dict[str, Any]:
         """Rediscover a config entry by it's unique_id."""
         return self.async_abort(reason="not_implemented")
 
+    async def async_step_user(
+        self, user_input: Optional[Dict[str, Any]] = None
+    ) -> Dict[str, Any]:
+        """Handle a flow initiated by the user."""
+        return self.async_abort(reason="not_implemented")
+
+    async def _async_handle_discovery_without_unique_id(self) -> None:
+        """Mark this flow discovered, without a unique identifier.
+
+        If a flow initiated by discovery, doesn't have a unique ID, this can
+        be used alternatively. It will ensure only 1 flow is started and only
+        when the handler has no existing config entries.
+
+        It ensures that the discovery can be ignored by the user.
+        """
+        if self.unique_id is not None:
+            return
+
+        # Abort if the handler has config entries already
+        if self._async_current_entries():
+            raise data_entry_flow.AbortFlow("already_configured")
+
+        # Use an special unique id to differentiate
+        await self.async_set_unique_id(DEFAULT_DISCOVERY_UNIQUE_ID)
+        self._abort_if_unique_id_configured()
+
+        # Abort if any other flow for this handler is already in progress
+        if self._async_in_progress():
+            raise data_entry_flow.AbortFlow("already_in_progress")
+
+    async def async_step_discovery(
+        self, discovery_info: Dict[str, Any]
+    ) -> Dict[str, Any]:
+        """Handle a flow initialized by discovery."""
+        await self._async_handle_discovery_without_unique_id()
+        return await self.async_step_user()
+
+    @callback
+    def async_abort(
+        self, *, reason: str, description_placeholders: Optional[Dict] = None
+    ) -> Dict[str, Any]:
+        """Abort the config flow."""
+        # Remove reauth notification if no reauth flows are in progress
+        if self.source == SOURCE_REAUTH and not any(
+            ent["context"]["source"] == SOURCE_REAUTH
+            for ent in self.opp.config_entries.flow.async_progress()
+            if ent["flow_id"] != self.flow_id
+        ):
+            self.opp.components.persistent_notification.async_dismiss(
+                RECONFIGURE_NOTIFICATION_ID
+            )
+
+        return super().async_abort(
+            reason=reason, description_placeholders=description_placeholders
+        )
+
+    async_step_oppio = async_step_discovery
+    async_step_homekit = async_step_discovery
+    async_step_mqtt = async_step_discovery
+    async_step_ssdp = async_step_discovery
+    async_step_zeroconf = async_step_discovery
+    async_step_dhcp = async_step_discovery
+
 
 class OptionsFlowManager(data_entry_flow.FlowManager):
     """Flow to set options for a configuration entry."""
@@ -898,7 +1125,7 @@ class OptionsFlowManager(data_entry_flow.FlowManager):
         *,
         context: Optional[Dict[str, Any]] = None,
         data: Optional[Dict[str, Any]] = None,
-    ) -> "OptionsFlow":
+    ) -> OptionsFlow:
         """Create an options flow for a config entry.
 
         Entry_id and flow.handler is the same thing to map entry with flow.
@@ -910,8 +1137,7 @@ class OptionsFlowManager(data_entry_flow.FlowManager):
         if entry.domain not in HANDLERS:
             raise data_entry_flow.UnknownHandler
 
-        flow = cast(OptionsFlow, HANDLERS[entry.domain].async_get_options_flow(entry))
-        return flow
+        return cast(OptionsFlow, HANDLERS[entry.domain].async_get_options_flow(entry))
 
     async def async_finish_flow(
         self, flow: data_entry_flow.FlowHandler, result: Dict[str, Any]
@@ -922,10 +1148,14 @@ class OptionsFlowManager(data_entry_flow.FlowManager):
         """
         flow = cast(OptionsFlow, flow)
 
+        if result["type"] != data_entry_flow.RESULT_TYPE_CREATE_ENTRY:
+            return result
+
         entry = self.opp.config_entries.async_get_entry(flow.handler)
         if entry is None:
             raise UnknownEntry(flow.handler)
-        self.opp.config_entries.async_update_entry(entry, options=result["data"])
+        if result["data"] is not None:
+            self.opp.config_entries.async_update_entry(entry, options=result["data"])
 
         result["result"] = True
         return result
@@ -941,7 +1171,7 @@ class OptionsFlow(data_entry_flow.FlowHandler):
 class SystemOptions:
     """Config entry system options."""
 
-    disable_new_entities = attr.ib(type=bool, default=False)
+    disable_new_entities: bool = attr.ib(default=False)
 
     def update(self, *, disable_new_entities: bool) -> None:
         """Update properties."""
@@ -955,8 +1185,6 @@ class SystemOptions:
 class EntityRegistryDisabledHandler:
     """Handler to handle when entities related to config entries updating disabled_by."""
 
-    RELOAD_AFTER_UPDATE_DELAY = 30
-
     def __init__(self, opp: OpenPeerPower) -> None:
         """Initialize the handler."""
         self.opp = opp
@@ -968,17 +1196,13 @@ class EntityRegistryDisabledHandler:
     def async_setup(self) -> None:
         """Set up the disable handler."""
         self.opp.bus.async_listen(
-            entity_registry.EVENT_ENTITY_REGISTRY_UPDATED, self._handle_entry_updated
+            entity_registry.EVENT_ENTITY_REGISTRY_UPDATED,
+            self._handle_entry_updated,
+            event_filter=_handle_entry_updated_filter,
         )
 
     async def _handle_entry_updated(self, event: Event) -> None:
         """Handle entity registry entry update."""
-        if (
-            event.data["action"] != "update"
-            or "disabled_by" not in event.data["changes"]
-        ):
-            return
-
         if self.registry is None:
             self.registry = await entity_registry.async_get_registry(self.opp)
 
@@ -1000,9 +1224,7 @@ class EntityRegistryDisabledHandler:
         )
         assert config_entry is not None
 
-        if config_entry.entry_id not in self.changed and await support_entry_unload(
-            self.opp, config_entry.domain
-        ):
+        if config_entry.entry_id not in self.changed and config_entry.supports_unload:
             self.changed.add(config_entry.entry_id)
 
         if not self.changed:
@@ -1015,7 +1237,7 @@ class EntityRegistryDisabledHandler:
             self._remove_call_later()
 
         self._remove_call_later = self.opp.helpers.event.async_call_later(
-            self.RELOAD_AFTER_UPDATE_DELAY, self._handle_reload
+            RELOAD_AFTER_UPDATE_DELAY, self._handle_reload
         )
 
     async def _handle_reload(self, _now: Any) -> None:
@@ -1034,6 +1256,22 @@ class EntityRegistryDisabledHandler:
         )
 
 
+@callback
+def _handle_entry_updated_filter(event: Event) -> bool:
+    """Handle entity registry entry update filter.
+
+    Only handle changes to "disabled_by".
+    If "disabled_by" was DISABLED_CONFIG_ENTRY, reload is not needed.
+    """
+    if (
+        event.data["action"] != "update"
+        or "disabled_by" not in event.data["changes"]
+        or event.data["changes"]["disabled_by"] == entity_registry.DISABLED_CONFIG_ENTRY
+    ):
+        return False
+    return True
+
+
 async def support_entry_unload(opp: OpenPeerPower, domain: str) -> bool:
     """Test if a domain supports entry unloading."""
     integration = await loader.async_get_integration(opp, domain)
