@@ -4,8 +4,9 @@ from collections import deque
 from datetime import datetime, timedelta
 import functools as ft
 import logging
+import re
 import sys
-from typing import Callable, Container, Optional, Set, Union, cast
+from typing import Any, Callable, Container, List, Optional, Set, Union, cast
 
 from openpeerpower.components import zone as zone_cmp
 from openpeerpower.components.device_automation import (
@@ -17,6 +18,7 @@ from openpeerpower.const import (
     ATTR_LONGITUDE,
     CONF_ABOVE,
     CONF_AFTER,
+    CONF_ATTRIBUTE,
     CONF_BEFORE,
     CONF_BELOW,
     CONF_CONDITION,
@@ -34,7 +36,14 @@ from openpeerpower.const import (
     WEEKDAYS,
 )
 from openpeerpower.core import OpenPeerPower, State, callback
-from openpeerpower.exceptions import OpenPeerPowerError, TemplateError
+from openpeerpower.exceptions import (
+    ConditionError,
+    ConditionErrorContainer,
+    ConditionErrorIndex,
+    ConditionErrorMessage,
+    OpenPeerPowerError,
+    TemplateError,
+)
 import openpeerpower.helpers.config_validation as cv
 from openpeerpower.helpers.sun import get_astral_event_date
 from openpeerpower.helpers.template import Template
@@ -47,30 +56,38 @@ ASYNC_FROM_CONFIG_FORMAT = "async_{}_from_config"
 
 _LOGGER = logging.getLogger(__name__)
 
+INPUT_ENTITY_ID = re.compile(
+    r"^input_(?:select|text|number|boolean|datetime)\.(?!.+__)(?!_)[\da-z_]+(?<!_)$"
+)
+
 ConditionCheckerType = Callable[[OpenPeerPower, TemplateVarsType], bool]
 
 
 async def async_from_config(
-    opp: OpenPeerPower, config: ConfigType, config_validation: bool = True
+    opp: OpenPeerPower,
+    config: Union[ConfigType, Template],
+    config_validation: bool = True,
 ) -> ConditionCheckerType:
     """Turn a condition configuration into a method.
 
     Should be run on the event loop.
     """
+    if isinstance(config, Template):
+        # We got a condition template, wrap it in a configuration to pass along.
+        config = {
+            CONF_CONDITION: "template",
+            CONF_VALUE_TEMPLATE: config,
+        }
+
+    condition = config.get(CONF_CONDITION)
     for fmt in (ASYNC_FROM_CONFIG_FORMAT, FROM_CONFIG_FORMAT):
-        factory = getattr(
-            sys.modules[__name__], fmt.format(config.get(CONF_CONDITION)), None
-        )
+        factory = getattr(sys.modules[__name__], fmt.format(condition), None)
 
         if factory:
             break
 
     if factory is None:
-        raise OpenPeerPowerError(
-            'Invalid condition "{}" specified {}'.format(
-                config.get(CONF_CONDITION), config
-            )
-        )
+        raise OpenPeerPowerError(f'Invalid condition "{condition}" specified {config}')
 
     # Check for partials to properly determine if coroutine function
     check_factory = factory
@@ -78,9 +95,7 @@ async def async_from_config(
         check_factory = check_factory.func
 
     if asyncio.iscoroutinefunction(check_factory):
-        return cast(
-            ConditionCheckerType, await factory(opp, config, config_validation)
-        )
+        return cast(ConditionCheckerType, await factory(opp, config, config_validation))
     return cast(ConditionCheckerType, factory(config, config_validation))
 
 
@@ -98,13 +113,19 @@ async def async_and_from_config(
         opp: OpenPeerPower, variables: TemplateVarsType = None
     ) -> bool:
         """Test and condition."""
-        try:
-            for check in checks:
+        errors = []
+        for index, check in enumerate(checks):
+            try:
                 if not check(opp, variables):
                     return False
-        except Exception as ex:  # pylint: disable=broad-except
-            _LOGGER.warning("Error during and-condition: %s", ex)
-            return False
+            except ConditionError as ex:
+                errors.append(
+                    ConditionErrorIndex("and", index=index, total=len(checks), error=ex)
+                )
+
+        # Raise the errors if no check was false
+        if errors:
+            raise ConditionErrorContainer("and", errors=errors)
 
         return True
 
@@ -121,90 +142,179 @@ async def async_or_from_config(
         await async_from_config(opp, entry, False) for entry in config["conditions"]
     ]
 
-    def if_or_condition(
-        opp: OpenPeerPower, variables: TemplateVarsType = None
-    ) -> bool:
-        """Test and condition."""
-        try:
-            for check in checks:
+    def if_or_condition(opp: OpenPeerPower, variables: TemplateVarsType = None) -> bool:
+        """Test or condition."""
+        errors = []
+        for index, check in enumerate(checks):
+            try:
                 if check(opp, variables):
                     return True
-        except Exception as ex:  # pylint: disable=broad-except
-            _LOGGER.warning("Error during or-condition: %s", ex)
+            except ConditionError as ex:
+                errors.append(
+                    ConditionErrorIndex("or", index=index, total=len(checks), error=ex)
+                )
+
+        # Raise the errors if no check was true
+        if errors:
+            raise ConditionErrorContainer("or", errors=errors)
 
         return False
 
     return if_or_condition
 
 
+async def async_not_from_config(
+    opp: OpenPeerPower, config: ConfigType, config_validation: bool = True
+) -> ConditionCheckerType:
+    """Create multi condition matcher using 'NOT'."""
+    if config_validation:
+        config = cv.NOT_CONDITION_SCHEMA(config)
+    checks = [
+        await async_from_config(opp, entry, False) for entry in config["conditions"]
+    ]
+
+    def if_not_condition(
+        opp: OpenPeerPower, variables: TemplateVarsType = None
+    ) -> bool:
+        """Test not condition."""
+        errors = []
+        for index, check in enumerate(checks):
+            try:
+                if check(opp, variables):
+                    return False
+            except ConditionError as ex:
+                errors.append(
+                    ConditionErrorIndex("not", index=index, total=len(checks), error=ex)
+                )
+
+        # Raise the errors if no check was true
+        if errors:
+            raise ConditionErrorContainer("not", errors=errors)
+
+        return True
+
+    return if_not_condition
+
+
 def numeric_state(
     opp: OpenPeerPower,
     entity: Union[None, str, State],
-    below: Optional[float] = None,
-    above: Optional[float] = None,
+    below: Optional[Union[float, str]] = None,
+    above: Optional[Union[float, str]] = None,
     value_template: Optional[Template] = None,
     variables: TemplateVarsType = None,
 ) -> bool:
     """Test a numeric state condition."""
-    return cast(
-        bool,
-        run_callback_threadsafe(
-            opp.loop,
-            async_numeric_state,
-            opp,
-            entity,
-            below,
-            above,
-            value_template,
-            variables,
-        ).result(),
-    )
+    return run_callback_threadsafe(
+        opp.loop,
+        async_numeric_state,
+        opp,
+        entity,
+        below,
+        above,
+        value_template,
+        variables,
+    ).result()
 
 
 def async_numeric_state(
     opp: OpenPeerPower,
     entity: Union[None, str, State],
-    below: Optional[float] = None,
-    above: Optional[float] = None,
+    below: Optional[Union[float, str]] = None,
+    above: Optional[Union[float, str]] = None,
     value_template: Optional[Template] = None,
     variables: TemplateVarsType = None,
+    attribute: Optional[str] = None,
 ) -> bool:
     """Test a numeric state condition."""
+    if entity is None:
+        raise ConditionErrorMessage("numeric_state", "no entity specified")
+
     if isinstance(entity, str):
+        entity_id = entity
         entity = opp.states.get(entity)
 
-    if entity is None:
-        return False
+        if entity is None:
+            raise ConditionErrorMessage("numeric_state", f"unknown entity {entity_id}")
+    else:
+        entity_id = entity.entity_id
 
+    if attribute is not None and attribute not in entity.attributes:
+        raise ConditionErrorMessage(
+            "numeric_state",
+            f"attribute '{attribute}' (of entity {entity_id}) does not exist",
+        )
+
+    value: Any = None
     if value_template is None:
-        value = entity.state
+        if attribute is None:
+            value = entity.state
+        else:
+            value = entity.attributes.get(attribute)
     else:
         variables = dict(variables or {})
         variables["state"] = entity
         try:
             value = value_template.async_render(variables)
         except TemplateError as ex:
-            _LOGGER.error("Template error: %s", ex)
-            return False
+            raise ConditionErrorMessage(
+                "numeric_state", f"template error: {ex}"
+            ) from ex
 
     if value in (STATE_UNAVAILABLE, STATE_UNKNOWN):
-        return False
+        raise ConditionErrorMessage(
+            "numeric_state", f"state of {entity_id} is unavailable"
+        )
 
     try:
         fvalue = float(value)
-    except ValueError:
-        _LOGGER.warning(
-            "Value cannot be processed as a number: %s (Offending entity: %s)",
-            entity,
-            value,
-        )
-        return False
+    except (ValueError, TypeError) as ex:
+        raise ConditionErrorMessage(
+            "numeric_state",
+            f"entity {entity_id} state '{value}' cannot be processed as a number",
+        ) from ex
 
-    if below is not None and fvalue >= below:
-        return False
+    if below is not None:
+        if isinstance(below, str):
+            below_entity = opp.states.get(below)
+            if not below_entity or below_entity.state in (
+                STATE_UNAVAILABLE,
+                STATE_UNKNOWN,
+            ):
+                raise ConditionErrorMessage(
+                    "numeric_state", f"the 'below' entity {below} is unavailable"
+                )
+            try:
+                if fvalue >= float(below_entity.state):
+                    return False
+            except (ValueError, TypeError) as ex:
+                raise ConditionErrorMessage(
+                    "numeric_state",
+                    f"the 'below' entity {below} state '{below_entity.state}' cannot be processed as a number",
+                ) from ex
+        elif fvalue >= below:
+            return False
 
-    if above is not None and fvalue <= above:
-        return False
+    if above is not None:
+        if isinstance(above, str):
+            above_entity = opp.states.get(above)
+            if not above_entity or above_entity.state in (
+                STATE_UNAVAILABLE,
+                STATE_UNKNOWN,
+            ):
+                raise ConditionErrorMessage(
+                    "numeric_state", f"the 'above' entity {above} is unavailable"
+                )
+            try:
+                if fvalue <= float(above_entity.state):
+                    return False
+            except (ValueError, TypeError) as ex:
+                raise ConditionErrorMessage(
+                    "numeric_state",
+                    f"the 'above' entity {above} state '{above_entity.state}' cannot be processed as a number",
+                ) from ex
+        elif fvalue <= above:
+            return False
 
     return True
 
@@ -215,7 +325,8 @@ def async_numeric_state_from_config(
     """Wrap action method with state based condition."""
     if config_validation:
         config = cv.NUMERIC_STATE_CONDITION_SCHEMA(config)
-    entity_id = config.get(CONF_ENTITY_ID)
+    entity_ids = config.get(CONF_ENTITY_ID, [])
+    attribute = config.get(CONF_ATTRIBUTE)
     below = config.get(CONF_BELOW)
     above = config.get(CONF_ABOVE)
     value_template = config.get(CONF_VALUE_TEMPLATE)
@@ -227,9 +338,25 @@ def async_numeric_state_from_config(
         if value_template is not None:
             value_template.opp = opp
 
-        return async_numeric_state(
-            opp, entity_id, below, above, value_template, variables
-        )
+        errors = []
+        for index, entity_id in enumerate(entity_ids):
+            try:
+                if not async_numeric_state(
+                    opp, entity_id, below, above, value_template, variables, attribute
+                ):
+                    return False
+            except ConditionError as ex:
+                errors.append(
+                    ConditionErrorIndex(
+                        "numeric_state", index=index, total=len(entity_ids), error=ex
+                    )
+                )
+
+        # Raise the errors if no check was false
+        if errors:
+            raise ConditionErrorContainer("numeric_state", errors=errors)
+
+        return True
 
     return if_numeric_state
 
@@ -237,21 +364,57 @@ def async_numeric_state_from_config(
 def state(
     opp: OpenPeerPower,
     entity: Union[None, str, State],
-    req_state: str,
+    req_state: Any,
     for_period: Optional[timedelta] = None,
+    attribute: Optional[str] = None,
 ) -> bool:
     """Test if state matches requirements.
 
     Async friendly.
     """
+    if entity is None:
+        raise ConditionErrorMessage("state", "no entity specified")
+
     if isinstance(entity, str):
+        entity_id = entity
         entity = opp.states.get(entity)
 
-    if entity is None:
-        return False
+        if entity is None:
+            raise ConditionErrorMessage("state", f"unknown entity {entity_id}")
+    else:
+        entity_id = entity.entity_id
+
+    if attribute is not None and attribute not in entity.attributes:
+        raise ConditionErrorMessage(
+            "state", f"attribute '{attribute}' (of entity {entity_id}) does not exist"
+        )
+
     assert isinstance(entity, State)
 
-    is_state = entity.state == req_state
+    if attribute is None:
+        value: Any = entity.state
+    else:
+        value = entity.attributes.get(attribute)
+
+    if not isinstance(req_state, list):
+        req_state = [req_state]
+
+    is_state = False
+    for req_state_value in req_state:
+        state_value = req_state_value
+        if (
+            isinstance(req_state_value, str)
+            and INPUT_ENTITY_ID.match(req_state_value) is not None
+        ):
+            state_entity = opp.states.get(req_state_value)
+            if not state_entity:
+                raise ConditionErrorMessage(
+                    "state", f"the 'state' entity {req_state_value} is unavailable"
+                )
+            state_value = state_entity.state
+        is_state = value == state_value
+        if is_state:
+            break
 
     if for_period is None or not is_state:
         return is_state
@@ -265,13 +428,33 @@ def state_from_config(
     """Wrap action method with state based condition."""
     if config_validation:
         config = cv.STATE_CONDITION_SCHEMA(config)
-    entity_id = config.get(CONF_ENTITY_ID)
-    req_state = cast(str, config.get(CONF_STATE))
+    entity_ids = config.get(CONF_ENTITY_ID, [])
+    req_states: Union[str, List[str]] = config.get(CONF_STATE, [])
     for_period = config.get("for")
+    attribute = config.get(CONF_ATTRIBUTE)
+
+    if not isinstance(req_states, list):
+        req_states = [req_states]
 
     def if_state(opp: OpenPeerPower, variables: TemplateVarsType = None) -> bool:
         """Test if condition."""
-        return state(opp, entity_id, req_state, for_period)
+        errors = []
+        for index, entity_id in enumerate(entity_ids):
+            try:
+                if not state(opp, entity_id, req_states, for_period, attribute):
+                    return False
+            except ConditionError as ex:
+                errors.append(
+                    ConditionErrorIndex(
+                        "state", index=index, total=len(entity_ids), error=ex
+                    )
+                )
+
+        # Raise the errors if no check was false
+        if errors:
+            raise ConditionErrorContainer("state", errors=errors)
+
+        return True
 
     return if_state
 
@@ -353,12 +536,9 @@ def template(
     opp: OpenPeerPower, value_template: Template, variables: TemplateVarsType = None
 ) -> bool:
     """Test if template condition matches."""
-    return cast(
-        bool,
-        run_callback_threadsafe(
-            opp.loop, async_template, opp, value_template, variables
-        ).result(),
-    )
+    return run_callback_threadsafe(
+        opp.loop, async_template, opp, value_template, variables
+    ).result()
 
 
 def async_template(
@@ -366,10 +546,9 @@ def async_template(
 ) -> bool:
     """Test if template condition matches."""
     try:
-        value = value_template.async_render(variables)
+        value: str = value_template.async_render(variables, parse_result=False)
     except TemplateError as ex:
-        _LOGGER.error("Error during template condition: %s", ex)
-        return False
+        raise ConditionErrorMessage("template", str(ex)) from ex
 
     return value.lower() == "true"
 
@@ -392,8 +571,9 @@ def async_template_from_config(
 
 
 def time(
-    before: Optional[dt_util.dt.time] = None,
-    after: Optional[dt_util.dt.time] = None,
+    opp: OpenPeerPower,
+    before: Optional[Union[dt_util.dt.time, str]] = None,
+    after: Optional[Union[dt_util.dt.time, str]] = None,
     weekday: Union[None, str, Container[str]] = None,
 ) -> bool:
     """Test if local time condition matches.
@@ -408,8 +588,28 @@ def time(
 
     if after is None:
         after = dt_util.dt.time(0)
+    elif isinstance(after, str):
+        after_entity = opp.states.get(after)
+        if not after_entity:
+            raise ConditionErrorMessage("time", f"unknown 'after' entity {after}")
+        after = dt_util.dt.time(
+            after_entity.attributes.get("hour", 23),
+            after_entity.attributes.get("minute", 59),
+            after_entity.attributes.get("second", 59),
+        )
+
     if before is None:
         before = dt_util.dt.time(23, 59, 59, 999999)
+    elif isinstance(before, str):
+        before_entity = opp.states.get(before)
+        if not before_entity:
+            raise ConditionErrorMessage("time", f"unknown 'before' entity {before}")
+        before = dt_util.dt.time(
+            before_entity.attributes.get("hour", 23),
+            before_entity.attributes.get("minute", 59),
+            before_entity.attributes.get("second", 59),
+            999999,
+        )
 
     if after < before:
         if not after <= now_time < before:
@@ -443,7 +643,7 @@ def time_from_config(
 
     def time_if(opp: OpenPeerPower, variables: TemplateVarsType = None) -> bool:
         """Validate time based if-condition."""
-        return time(before, after, weekday)
+        return time(opp, before, after, weekday)
 
     return time_if
 
@@ -457,23 +657,40 @@ def zone(
 
     Async friendly.
     """
+    if zone_ent is None:
+        raise ConditionErrorMessage("zone", "no zone specified")
+
     if isinstance(zone_ent, str):
+        zone_ent_id = zone_ent
         zone_ent = opp.states.get(zone_ent)
 
-    if zone_ent is None:
-        return False
-
-    if isinstance(entity, str):
-        entity = opp.states.get(entity)
+        if zone_ent is None:
+            raise ConditionErrorMessage("zone", f"unknown zone {zone_ent_id}")
 
     if entity is None:
-        return False
+        raise ConditionErrorMessage("zone", "no entity specified")
+
+    if isinstance(entity, str):
+        entity_id = entity
+        entity = opp.states.get(entity)
+
+        if entity is None:
+            raise ConditionErrorMessage("zone", f"unknown entity {entity_id}")
+    else:
+        entity_id = entity.entity_id
 
     latitude = entity.attributes.get(ATTR_LATITUDE)
     longitude = entity.attributes.get(ATTR_LONGITUDE)
 
-    if latitude is None or longitude is None:
-        return False
+    if latitude is None:
+        raise ConditionErrorMessage(
+            "zone", f"entity {entity_id} has no 'latitude' attribute"
+        )
+
+    if longitude is None:
+        raise ConditionErrorMessage(
+            "zone", f"entity {entity_id} has no 'longitude' attribute"
+        )
 
     return zone_cmp.in_zone(
         zone_ent, latitude, longitude, entity.attributes.get(ATTR_GPS_ACCURACY, 0)
@@ -486,12 +703,36 @@ def zone_from_config(
     """Wrap action method with zone based condition."""
     if config_validation:
         config = cv.ZONE_CONDITION_SCHEMA(config)
-    entity_id = config.get(CONF_ENTITY_ID)
-    zone_entity_id = config.get(CONF_ZONE)
+    entity_ids = config.get(CONF_ENTITY_ID, [])
+    zone_entity_ids = config.get(CONF_ZONE, [])
 
     def if_in_zone(opp: OpenPeerPower, variables: TemplateVarsType = None) -> bool:
         """Test if condition."""
-        return zone(opp, zone_entity_id, entity_id)
+        errors = []
+
+        all_ok = True
+        for entity_id in entity_ids:
+            entity_ok = False
+            for zone_entity_id in zone_entity_ids:
+                try:
+                    if zone(opp, zone_entity_id, entity_id):
+                        entity_ok = True
+                except ConditionErrorMessage as ex:
+                    errors.append(
+                        ConditionErrorMessage(
+                            "zone",
+                            f"error matching {entity_id} with {zone_entity_id}: {ex.message}",
+                        )
+                    )
+
+            if not entity_ok:
+                all_ok = False
+
+        # Raise the errors only if no definitive result was found
+        if errors and not all_ok:
+            raise ConditionErrorContainer("zone", errors=errors)
+
+        return all_ok
 
     return if_in_zone
 
@@ -512,11 +753,14 @@ async def async_device_from_config(
 
 
 async def async_validate_condition_config(
-    opp: OpenPeerPower, config: ConfigType
-) -> ConfigType:
+    opp: OpenPeerPower, config: Union[ConfigType, Template]
+) -> Union[ConfigType, Template]:
     """Validate config."""
+    if isinstance(config, Template):
+        return config
+
     condition = config[CONF_CONDITION]
-    if condition in ("and", "or"):
+    if condition in ("and", "not", "or"):
         conditions = []
         for sub_cond in config["conditions"]:
             sub_cond = await async_validate_condition_config(opp, sub_cond)
@@ -525,6 +769,7 @@ async def async_validate_condition_config(
 
     if condition == "device":
         config = cv.DEVICE_CONDITION_SCHEMA(config)
+        assert not isinstance(config, Template)
         platform = await async_get_device_automation_platform(
             opp, config[CONF_DOMAIN], "condition"
         )
@@ -534,38 +779,47 @@ async def async_validate_condition_config(
 
 
 @callback
-def async_extract_entities(config: ConfigType) -> Set[str]:
+def async_extract_entities(config: Union[ConfigType, Template]) -> Set[str]:
     """Extract entities from a condition."""
-    referenced = set()
+    referenced: Set[str] = set()
     to_process = deque([config])
 
     while to_process:
         config = to_process.popleft()
+        if isinstance(config, Template):
+            continue
+
         condition = config[CONF_CONDITION]
 
-        if condition in ("and", "or"):
+        if condition in ("and", "not", "or"):
             to_process.extend(config["conditions"])
             continue
 
-        entity_id = config.get(CONF_ENTITY_ID)
+        entity_ids = config.get(CONF_ENTITY_ID)
 
-        if entity_id is not None:
-            referenced.add(entity_id)
+        if isinstance(entity_ids, str):
+            entity_ids = [entity_ids]
+
+        if entity_ids is not None:
+            referenced.update(entity_ids)
 
     return referenced
 
 
 @callback
-def async_extract_devices(config: ConfigType) -> Set[str]:
+def async_extract_devices(config: Union[ConfigType, Template]) -> Set[str]:
     """Extract devices from a condition."""
     referenced = set()
     to_process = deque([config])
 
     while to_process:
         config = to_process.popleft()
+        if isinstance(config, Template):
+            continue
+
         condition = config[CONF_CONDITION]
 
-        if condition in ("and", "or"):
+        if condition in ("and", "not", "or"):
             to_process.extend(config["conditions"])
             continue
 
