@@ -7,11 +7,20 @@ The Entity Registry will persist itself 10 seconds after a new entity is
 registered. Registering a new entity while a timer is in progress resets the
 timer.
 """
-import asyncio
 from collections import OrderedDict
-from itertools import chain
 import logging
-from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Optional, cast
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Dict,
+    Iterable,
+    List,
+    Optional,
+    Tuple,
+    Union,
+    cast,
+)
 
 import attr
 
@@ -19,61 +28,72 @@ from openpeerpower.const import (
     ATTR_DEVICE_CLASS,
     ATTR_FRIENDLY_NAME,
     ATTR_ICON,
+    ATTR_RESTORED,
     ATTR_SUPPORTED_FEATURES,
     ATTR_UNIT_OF_MEASUREMENT,
     EVENT_OPENPEERPOWER_START,
     STATE_UNAVAILABLE,
 )
 from openpeerpower.core import Event, callback, split_entity_id, valid_entity_id
+from openpeerpower.helpers import device_registry as dr
 from openpeerpower.helpers.device_registry import EVENT_DEVICE_REGISTRY_UPDATED
 from openpeerpower.loader import bind_opp
-from openpeerpower.util import ensure_unique_string, slugify
+from openpeerpower.util import slugify
 from openpeerpower.util.yaml import load_yaml
 
-from .typing import OpenPeerPowerType
+from .typing import UNDEFINED, OpenPeerPowerType, UndefinedType
 
 if TYPE_CHECKING:
-    from openpeerpower.config_entries import ConfigEntry  # noqa: F401
-
-# mypy: allow-untyped-defs, no-check-untyped-defs
+    from openpeerpower.config_entries import ConfigEntry
 
 PATH_REGISTRY = "entity_registry.yaml"
 DATA_REGISTRY = "entity_registry"
 EVENT_ENTITY_REGISTRY_UPDATED = "entity_registry_updated"
 SAVE_DELAY = 10
 _LOGGER = logging.getLogger(__name__)
-_UNDEF = object()
 DISABLED_CONFIG_ENTRY = "config_entry"
+DISABLED_DEVICE = "device"
 DISABLED_OPP = "opp"
-DISABLED_USER = "user"
 DISABLED_INTEGRATION = "integration"
-
-ATTR_RESTORED = "restored"
+DISABLED_USER = "user"
 
 STORAGE_VERSION = 1
 STORAGE_KEY = "core.entity_registry"
 
+# Attributes relevant to describing entity
+# to external services.
+ENTITY_DESCRIBING_ATTRIBUTES = {
+    "entity_id",
+    "name",
+    "original_name",
+    "capabilities",
+    "supported_features",
+    "device_class",
+    "unit_of_measurement",
+}
+
 
 @attr.s(slots=True, frozen=True)
 class RegistryEntry:
     """Entity Registry Entry."""
 
-    entity_id = attr.ib(type=str)
-    unique_id = attr.ib(type=str)
-    platform = attr.ib(type=str)
-    name = attr.ib(type=str, default=None)
-    icon = attr.ib(type=str, default=None)
+    entity_id: str = attr.ib()
+    unique_id: str = attr.ib()
+    platform: str = attr.ib()
+    name: Optional[str] = attr.ib(default=None)
+    icon: Optional[str] = attr.ib(default=None)
     device_id: Optional[str] = attr.ib(default=None)
+    area_id: Optional[str] = attr.ib(default=None)
     config_entry_id: Optional[str] = attr.ib(default=None)
-    disabled_by = attr.ib(
-        type=Optional[str],
+    disabled_by: Optional[str] = attr.ib(
         default=None,
         validator=attr.validators.in_(
             (
-                DISABLED_OPP,
-                DISABLED_USER,
-                DISABLED_INTEGRATION,
                 DISABLED_CONFIG_ENTRY,
+                DISABLED_DEVICE,
+                DISABLED_OPP,
+                DISABLED_INTEGRATION,
+                DISABLED_USER,
                 None,
             )
         ),
@@ -85,7 +105,7 @@ class RegistryEntry:
     # As set by integration
     original_name: Optional[str] = attr.ib(default=None)
     original_icon: Optional[str] = attr.ib(default=None)
-    domain = attr.ib(type=str, init=False, repr=False)
+    domain: str = attr.ib(init=False, repr=False)
 
     @domain.default
     def _domain_default(self) -> str:
@@ -97,6 +117,33 @@ class RegistryEntry:
         """Return if entry is disabled."""
         return self.disabled_by is not None
 
+    @callback
+    def write_unavailable_state(self, opp: OpenPeerPowerType) -> None:
+        """Write the unavailable state to the state machine."""
+        attrs: Dict[str, Any] = {ATTR_RESTORED: True}
+
+        if self.capabilities is not None:
+            attrs.update(self.capabilities)
+
+        if self.supported_features is not None:
+            attrs[ATTR_SUPPORTED_FEATURES] = self.supported_features
+
+        if self.device_class is not None:
+            attrs[ATTR_DEVICE_CLASS] = self.device_class
+
+        if self.unit_of_measurement is not None:
+            attrs[ATTR_UNIT_OF_MEASUREMENT] = self.unit_of_measurement
+
+        name = self.name or self.original_name
+        if name is not None:
+            attrs[ATTR_FRIENDLY_NAME] = name
+
+        icon = self.icon or self.original_icon
+        if icon is not None:
+            attrs[ATTR_ICON] = icon
+
+        opp.states.async_set(self.entity_id, STATE_UNAVAILABLE, attrs)
+
 
 class EntityRegistry:
     """Class to hold a registry of entities."""
@@ -105,11 +152,28 @@ class EntityRegistry:
         """Initialize the registry."""
         self.opp = opp
         self.entities: Dict[str, RegistryEntry]
+        self._index: Dict[Tuple[str, str, str], str] = {}
         self._store = opp.helpers.storage.Store(STORAGE_VERSION, STORAGE_KEY)
         self.opp.bus.async_listen(
-            EVENT_DEVICE_REGISTRY_UPDATED, self.async_device_removed
+            EVENT_DEVICE_REGISTRY_UPDATED, self.async_device_modified
         )
 
+    @callback
+    def async_get_device_class_lookup(self, domain_device_classes: set) -> dict:
+        """Return a lookup for the device class by domain."""
+        lookup: Dict[str, Dict[Tuple[Any, Any], str]] = {}
+        for entity in self.entities.values():
+            if not entity.device_id:
+                continue
+            domain_device_class = (entity.domain, entity.device_class)
+            if domain_device_class not in domain_device_classes:
+                continue
+            if entity.device_id not in lookup:
+                lookup[entity.device_id] = {domain_device_class: entity.entity_id}
+            else:
+                lookup[entity.device_id][domain_device_class] = entity.entity_id
+        return lookup
+
     @callback
     def async_is_registered(self, entity_id: str) -> bool:
         """Check if an entity_id is currently registered."""
@@ -125,14 +189,7 @@ class EntityRegistry:
         self, domain: str, platform: str, unique_id: str
     ) -> Optional[str]:
         """Check if an entity_id is currently registered."""
-        for entity in self.entities.values():
-            if (
-                entity.domain == domain
-                and entity.platform == platform
-                and entity.unique_id == unique_id
-            ):
-                return entity.entity_id
-        return None
+        return self._index.get((domain, platform, unique_id))
 
     @callback
     def async_generate_entity_id(
@@ -145,14 +202,21 @@ class EntityRegistry:
 
         Conflicts checked against registered and currently existing entities.
         """
-        return ensure_unique_string(
-            "{}.{}".format(domain, slugify(suggested_object_id)),
-            chain(
-                self.entities.keys(),
-                self.opp.states.async_entity_ids(domain),
-                known_object_ids if known_object_ids else [],
-            ),
-        )
+        preferred_string = f"{domain}.{slugify(suggested_object_id)}"
+        test_string = preferred_string
+        if not known_object_ids:
+            known_object_ids = {}
+
+        tries = 1
+        while (
+            test_string in self.entities
+            or test_string in known_object_ids
+            or not self.opp.states.async_available(test_string)
+        ):
+            tries += 1
+            test_string = f"{preferred_string}_{tries}"
+
+        return test_string
 
     @callback
     def async_get_or_create(
@@ -169,6 +233,7 @@ class EntityRegistry:
         # Data that we want entry to have
         config_entry: Optional["ConfigEntry"] = None,
         device_id: Optional[str] = None,
+        area_id: Optional[str] = None,
         capabilities: Optional[Dict[str, Any]] = None,
         supported_features: Optional[int] = None,
         device_class: Optional[str] = None,
@@ -184,16 +249,17 @@ class EntityRegistry:
         entity_id = self.async_get_entity_id(domain, platform, unique_id)
 
         if entity_id:
-            return self._async_update_entity(  # type: ignore
+            return self._async_update_entity(
                 entity_id,
-                config_entry_id=config_entry_id or _UNDEF,
-                device_id=device_id or _UNDEF,
-                capabilities=capabilities or _UNDEF,
-                supported_features=supported_features or _UNDEF,
-                device_class=device_class or _UNDEF,
-                unit_of_measurement=unit_of_measurement or _UNDEF,
-                original_name=original_name or _UNDEF,
-                original_icon=original_icon or _UNDEF,
+                config_entry_id=config_entry_id or UNDEFINED,
+                device_id=device_id or UNDEFINED,
+                area_id=area_id or UNDEFINED,
+                capabilities=capabilities or UNDEFINED,
+                supported_features=supported_features or UNDEFINED,
+                device_class=device_class or UNDEFINED,
+                unit_of_measurement=unit_of_measurement or UNDEFINED,
+                original_name=original_name or UNDEFINED,
+                original_icon=original_icon or UNDEFINED,
                 # When we changed our slugify algorithm, we invalidated some
                 # stored entity IDs with either a __ or ending in _.
                 # Fix introduced in 0.86 (Jan 23, 2019). Next line can be
@@ -218,6 +284,7 @@ class EntityRegistry:
             entity_id=entity_id,
             config_entry_id=config_entry_id,
             device_id=device_id,
+            area_id=area_id,
             unique_id=unique_id,
             platform=platform,
             disabled_by=disabled_by,
@@ -228,7 +295,7 @@ class EntityRegistry:
             original_name=original_name,
             original_icon=original_icon,
         )
-        self.entities[entity_id] = entity
+        self._register_entry(entity)
         _LOGGER.info("Registered new %s.%s entity: %s", domain, platform, entity_id)
         self.async_schedule_save()
 
@@ -241,78 +308,111 @@ class EntityRegistry:
     @callback
     def async_remove(self, entity_id: str) -> None:
         """Remove an entity from registry."""
-        self.entities.pop(entity_id)
+        self._unregister_entry(self.entities[entity_id])
         self.opp.bus.async_fire(
             EVENT_ENTITY_REGISTRY_UPDATED, {"action": "remove", "entity_id": entity_id}
         )
         self.async_schedule_save()
 
     @callback
-    def async_device_removed(self, event: Event) -> None:
-        """Handle the removal of a device.
+    def async_device_modified(self, event: Event) -> None:
+        """Handle the removal or update of a device.
 
         Remove entities from the registry that are associated to a device when
         the device is removed.
+
+        Disable entities in the registry that are associated to a device when
+        the device is disabled.
         """
-        if event.data["action"] != "remove":
+        if event.data["action"] == "remove":
+            entities = async_entries_for_device(
+                self, event.data["device_id"], include_disabled_entities=True
+            )
+            for entity in entities:
+                self.async_remove(entity.entity_id)
             return
+
+        if event.data["action"] != "update":
+            return
+
+        device_registry = dr.async_get(self.opp)
+        device = device_registry.async_get(event.data["device_id"])
+
+        # The device may be deleted already if the event handling is late
+        if not device or not device.disabled:
+            entities = async_entries_for_device(
+                self, event.data["device_id"], include_disabled_entities=True
+            )
+            for entity in entities:
+                if entity.disabled_by != DISABLED_DEVICE:
+                    continue
+                self.async_update_entity(entity.entity_id, disabled_by=None)
+            return
+
+        if device.disabled_by == dr.DISABLED_CONFIG_ENTRY:
+            # Handled by async_config_entry_disabled
+            return
+
+        # Fetch entities which are not already disabled
         entities = async_entries_for_device(self, event.data["device_id"])
         for entity in entities:
-            self.async_remove(entity.entity_id)
+            self.async_update_entity(entity.entity_id, disabled_by=DISABLED_DEVICE)
 
     @callback
     def async_update_entity(
         self,
-        entity_id,
+        entity_id: str,
         *,
-        name=_UNDEF,
-        icon=_UNDEF,
-        new_entity_id=_UNDEF,
-        new_unique_id=_UNDEF,
-        disabled_by=_UNDEF,
-    ):
+        name: Union[str, None, UndefinedType] = UNDEFINED,
+        icon: Union[str, None, UndefinedType] = UNDEFINED,
+        area_id: Union[str, None, UndefinedType] = UNDEFINED,
+        new_entity_id: Union[str, UndefinedType] = UNDEFINED,
+        new_unique_id: Union[str, UndefinedType] = UNDEFINED,
+        disabled_by: Union[str, None, UndefinedType] = UNDEFINED,
+    ) -> RegistryEntry:
         """Update properties of an entity."""
-        return cast(  # cast until we have _async_update_entity type hinted
-            RegistryEntry,
-            self._async_update_entity(
-                entity_id,
-                name=name,
-                icon=icon,
-                new_entity_id=new_entity_id,
-                new_unique_id=new_unique_id,
-                disabled_by=disabled_by,
-            ),
+        return self._async_update_entity(
+            entity_id,
+            name=name,
+            icon=icon,
+            area_id=area_id,
+            new_entity_id=new_entity_id,
+            new_unique_id=new_unique_id,
+            disabled_by=disabled_by,
         )
 
     @callback
     def _async_update_entity(
         self,
-        entity_id,
+        entity_id: str,
         *,
-        name=_UNDEF,
-        icon=_UNDEF,
-        config_entry_id=_UNDEF,
-        new_entity_id=_UNDEF,
-        device_id=_UNDEF,
-        new_unique_id=_UNDEF,
-        disabled_by=_UNDEF,
-        capabilities=_UNDEF,
-        supported_features=_UNDEF,
-        device_class=_UNDEF,
-        unit_of_measurement=_UNDEF,
-        original_name=_UNDEF,
-        original_icon=_UNDEF,
-    ):
+        name: Union[str, None, UndefinedType] = UNDEFINED,
+        icon: Union[str, None, UndefinedType] = UNDEFINED,
+        config_entry_id: Union[str, None, UndefinedType] = UNDEFINED,
+        new_entity_id: Union[str, UndefinedType] = UNDEFINED,
+        device_id: Union[str, None, UndefinedType] = UNDEFINED,
+        area_id: Union[str, None, UndefinedType] = UNDEFINED,
+        new_unique_id: Union[str, UndefinedType] = UNDEFINED,
+        disabled_by: Union[str, None, UndefinedType] = UNDEFINED,
+        capabilities: Union[Dict[str, Any], None, UndefinedType] = UNDEFINED,
+        supported_features: Union[int, UndefinedType] = UNDEFINED,
+        device_class: Union[str, None, UndefinedType] = UNDEFINED,
+        unit_of_measurement: Union[str, None, UndefinedType] = UNDEFINED,
+        original_name: Union[str, None, UndefinedType] = UNDEFINED,
+        original_icon: Union[str, None, UndefinedType] = UNDEFINED,
+    ) -> RegistryEntry:
         """Private facing update properties method."""
         old = self.entities[entity_id]
 
-        changes = {}
+        new_values = {}  # Dict with new key/value pairs
+        old_values = {}  # Dict with old key/value pairs
 
         for attr_name, value in (
             ("name", name),
             ("icon", icon),
             ("config_entry_id", config_entry_id),
             ("device_id", device_id),
+            ("area_id", area_id),
             ("disabled_by", disabled_by),
             ("capabilities", capabilities),
             ("supported_features", supported_features),
@@ -321,12 +421,13 @@ class EntityRegistry:
             ("original_name", original_name),
             ("original_icon", original_icon),
         ):
-            if value is not _UNDEF and value != getattr(old, attr_name):
-                changes[attr_name] = value
+            if value is not UNDEFINED and value != getattr(old, attr_name):
+                new_values[attr_name] = value
+                old_values[attr_name] = getattr(old, attr_name)
 
-        if new_entity_id is not _UNDEF and new_entity_id != old.entity_id:
+        if new_entity_id is not UNDEFINED and new_entity_id != old.entity_id:
             if self.async_is_registered(new_entity_id):
-                raise ValueError("Entity is already registered")
+                raise ValueError("Entity with this ID is already registered")
 
             if not valid_entity_id(new_entity_id):
                 raise ValueError("Invalid entity ID")
@@ -335,34 +436,31 @@ class EntityRegistry:
                 raise ValueError("New entity ID should be same domain")
 
             self.entities.pop(entity_id)
-            entity_id = changes["entity_id"] = new_entity_id
+            entity_id = new_values["entity_id"] = new_entity_id
+            old_values["entity_id"] = old.entity_id
 
-        if new_unique_id is not _UNDEF:
-            conflict = next(
-                (
-                    entity
-                    for entity in self.entities.values()
-                    if entity.unique_id == new_unique_id
-                    and entity.domain == old.domain
-                    and entity.platform == old.platform
-                ),
-                None,
+        if new_unique_id is not UNDEFINED:
+            conflict_entity_id = self.async_get_entity_id(
+                old.domain, old.platform, new_unique_id
             )
-            if conflict:
+            if conflict_entity_id:
                 raise ValueError(
                     f"Unique id '{new_unique_id}' is already in use by "
-                    f"'{conflict.entity_id}'"
+                    f"'{conflict_entity_id}'"
                 )
-            changes["unique_id"] = new_unique_id
+            new_values["unique_id"] = new_unique_id
+            old_values["unique_id"] = old.unique_id
 
-        if not changes:
+        if not new_values:
             return old
 
-        new = self.entities[entity_id] = attr.evolve(old, **changes)
+        self._remove_index(old)
+        new = attr.evolve(old, **new_values)
+        self._register_entry(new)
 
         self.async_schedule_save()
 
-        data = {"action": "update", "entity_id": entity_id, "changes": list(changes)}
+        data = {"action": "update", "entity_id": entity_id, "changes": old_values}
 
         if old.entity_id != entity_id:
             data["old_entity_id"] = old.entity_id
@@ -385,10 +483,17 @@ class EntityRegistry:
 
         if data is not None:
             for entity in data["entities"]:
+                # Some old installations can have some bad entities.
+                # Filter them out as they cause errors down the line.
+                # Can be removed in Jan 2021
+                if not valid_entity_id(entity["entity_id"]):
+                    continue
+
                 entities[entity["entity_id"]] = RegistryEntry(
                     entity_id=entity["entity_id"],
                     config_entry_id=entity.get("config_entry_id"),
                     device_id=entity.get("device_id"),
+                    area_id=entity.get("area_id"),
                     unique_id=entity["unique_id"],
                     platform=entity["platform"],
                     name=entity.get("name"),
@@ -403,6 +508,7 @@ class EntityRegistry:
                 )
 
         self.entities = entities
+        self._rebuild_index()
 
     @callback
     def async_schedule_save(self) -> None:
@@ -419,6 +525,7 @@ class EntityRegistry:
                 "entity_id": entry.entity_id,
                 "config_entry_id": entry.config_entry_id,
                 "device_id": entry.device_id,
+                "area_id": entry.area_id,
                 "unique_id": entry.unique_id,
                 "platform": entry.platform,
                 "name": entry.name,
@@ -446,40 +553,76 @@ class EntityRegistry:
         ]:
             self.async_remove(entity_id)
 
+    @callback
+    def async_clear_area_id(self, area_id: str) -> None:
+        """Clear area id from registry entries."""
+        for entity_id, entry in self.entities.items():
+            if area_id == entry.area_id:
+                self._async_update_entity(entity_id, area_id=None)
+
+    def _register_entry(self, entry: RegistryEntry) -> None:
+        self.entities[entry.entity_id] = entry
+        self._add_index(entry)
+
+    def _add_index(self, entry: RegistryEntry) -> None:
+        self._index[(entry.domain, entry.platform, entry.unique_id)] = entry.entity_id
+
+    def _unregister_entry(self, entry: RegistryEntry) -> None:
+        self._remove_index(entry)
+        del self.entities[entry.entity_id]
+
+    def _remove_index(self, entry: RegistryEntry) -> None:
+        del self._index[(entry.domain, entry.platform, entry.unique_id)]
+
+    def _rebuild_index(self) -> None:
+        self._index = {}
+        for entry in self.entities.values():
+            self._add_index(entry)
+
+
+@callback
+def async_get(opp: OpenPeerPowerType) -> EntityRegistry:
+    """Get entity registry."""
+    return cast(EntityRegistry, opp.data[DATA_REGISTRY])
+
+
+async def async_load(opp: OpenPeerPowerType) -> None:
+    """Load entity registry."""
+    assert DATA_REGISTRY not in opp.data
+    opp.data[DATA_REGISTRY] = EntityRegistry(opp)
+    await opp.data[DATA_REGISTRY].async_load()
+
 
 @bind_opp
 async def async_get_registry(opp: OpenPeerPowerType) -> EntityRegistry:
-    """Return entity registry instance."""
-    reg_or_evt = opp.data.get(DATA_REGISTRY)
+    """Get entity registry.
 
-    if not reg_or_evt:
-        evt = opp.data[DATA_REGISTRY] = asyncio.Event()
-
-        reg = EntityRegistry(opp)
-        await reg.async_load()
-
-        opp.data[DATA_REGISTRY] = reg
-        evt.set()
-        return reg
-
-    if isinstance(reg_or_evt, asyncio.Event):
-        evt = reg_or_evt
-        await evt.wait()
-        return cast(EntityRegistry, opp.data.get(DATA_REGISTRY))
-
-    return cast(EntityRegistry, reg_or_evt)
+    This is deprecated and will be removed in the future. Use async_get instead.
+    """
+    return async_get(opp)
 
 
 @callback
 def async_entries_for_device(
-    registry: EntityRegistry, device_id: str
+    registry: EntityRegistry, device_id: str, include_disabled_entities: bool = False
 ) -> List[RegistryEntry]:
     """Return entries that match a device."""
     return [
-        entry for entry in registry.entities.values() if entry.device_id == device_id
+        entry
+        for entry in registry.entities.values()
+        if entry.device_id == device_id
+        and (not entry.disabled_by or include_disabled_entities)
     ]
 
 
+@callback
+def async_entries_for_area(
+    registry: EntityRegistry, area_id: str
+) -> List[RegistryEntry]:
+    """Return entries that match an area."""
+    return [entry for entry in registry.entities.values() if entry.area_id == area_id]
+
+
 @callback
 def async_entries_for_config_entry(
     registry: EntityRegistry, config_entry_id: str
@@ -492,6 +635,36 @@ def async_entries_for_config_entry(
     ]
 
 
+@callback
+def async_config_entry_disabled_by_changed(
+    registry: EntityRegistry, config_entry: "ConfigEntry"
+) -> None:
+    """Handle a config entry being disabled or enabled.
+
+    Disable entities in the registry that are associated with a config entry when
+    the config entry is disabled, enable entities in the registry that are associated
+    with a config entry when the config entry is enabled and the entities are marked
+    DISABLED_CONFIG_ENTRY.
+    """
+
+    entities = async_entries_for_config_entry(registry, config_entry.entry_id)
+
+    if not config_entry.disabled_by:
+        for entity in entities:
+            if entity.disabled_by != DISABLED_CONFIG_ENTRY:
+                continue
+            registry.async_update_entity(entity.entity_id, disabled_by=None)
+        return
+
+    for entity in entities:
+        if entity.disabled:
+            # Entity already disabled, do not overwrite
+            continue
+        registry.async_update_entity(
+            entity.entity_id, disabled_by=DISABLED_CONFIG_ENTRY
+        )
+
+
 async def _async_migrate(entities: Dict[str, Any]) -> Dict[str, List[Dict[str, Any]]]:
     """Migrate the YAML config file to storage helper format."""
     return {
@@ -507,20 +680,26 @@ def async_setup_entity_restore(
 ) -> None:
     """Set up the entity restore mechanism."""
 
+    @callback
+    def cleanup_restored_states_filter(event: Event) -> bool:
+        """Clean up restored states filter."""
+        return bool(event.data["action"] == "remove")
+
     @callback
     def cleanup_restored_states(event: Event) -> None:
         """Clean up restored states."""
-        if event.data["action"] != "remove":
-            return
-
         state = opp.states.get(event.data["entity_id"])
 
         if state is None or not state.attributes.get(ATTR_RESTORED):
             return
 
-        opp.states.async_remove(event.data["entity_id"])
+        opp.states.async_remove(event.data["entity_id"], context=event.context)
 
-    opp.bus.async_listen(EVENT_ENTITY_REGISTRY_UPDATED, cleanup_restored_states)
+    opp.bus.async_listen(
+        EVENT_ENTITY_REGISTRY_UPDATED,
+        cleanup_restored_states,
+        event_filter=cleanup_restored_states_filter,
+    )
 
     if opp.is_running:
         return
@@ -528,35 +707,30 @@ def async_setup_entity_restore(
     @callback
     def _write_unavailable_states(_: Event) -> None:
         """Make sure state machine contains entry for each registered entity."""
-        states = opp.states
-        existing = set(states.async_entity_ids())
+        existing = set(opp.states.async_entity_ids())
 
         for entry in registry.entities.values():
             if entry.entity_id in existing or entry.disabled:
                 continue
 
-            attrs: Dict[str, Any] = {ATTR_RESTORED: True}
-
-            if entry.capabilities is not None:
-                attrs.update(entry.capabilities)
-
-            if entry.supported_features is not None:
-                attrs[ATTR_SUPPORTED_FEATURES] = entry.supported_features
-
-            if entry.device_class is not None:
-                attrs[ATTR_DEVICE_CLASS] = entry.device_class
-
-            if entry.unit_of_measurement is not None:
-                attrs[ATTR_UNIT_OF_MEASUREMENT] = entry.unit_of_measurement
-
-            name = entry.name or entry.original_name
-            if name is not None:
-                attrs[ATTR_FRIENDLY_NAME] = name
-
-            icon = entry.icon or entry.original_icon
-            if icon is not None:
-                attrs[ATTR_ICON] = icon
-
-            states.async_set(entry.entity_id, STATE_UNAVAILABLE, attrs)
+            entry.write_unavailable_state(opp)
 
     opp.bus.async_listen(EVENT_OPENPEERPOWER_START, _write_unavailable_states)
+
+
+async def async_migrate_entries(
+    opp: OpenPeerPowerType,
+    config_entry_id: str,
+    entry_callback: Callable[[RegistryEntry], Optional[dict]],
+) -> None:
+    """Migrator of unique IDs."""
+    ent_reg = await async_get_registry(opp)
+
+    for entry in ent_reg.entities.values():
+        if entry.config_entry_id != config_entry_id:
+            continue
+
+        updates = entry_callback(entry)
+
+        if updates is not None:
+            ent_reg.async_update_entity(entry.entity_id, **updates)
