@@ -4,6 +4,8 @@ The methods for loading Open Peer Power integrations.
 This module has quite some complex parts. I have tried to add as much
 documentation as possible to keep it understandable.
 """
+from __future__ import annotations
+
 import asyncio
 import functools as ft
 import importlib
@@ -20,19 +22,28 @@ from typing import (
     List,
     Optional,
     Set,
+    TypedDict,
     TypeVar,
     Union,
     cast,
 )
 
+from awesomeversion import AwesomeVersion, AwesomeVersionStrategy
+
+from openpeerpower.generated.dhcp import DHCP
+from openpeerpower.generated.mqtt import MQTT
+from openpeerpower.generated.ssdp import SSDP
+from openpeerpower.generated.zeroconf import HOMEKIT, ZEROCONF
+
 # Typing imports that create a circular dependency
-# pylint: disable=unused-import
 if TYPE_CHECKING:
     from openpeerpower.core import OpenPeerPower
 
-CALLABLE_T = TypeVar("CALLABLE_T", bound=Callable)  # pylint: disable=invalid-name
+# mypy: disallow-any-generics
 
-DEPENDENCY_BLACKLIST = {"config"}
+CALLABLE_T = TypeVar(  # pylint: disable=invalid-name
+    "CALLABLE_T", bound=Callable[..., Any]
+)
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -42,20 +53,61 @@ DATA_CUSTOM_COMPONENTS = "custom_components"
 PACKAGE_CUSTOM_COMPONENTS = "custom_components"
 PACKAGE_BUILTIN = "openpeerpower.components"
 CUSTOM_WARNING = (
-    "You are using a custom integration for %s which has not "
+    "You are using a custom integration %s which has not "
     "been tested by Open Peer Power. This component might "
     "cause stability problems, be sure to disable it if you "
-    "do experience issues with Open Peer Power."
+    "experience issues with Open Peer Power"
 )
-_UNDEF = object()
+CUSTOM_WARNING_VERSION_MISSING = (
+    "No 'version' key in the manifest file for "
+    "custom integration '%s'. This will not be "
+    "allowed in a future version of Open "
+    "Peer Power. Please report this to the "
+    "maintainer of '%s'"
+)
+CUSTOM_WARNING_VERSION_TYPE = (
+    "'%s' is not a valid version for "
+    "custom integration '%s'. "
+    "Please report this to the maintainer of '%s'"
+)
+_UNDEF = object()  # Internal; not helpers.typing.UNDEFINED due to circular dependency
+
+MAX_LOAD_CONCURRENTLY = 4
 
 
-def manifest_from_legacy_module(domain: str, module: ModuleType) -> Dict:
+class Manifest(TypedDict, total=False):
+    """
+    Integration manifest.
+
+    Note that none of the attributes are marked Optional here. However, some of them may be optional in manifest.json
+    in the sense that they can be omitted altogether. But when present, they should not have null values in it.
+    """
+
+    name: str
+    disabled: str
+    domain: str
+    dependencies: List[str]
+    after_dependencies: List[str]
+    requirements: List[str]
+    config_flow: bool
+    documentation: str
+    issue_tracker: str
+    quality_scale: str
+    mqtt: List[str]
+    ssdp: List[Dict[str, str]]
+    zeroconf: List[Union[str, Dict[str, str]]]
+    dhcp: List[Dict[str, str]]
+    homekit: Dict[str, List[str]]
+    is_built_in: bool
+    version: str
+    codeowners: List[str]
+
+
+def manifest_from_legacy_module(domain: str, module: ModuleType) -> Manifest:
     """Generate a manifest from a legacy module."""
     return {
         "domain": domain,
         "name": domain,
-        "documentation": None,
         "requirements": getattr(module, "REQUIREMENTS", []),
         "dependencies": getattr(module, "DEPENDENCIES", []),
         "codeowners": [],
@@ -64,17 +116,17 @@ def manifest_from_legacy_module(domain: str, module: ModuleType) -> Dict:
 
 async def _async_get_custom_components(
     opp: "OpenPeerPower",
-) -> Dict[str, "Integration"]:
+) -> Dict[str, Integration]:
     """Return list of custom integrations."""
     if opp.config.safe_mode:
         return {}
 
     try:
-        import custom_components
+        import custom_components  # pylint: disable=import-outside-toplevel
     except ImportError:
         return {}
 
-    def get_sub_directories(paths: List) -> List:
+    def get_sub_directories(paths: List[str]) -> List[pathlib.Path]:
         """Return all sub directories in a set of paths."""
         return [
             entry
@@ -105,7 +157,7 @@ async def _async_get_custom_components(
 
 async def async_get_custom_components(
     opp: "OpenPeerPower",
-) -> Dict[str, "Integration"]:
+) -> Dict[str, Integration]:
     """Return cached list of custom integrations."""
     reg_or_evt = opp.data.get(DATA_CUSTOM_COMPONENTS)
 
@@ -125,8 +177,9 @@ async def async_get_custom_components(
     return cast(Dict[str, "Integration"], reg_or_evt)
 
 
-async def async_get_config_flows(opp: "OpenPeerPower") -> Set[str]:
+async def async_get_config_flows(opp: OpenPeerPower) -> Set[str]:
     """Return cached list of config flows."""
+    # pylint: disable=import-outside-toplevel
     from openpeerpower.generated.config_flows import FLOWS
 
     flows: Set[str] = set()
@@ -144,13 +197,99 @@ async def async_get_config_flows(opp: "OpenPeerPower") -> Set[str]:
     return flows
 
 
+async def async_get_zeroconf(opp: OpenPeerPower) -> Dict[str, List[Dict[str, str]]]:
+    """Return cached list of zeroconf types."""
+    zeroconf: Dict[str, List[Dict[str, str]]] = ZEROCONF.copy()
+
+    integrations = await async_get_custom_components(opp)
+    for integration in integrations.values():
+        if not integration.zeroconf:
+            continue
+        for entry in integration.zeroconf:
+            data = {"domain": integration.domain}
+            if isinstance(entry, dict):
+                typ = entry["type"]
+                entry_without_type = entry.copy()
+                del entry_without_type["type"]
+                data.update(entry_without_type)
+            else:
+                typ = entry
+
+            zeroconf.setdefault(typ, []).append(data)
+
+    return zeroconf
+
+
+async def async_get_dhcp(opp: OpenPeerPower) -> List[Dict[str, str]]:
+    """Return cached list of dhcp types."""
+    dhcp: List[Dict[str, str]] = DHCP.copy()
+
+    integrations = await async_get_custom_components(opp)
+    for integration in integrations.values():
+        if not integration.dhcp:
+            continue
+        for entry in integration.dhcp:
+            dhcp.append({"domain": integration.domain, **entry})
+
+    return dhcp
+
+
+async def async_get_homekit(opp: OpenPeerPower) -> Dict[str, str]:
+    """Return cached list of homekit models."""
+
+    homekit: Dict[str, str] = HOMEKIT.copy()
+
+    integrations = await async_get_custom_components(opp)
+    for integration in integrations.values():
+        if (
+            not integration.homekit
+            or "models" not in integration.homekit
+            or not integration.homekit["models"]
+        ):
+            continue
+        for model in integration.homekit["models"]:
+            homekit[model] = integration.domain
+
+    return homekit
+
+
+async def async_get_ssdp(opp: OpenPeerPower) -> Dict[str, List[Dict[str, str]]]:
+    """Return cached list of ssdp mappings."""
+
+    ssdp: Dict[str, List[Dict[str, str]]] = SSDP.copy()
+
+    integrations = await async_get_custom_components(opp)
+    for integration in integrations.values():
+        if not integration.ssdp:
+            continue
+
+        ssdp[integration.domain] = integration.ssdp
+
+    return ssdp
+
+
+async def async_get_mqtt(opp: OpenPeerPower) -> Dict[str, List[str]]:
+    """Return cached list of MQTT mappings."""
+
+    mqtt: Dict[str, List[str]] = MQTT.copy()
+
+    integrations = await async_get_custom_components(opp)
+    for integration in integrations.values():
+        if not integration.mqtt:
+            continue
+
+        mqtt[integration.domain] = integration.mqtt
+
+    return mqtt
+
+
 class Integration:
     """An integration in Open Peer Power."""
 
     @classmethod
     def resolve_from_root(
         cls, opp: "OpenPeerPower", root_module: ModuleType, domain: str
-    ) -> "Optional[Integration]":
+    ) -> Optional[Integration]:
         """Resolve an integration from a root module."""
         for base in root_module.__path__:  # type: ignore
             manifest_path = pathlib.Path(base) / domain / "manifest.json"
@@ -173,9 +312,7 @@ class Integration:
         return None
 
     @classmethod
-    def resolve_legacy(
-        cls, opp: "OpenPeerPower", domain: str
-    ) -> "Optional[Integration]":
+    def resolve_legacy(cls, opp: "OpenPeerPower", domain: str) -> Optional[Integration]:
         """Resolve legacy component.
 
         Will create a stub manifest.
@@ -197,70 +334,154 @@ class Integration:
         opp: "OpenPeerPower",
         pkg_path: str,
         file_path: pathlib.Path,
-        manifest: Dict[str, Any],
+        manifest: Manifest,
     ):
         """Initialize an integration."""
         self.opp = opp
         self.pkg_path = pkg_path
         self.file_path = file_path
         self.manifest = manifest
+        manifest["is_built_in"] = self.is_built_in
+
+        if self.dependencies:
+            self._all_dependencies_resolved: Optional[bool] = None
+            self._all_dependencies: Optional[Set[str]] = None
+        else:
+            self._all_dependencies_resolved = True
+            self._all_dependencies = set()
+
         _LOGGER.info("Loaded %s from %s", self.domain, pkg_path)
 
     @property
     def name(self) -> str:
         """Return name."""
-        return cast(str, self.manifest["name"])
+        return self.manifest["name"]
+
+    @property
+    def disabled(self) -> Optional[str]:
+        """Return reason integration is disabled."""
+        return self.manifest.get("disabled")
 
     @property
     def domain(self) -> str:
         """Return domain."""
-        return cast(str, self.manifest["domain"])
+        return self.manifest["domain"]
 
     @property
     def dependencies(self) -> List[str]:
         """Return dependencies."""
-        return cast(List[str], self.manifest.get("dependencies", []))
+        return self.manifest.get("dependencies", [])
 
     @property
     def after_dependencies(self) -> List[str]:
         """Return after_dependencies."""
-        return cast(List[str], self.manifest.get("after_dependencies", []))
+        return self.manifest.get("after_dependencies", [])
 
     @property
     def requirements(self) -> List[str]:
         """Return requirements."""
-        return cast(List[str], self.manifest.get("requirements", []))
+        return self.manifest.get("requirements", [])
 
     @property
     def config_flow(self) -> bool:
         """Return config_flow."""
-        return cast(bool, self.manifest.get("config_flow", False))
+        return self.manifest.get("config_flow") or False
 
     @property
     def documentation(self) -> Optional[str]:
         """Return documentation."""
-        return cast(str, self.manifest.get("documentation"))
+        return self.manifest.get("documentation")
+
+    @property
+    def issue_tracker(self) -> Optional[str]:
+        """Return issue tracker link."""
+        return self.manifest.get("issue_tracker")
 
     @property
     def quality_scale(self) -> Optional[str]:
         """Return Integration Quality Scale."""
-        return cast(str, self.manifest.get("quality_scale"))
+        return self.manifest.get("quality_scale")
 
     @property
-    def logo(self) -> Optional[str]:
-        """Return Integration Logo."""
-        return cast(str, self.manifest.get("logo"))
+    def mqtt(self) -> Optional[List[str]]:
+        """Return Integration MQTT entries."""
+        return self.manifest.get("mqtt")
 
     @property
-    def icon(self) -> Optional[str]:
-        """Return Integration Icon."""
-        return cast(str, self.manifest.get("icon"))
+    def ssdp(self) -> Optional[List[Dict[str, str]]]:
+        """Return Integration SSDP entries."""
+        return self.manifest.get("ssdp")
+
+    @property
+    def zeroconf(self) -> Optional[List[Union[str, Dict[str, str]]]]:
+        """Return Integration zeroconf entries."""
+        return self.manifest.get("zeroconf")
+
+    @property
+    def dhcp(self) -> Optional[List[Dict[str, str]]]:
+        """Return Integration dhcp entries."""
+        return self.manifest.get("dhcp")
+
+    @property
+    def homekit(self) -> Optional[Dict[str, List[str]]]:
+        """Return Integration homekit entries."""
+        return self.manifest.get("homekit")
 
     @property
     def is_built_in(self) -> bool:
         """Test if package is a built-in integration."""
         return self.pkg_path.startswith(PACKAGE_BUILTIN)
 
+    @property
+    def version(self) -> Optional[AwesomeVersion]:
+        """Return the version of the integration."""
+        if "version" not in self.manifest:
+            return None
+        return AwesomeVersion(self.manifest["version"])
+
+    @property
+    def all_dependencies(self) -> Set[str]:
+        """Return all dependencies including sub-dependencies."""
+        if self._all_dependencies is None:
+            raise RuntimeError("Dependencies not resolved!")
+
+        return self._all_dependencies
+
+    @property
+    def all_dependencies_resolved(self) -> bool:
+        """Return if all dependencies have been resolved."""
+        return self._all_dependencies_resolved is not None
+
+    async def resolve_dependencies(self) -> bool:
+        """Resolve all dependencies."""
+        if self._all_dependencies_resolved is not None:
+            return self._all_dependencies_resolved
+
+        try:
+            dependencies = await _async_component_dependencies(
+                self.opp, self.domain, self, set(), set()
+            )
+            dependencies.discard(self.domain)
+            self._all_dependencies = dependencies
+            self._all_dependencies_resolved = True
+        except IntegrationNotFound as err:
+            _LOGGER.error(
+                "Unable to resolve dependencies for %s:  we are unable to resolve (sub)dependency %s",
+                self.domain,
+                err.domain,
+            )
+            self._all_dependencies_resolved = False
+        except CircularDependency as err:
+            _LOGGER.error(
+                "Unable to resolve dependencies for %s:  it contains a circular dependency: %s -> %s",
+                self.domain,
+                err.from_domain,
+                err.to_domain,
+            )
+            self._all_dependencies_resolved = False
+
+        return self._all_dependencies_resolved
+
     def get_component(self) -> ModuleType:
         """Return the component."""
         cache = self.opp.data.setdefault(DATA_COMPONENTS, {})
@@ -273,11 +494,13 @@ class Integration:
         cache = self.opp.data.setdefault(DATA_COMPONENTS, {})
         full_name = f"{self.domain}.{platform_name}"
         if full_name not in cache:
-            cache[full_name] = importlib.import_module(
-                f"{self.pkg_path}.{platform_name}"
-            )
+            cache[full_name] = self._import_platform(platform_name)
         return cache[full_name]  # type: ignore
 
+    def _import_platform(self, platform_name: str) -> ModuleType:
+        """Import the platform."""
+        return importlib.import_module(f"{self.pkg_path}.{platform_name}")
+
     def __repr__(self) -> str:
         """Text representation of class."""
         return f"<Integration {self.domain}: {self.pkg_path}>"
@@ -312,12 +535,12 @@ async def async_get_integration(opp: "OpenPeerPower", domain: str) -> Integratio
     # components to find the integration.
     integration = (await async_get_custom_components(opp)).get(domain)
     if integration is not None:
-        _LOGGER.warning(CUSTOM_WARNING, domain)
+        custom_integration_warning(integration)
         cache[domain] = integration
         event.set()
         return integration
 
-    from openpeerpower import components
+    from openpeerpower import components  # pylint: disable=import-outside-toplevel
 
     integration = await opp.async_add_executor_job(
         Integration.resolve_from_root, opp, components, domain
@@ -330,6 +553,7 @@ async def async_get_integration(opp: "OpenPeerPower", domain: str) -> Integratio
 
     integration = Integration.resolve_legacy(opp, domain)
     if integration is not None:
+        custom_integration_warning(integration)
         cache[domain] = integration
     else:
         # Remove event from cache.
@@ -404,9 +628,6 @@ def _load_file(
 
             cache[comp_or_platform] = module
 
-            if module.__name__.startswith(PACKAGE_CUSTOM_COMPONENTS):
-                _LOGGER.warning(CUSTOM_WARNING, comp_or_platform)
-
             return module
 
         except ImportError as err:
@@ -418,14 +639,11 @@ def _load_file(
             parts = []
             for part in path.split("."):
                 parts.append(part)
-                white_listed_errors.append(
-                    "No module named '{}'".format(".".join(parts))
-                )
+                white_listed_errors.append(f"No module named '{'.'.join(parts)}'")
 
             if str(err) not in white_listed_errors:
                 _LOGGER.exception(
-                    ("Error loading %s. Make sure all dependencies are installed"),
-                    path,
+                    ("Error loading %s. Make sure all dependencies are installed"), path
                 )
 
     return None
@@ -453,7 +671,7 @@ class ModuleWrapper:
 class Components:
     """Helper to load components."""
 
-    def __init__(self, opp: "OpenPeerPower") -> None:
+    def __init__(self, opp: OpenPeerPower) -> None:
         """Initialize the Components class."""
         self._opp = opp
 
@@ -479,7 +697,7 @@ class Components:
 class Helpers:
     """Helper to load helpers."""
 
-    def __init__(self, opp: "OpenPeerPower") -> None:
+    def __init__(self, opp: OpenPeerPower) -> None:
         """Initialize the Helpers class."""
         self._opp = opp
 
@@ -497,23 +715,18 @@ def bind_opp(func: CALLABLE_T) -> CALLABLE_T:
     return func
 
 
-async def async_component_dependencies(opp: "OpenPeerPower", domain: str) -> Set[str]:
-    """Return all dependencies and subdependencies of components.
-
-    Raises CircularDependency if a circular dependency is found.
-    """
-    return await _async_component_dependencies(opp, domain, set(), set())
-
-
 async def _async_component_dependencies(
-    opp: "OpenPeerPower", domain: str, loaded: Set[str], loading: Set
+    opp: "OpenPeerPower",
+    start_domain: str,
+    integration: Integration,
+    loaded: Set[str],
+    loading: Set[str],
 ) -> Set[str]:
     """Recursive function to get component dependencies.
 
     Async friendly.
     """
-    integration = await async_get_integration(opp, domain)
-
+    domain = integration.domain
     loading.add(domain)
 
     for dependency_domain in integration.dependencies:
@@ -525,11 +738,19 @@ async def _async_component_dependencies(
         if dependency_domain in loading:
             raise CircularDependency(domain, dependency_domain)
 
-        dep_loaded = await _async_component_dependencies(
-            opp, dependency_domain, loaded, loading
-        )
+        loaded.add(dependency_domain)
 
-        loaded.update(dep_loaded)
+        dep_integration = await async_get_integration(opp, dependency_domain)
+
+        if start_domain in dep_integration.after_dependencies:
+            raise CircularDependency(start_domain, dependency_domain)
+
+        if dep_integration.dependencies:
+            dep_loaded = await _async_component_dependencies(
+                opp, start_domain, dep_integration, loaded, loading
+            )
+
+            loaded.update(dep_loaded)
 
     loaded.add(domain)
     loading.remove(domain)
@@ -537,7 +758,7 @@ async def _async_component_dependencies(
     return loaded
 
 
-def _async_mount_config_dir(opp: "OpenPeerPower") -> bool:
+def _async_mount_config_dir(opp: OpenPeerPower) -> bool:
     """Mount config dir in order to load custom_component.
 
     Async friendly but not a coroutine.
@@ -550,8 +771,40 @@ def _async_mount_config_dir(opp: "OpenPeerPower") -> bool:
     return True
 
 
-def _lookup_path(opp: "OpenPeerPower") -> List[str]:
+def _lookup_path(opp: OpenPeerPower) -> List[str]:
     """Return the lookup paths for legacy lookups."""
     if opp.config.safe_mode:
         return [PACKAGE_BUILTIN]
     return [PACKAGE_CUSTOM_COMPONENTS, PACKAGE_BUILTIN]
+
+
+def validate_custom_integration_version(version: str) -> bool:
+    """Validate the version of custom integrations."""
+    return AwesomeVersion(version).strategy in (
+        AwesomeVersionStrategy.CALVER,
+        AwesomeVersionStrategy.SEMVER,
+        AwesomeVersionStrategy.SIMPLEVER,
+        AwesomeVersionStrategy.BUILDVER,
+        AwesomeVersionStrategy.PEP440,
+    )
+
+
+def custom_integration_warning(integration: Integration) -> None:
+    """Create logs for custom integrations."""
+    if not integration.pkg_path.startswith(PACKAGE_CUSTOM_COMPONENTS):
+        return None
+
+    _LOGGER.warning(CUSTOM_WARNING, integration.domain)
+
+    if integration.manifest.get("version") is None:
+        _LOGGER.warning(
+            CUSTOM_WARNING_VERSION_MISSING, integration.domain, integration.domain
+        )
+    else:
+        if not validate_custom_integration_version(integration.manifest["version"]):
+            _LOGGER.warning(
+                CUSTOM_WARNING_VERSION_TYPE,
+                integration.manifest["version"],
+                integration.domain,
+                integration.domain,
+            )
