@@ -1,5 +1,6 @@
 """Script to ensure a configuration file exists."""
 import argparse
+import asyncio
 import os
 
 import openpeerpower.config as config_util
@@ -11,7 +12,9 @@ from openpeerpower.core import OpenPeerPower
 def run(args):
     """Handle ensure config commandline script."""
     parser = argparse.ArgumentParser(
-        description=("Ensure a Open Peer Power config exists, creates one if necessary.")
+        description=(
+            "Ensure a Open Peer Power config exists, creates one if necessary."
+        )
     )
     parser.add_argument(
         "-c",
@@ -31,15 +34,15 @@ def run(args):
         print("Creating directory", config_dir)
         os.makedirs(config_dir)
 
-    opp = OpenPeerPower()
-    opp.config.config_dir = config_dir
-    config_path = opp.loop.run_until_complete(async_run(opp))
+    config_path = asyncio.run(async_run(config_dir))
     print("Configuration file:", config_path)
     return 0
 
 
-async def async_run(opp):
+async def async_run(config_dir):
     """Make sure config exists."""
+    opp = OpenPeerPower()
+    opp.config.config_dir = config_dir
     path = await config_util.async_ensure_config_exists(opp)
     await opp.async_stop(force=True)
     return path
