@@ -1,6 +1,4 @@
 """Pressure util functions."""
-
-import logging
 from numbers import Number
 
 from openpeerpower.const import (
@@ -13,8 +11,6 @@ from openpeerpower.const import (
     UNIT_NOT_RECOGNIZED_TEMPLATE,
 )
 
-_LOGGER = logging.getLogger(__name__)
-
 VALID_UNITS = [PRESSURE_PA, PRESSURE_HPA, PRESSURE_MBAR, PRESSURE_INHG, PRESSURE_PSI]
 
 UNIT_CONVERSION = {
@@ -36,8 +32,7 @@ def convert(value: float, unit_1: str, unit_2: str) -> float:
     if not isinstance(value, Number):
         raise TypeError(f"{value} is not of numeric type")
 
-    # type ignore: https://github.com/python/mypy/issues/7207
-    if unit_1 == unit_2 or unit_1 not in VALID_UNITS:  # type: ignore
+    if unit_1 == unit_2 or unit_1 not in VALID_UNITS:
         return value
 
     pascals = value / UNIT_CONVERSION[unit_1]
