@@ -32,13 +32,14 @@ def printc(the_color, *args):
         return
     try:
         print(escape_codes[the_color] + msg + escape_codes["reset"])
-    except KeyError:
+    except KeyError as err:
         print(msg)
-        raise ValueError(f"Invalid color {the_color}")
+        raise ValueError(f"Invalid color {the_color}") from err
 
 
 def validate_requirements_ok():
     """Validate requirements, returns True of ok."""
+    # pylint: disable=import-error,import-outside-toplevel
     from gen_requirements_all import main as req_main
 
     return req_main(True) == 0
@@ -67,7 +68,6 @@ async def async_exec(*args, display=False):
     printc("cyan", *argsp)
     try:
         kwargs = {
-            "loop": LOOP,
             "stdout": asyncio.subprocess.PIPE,
             "stderr": asyncio.subprocess.STDOUT,
         }
@@ -232,15 +232,7 @@ async def main():
 
 
 if __name__ == "__main__":
-    LOOP = (
-        asyncio.ProactorEventLoop()
-        if sys.platform == "win32"
-        else asyncio.get_event_loop()
-    )
-
     try:
-        LOOP.run_until_complete(main())
+        asyncio.run(main())
     except (FileNotFoundError, KeyboardInterrupt):
         pass
-    finally:
-        LOOP.close()
